/*
 * $RCSfile: EXIFGPSTagSet.java,v $
 *
 *
 * Copyright (c) 2005 Sun Microsystems, Inc. All  Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistribution of source code must retain the above copyright
 *   notice, this  list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
 * REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that this software is not designed or intended for
 * use in the design, construction, operation or maintenance of any
 * nuclear facility.
 *
 * $Revision: 1.1 $
 * $Date: 2005/10/28 16:56:45 $
 * $State: Exp $
 */
package com.sun.media.imageio.plugins.tiff;

import java.util.ArrayList;
import java.util.List;

/**
 * A class representing the tags found in an EXIF GPS Info IFD.
 *
 * <p>
 * The definitions of the data types referenced by the field
 * definitions may be found in the {@link TIFFTag
 * <code>TIFFTag</code>} class.
 *
 * @see EXIFTIFFTagSet
 */
public final class EXIFGPSTagSet extends TIFFTagSet {

   private static EXIFGPSTagSet _theInstance;

   /**
    * A tag indicating the GPS tag version (type BYTE, count = 4).
    */
   private static final int TAG_GPS_VERSION_ID = 0;

   /**
    * A tag indicating the North or South latitude (type ASCII, count = 2).
    */
   private static final int TAG_GPS_LATITUDE_REF = 1;

   /**
    * A tag indicating the Latitude (type RATIONAL, count = 3).
    */
   private static final int TAG_GPS_LATITUDE = 2;

   /**
    * A tag indicating the East or West Longitude (type ASCII, count = 2).
    */
   private static final int TAG_GPS_LONGITUDE_REF = 3;

   /**
    * A tag indicating the Longitude (type RATIONAL, count = 3).
    */
   private static final int TAG_GPS_LONGITUDE = 4;

   /**
    * A tag indicating the Altitude reference (type BYTE, count = 1).
    *
    * @see #ALTITUDE_REF_SEA_LEVEL
    * @see #ALTITUDE_REF_SEA_LEVEL_REFERENCE
    */
   private static final int TAG_GPS_ALTITUDE_REF = 5;

   /**
    * A tag indicating the Altitude (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_ALTITUDE = 6;

   /**
    * A tag indicating the GPS time (atomic clock) (type RATIONAL, count = 3).
    */
   private static final int TAG_GPS_TIME_STAMP = 7;

   /**
    * A tag indicating the GPS satellites used for measurement (type ASCII).
    */
   private static final int TAG_GPS_SATELLITES = 8;

   /**
    * A tag indicating the GPS receiver status (type ASCII, count = 2).
    */
   private static final int TAG_GPS_STATUS = 9;

   /**
    * A tag indicating the GPS measurement mode (type ASCII, count = 2).
    */
   private static final int TAG_GPS_MEASURE_MODE = 10;

   /**
    * A tag indicating the Measurement precision (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_DOP = 11;

   /**
    * A tag indicating the Speed unit (type ASCII, count = 2).
    */
   private static final int TAG_GPS_SPEED_REF = 12;

   /**
    * A tag indicating the Speed of GPS receiver (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_SPEED = 13;

   /**
    * A tag indicating the Reference for direction of movement (type ASCII,
    * count = 2).
    */
   private static final int TAG_GPS_TRACK_REF = 14;

   /**
    * A tag indicating the Direction of movement (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_TRACK = 15;

   /**
    * A tag indicating the Reference for direction of image (type ASCII,
    * count = 2).
    */
   private static final int TAG_GPS_IMG_DIRECTION_REF = 16;

   /**
    * A tag indicating the Direction of image (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_IMG_DIRECTION = 17;

   /**
    * A tag indicating the Geodetic survey data used (type ASCII).
    */
   private static final int TAG_GPS_MAP_DATUM = 18;

   /**
    * A tag indicating the Reference for latitude of destination (type
    * ASCII, count = 2).
    */
   private static final int TAG_GPS_DEST_LATITUDE_REF = 19;

   /**
    * A tag indicating the Latitude of destination (type RATIONAL, count = 3).
    */
   private static final int TAG_GPS_DEST_LATITUDE = 20;

   /**
    * A tag indicating the Reference for longitude of destination (type
    * ASCII, count = 2).
    */
   private static final int TAG_GPS_DEST_LONGITUDE_REF = 21;

   /**
    * A tag indicating the Longitude of destination (type RATIONAL,
    * count = 3).
    */
   private static final int TAG_GPS_DEST_LONGITUDE = 22;

   /**
    * A tag indicating the Reference for bearing of destination (type ASCII,
    * count = 2).
    */
   private static final int TAG_GPS_DEST_BEARING_REF = 23;

   /**
    * A tag indicating the Bearing of destination (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_DEST_BEARING = 24;

   /**
    * A tag indicating the Reference for distance to destination (type ASCII,
    * count = 2).
    */
   private static final int TAG_GPS_DEST_DISTANCE_REF = 25;

   /**
    * A tag indicating the Distance to destination (type RATIONAL, count = 1).
    */
   private static final int TAG_GPS_DEST_DISTANCE = 26;

   /**
    * A tag indicating the Name of GPS processing method (type UNDEFINED).
    */
   private static final int TAG_GPS_PROCESSING_METHOD = 27;

   /**
    * A tag indicating the Name of GPS area (type UNDEFINED).
    */
   private static final int TAG_GPS_AREA_INFORMATION = 28;

   /**
    * A tag indicating the GPS date (type ASCII, count 11).
    */
   private static final int TAG_GPS_DATE_STAMP = 29;

   /**
    * A tag indicating the GPS differential correction (type SHORT,
    * count = 1).
    *
    * @see #DIFFERENTIAL_CORRECTION_NONE
    * @see #DIFFERENTIAL_CORRECTION_APPLIED
    */
   private static final int TAG_GPS_DIFFERENTIAL = 30;

   /**
    * A value to be used with the "GPSAltitudeRef" tag.
    *
    * @see #TAG_GPS_ALTITUDE_REF
    */
   private static final int ALTITUDE_REF_SEA_LEVEL = 0;

   /**
    * A value to be used with the "GPSAltitudeRef" tag.
    *
    * @see #TAG_GPS_ALTITUDE_REF
    */
   private static final int ALTITUDE_REF_SEA_LEVEL_REFERENCE = 1;

   /**
    * A value to be used with the "GPSDifferential" tag.
    *
    * @see #TAG_GPS_DIFFERENTIAL
    */
   private static final int DIFFERENTIAL_CORRECTION_NONE = 0;

   /**
    * A value to be used with the "GPSDifferential" tag.
    *
    * @see #TAG_GPS_DIFFERENTIAL
    */
   private static final int DIFFERENTIAL_CORRECTION_APPLIED = 1;

   static class GPSVersionID extends TIFFTag {

      GPSVersionID() {
         super("GPSVersionID",
             TAG_GPS_VERSION_ID,
             1 << TIFFTag.TIFF_BYTE);
      }
   }

   static class GPSLatitudeRef extends TIFFTag {

      GPSLatitudeRef() {
         super("GPSLatitudeRef",
             TAG_GPS_LATITUDE_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSLatitude extends TIFFTag {

      GPSLatitude() {
         super("GPSLatitude",
             TAG_GPS_LATITUDE,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSLongitudeRef extends TIFFTag {

      GPSLongitudeRef() {
         super("GPSLongitudeRef",
             TAG_GPS_LONGITUDE_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSLongitude extends TIFFTag {

      GPSLongitude() {
         super("GPSLongitude",
             TAG_GPS_LONGITUDE,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSAltitudeRef extends TIFFTag {

      GPSAltitudeRef() {
         super("GPSAltitudeRef",
             TAG_GPS_ALTITUDE_REF,
             1 << TIFFTag.TIFF_BYTE);

         addValueName(ALTITUDE_REF_SEA_LEVEL, "Sea level");
         addValueName(ALTITUDE_REF_SEA_LEVEL_REFERENCE,
             "Sea level reference (negative value)");
      }
   }

   static class GPSAltitude extends TIFFTag {

      GPSAltitude() {
         super("GPSAltitude",
             TAG_GPS_ALTITUDE,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSTimeStamp extends TIFFTag {

      GPSTimeStamp() {
         super("GPSTimeStamp",
             TAG_GPS_TIME_STAMP,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSSatellites extends TIFFTag {

      GPSSatellites() {
         super("GPSSatellites",
             TAG_GPS_SATELLITES,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSStatus extends TIFFTag {

      GPSStatus() {
         super("GPSStatus",
             TAG_GPS_STATUS,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSMeasureMode extends TIFFTag {

      GPSMeasureMode() {
         super("GPSMeasureMode",
             TAG_GPS_MEASURE_MODE,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDOP extends TIFFTag {

      GPSDOP() {
         super("GPSDOP",
             TAG_GPS_DOP,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSSpeedRef extends TIFFTag {

      GPSSpeedRef() {
         super("GPSSpeedRef",
             TAG_GPS_SPEED_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSSpeed extends TIFFTag {

      GPSSpeed() {
         super("GPSSpeed",
             TAG_GPS_SPEED,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSTrackRef extends TIFFTag {

      GPSTrackRef() {
         super("GPSTrackRef",
             TAG_GPS_TRACK_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSTrack extends TIFFTag {

      GPSTrack() {
         super("GPSTrack",
             TAG_GPS_TRACK,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSImgDirectionRef extends TIFFTag {

      GPSImgDirectionRef() {
         super("GPSImgDirectionRef",
             TAG_GPS_IMG_DIRECTION_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSImgDirection extends TIFFTag {

      GPSImgDirection() {
         super("GPSImgDirection",
             TAG_GPS_IMG_DIRECTION,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSMapDatum extends TIFFTag {

      GPSMapDatum() {
         super("GPSMapDatum",
             TAG_GPS_MAP_DATUM,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDestLatitudeRef extends TIFFTag {

      GPSDestLatitudeRef() {
         super("GPSDestLatitudeRef",
             TAG_GPS_DEST_LATITUDE_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDestLatitude extends TIFFTag {

      GPSDestLatitude() {
         super("GPSDestLatitude",
             TAG_GPS_DEST_LATITUDE,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSDestLongitudeRef extends TIFFTag {

      GPSDestLongitudeRef() {
         super("GPSDestLongitudeRef",
             TAG_GPS_DEST_LONGITUDE_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDestLongitude extends TIFFTag {

      GPSDestLongitude() {
         super("GPSDestLongitude",
             TAG_GPS_DEST_LONGITUDE,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSDestBearingRef extends TIFFTag {

      GPSDestBearingRef() {
         super("GPSDestBearingRef",
             TAG_GPS_DEST_BEARING_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDestBearing extends TIFFTag {

      GPSDestBearing() {
         super("GPSDestBearing",
             TAG_GPS_DEST_BEARING,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSDestDistanceRef extends TIFFTag {

      GPSDestDistanceRef() {
         super("GPSDestDistanceRef",
             TAG_GPS_DEST_DISTANCE_REF,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDestDistance extends TIFFTag {

      GPSDestDistance() {
         super("GPSDestDistance",
             TAG_GPS_DEST_DISTANCE,
             1 << TIFFTag.TIFF_RATIONAL);
      }
   }

   static class GPSProcessingMethod extends TIFFTag {

      GPSProcessingMethod() {
         super("GPSProcessingMethod",
             TAG_GPS_PROCESSING_METHOD,
             1 << TIFFTag.TIFF_UNDEFINED);
      }
   }

   static class GPSAreaInformation extends TIFFTag {

      GPSAreaInformation() {
         super("GPSAreaInformation",
             TAG_GPS_AREA_INFORMATION,
             1 << TIFFTag.TIFF_UNDEFINED);
      }
   }

   static class GPSDateStamp extends TIFFTag {

      GPSDateStamp() {
         super("GPSDateStamp",
             TAG_GPS_DATE_STAMP,
             1 << TIFFTag.TIFF_ASCII);
      }
   }

   static class GPSDifferential extends TIFFTag {

      GPSDifferential() {
         super("GPSDifferential",
             TAG_GPS_DIFFERENTIAL,
             1 << TIFFTag.TIFF_SHORT);
         addValueName(DIFFERENTIAL_CORRECTION_NONE,
             "Measurement without differential correction");
         addValueName(DIFFERENTIAL_CORRECTION_APPLIED,
             "Differential correction applied");

      }
   }

   private static List<TIFFTag> initTags() {
      ArrayList<TIFFTag> tags = new ArrayList<>(31);

      tags.add(new GPSVersionID());
      tags.add(new GPSLatitudeRef());
      tags.add(new GPSLatitude());
      tags.add(new GPSLongitudeRef());
      tags.add(new GPSLongitude());
      tags.add(new GPSAltitudeRef());
      tags.add(new GPSAltitude());
      tags.add(new GPSTimeStamp());
      tags.add(new GPSSatellites());
      tags.add(new GPSStatus());
      tags.add(new GPSMeasureMode());
      tags.add(new GPSDOP());
      tags.add(new GPSSpeedRef());
      tags.add(new GPSSpeed());
      tags.add(new GPSTrackRef());
      tags.add(new GPSTrack());
      tags.add(new GPSImgDirectionRef());
      tags.add(new GPSImgDirection());
      tags.add(new GPSMapDatum());
      tags.add(new GPSDestLatitudeRef());
      tags.add(new GPSDestLatitude());
      tags.add(new GPSDestLongitudeRef());
      tags.add(new GPSDestLongitude());
      tags.add(new GPSDestBearingRef());
      tags.add(new GPSDestBearing());
      tags.add(new GPSDestDistanceRef());
      tags.add(new GPSDestDistance());
      tags.add(new GPSProcessingMethod());
      tags.add(new GPSAreaInformation());
      tags.add(new GPSDateStamp());
      tags.add(new GPSDifferential());
      return tags;
   }

   private EXIFGPSTagSet() {
      super(initTags());
   }

   /**
    * Returns a shared instance of an <code>EXIFGPSTagSet</code>.
    *
    * @return an <code>EXIFGPSTagSet</code> instance.
    */
   public static synchronized EXIFGPSTagSet getInstance() {
      if (_theInstance == null) {
         _theInstance = new EXIFGPSTagSet();
      }
      return _theInstance;
   }
}
