/*
 * $RCSfile: EXIFTIFFTagSet.java,v $
 *
 *
 * Copyright (c) 2005 Sun Microsystems, Inc. All  Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistribution of source code must retain the above copyright
 *   notice, this  list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
 * REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that this software is not designed or intended for
 * use in the design, construction, operation or maintenance of any
 * nuclear facility.
 *
 * $Revision: 1.4 $
 * $Date: 2005/10/28 16:56:45 $
 * $State: Exp $
 */
package com.sun.media.imageio.plugins.tiff;

import java.util.ArrayList;
import java.util.List;

/**
 * A class representing the tags found in an EXIF IFD.  EXIF is a
 * standard for annotating images used by most digital camera
 * manufacturers.  The EXIF specification may be found at
 * <a href="http://www.exif.org/Exif2-2.PDF">
 * <code>http://www.exif.org/Exif2-2.PDF</code>
 * </a>.
 * <p>
 * <p> The definitions of the data types referenced by the field
 * definitions may be found in the {@link TIFFTag
 * <code>TIFFTag</code>} class.
 */
public final class EXIFTIFFTagSet extends TIFFTagSet {

    @Deprecated
    public static final int TAG_GPS_INFO_IFD_POINTER = 34853;
    private static final int COMPONENTS_CONFIGURATION_DOES_NOT_EXIST = 0;
    private static final int COMPONENTS_CONFIGURATION_Y = 1;
    private static final int COMPONENTS_CONFIGURATION_CB = 2;
    private static final int COMPONENTS_CONFIGURATION_CR = 3;
    private static final int COMPONENTS_CONFIGURATION_R = 4;
    private static final int COMPONENTS_CONFIGURATION_G = 5;
    private static final int COMPONENTS_CONFIGURATION_B = 6;
    private static final int TAG_COMPRESSED_BITS_PER_PIXEL = 37122;
    private static final int TAG_PIXEL_X_DIMENSION = 40962;
    private static final int TAG_PIXEL_Y_DIMENSION = 40963;
    private static final int TAG_MAKER_NOTE = 37500;
    @Deprecated
    public static final int TAG_MARKER_NOTE = TAG_MAKER_NOTE;
    private static final int TAG_USER_COMMENT = 37510;
    private static final int TAG_RELATED_SOUND_FILE = 40964;
    private static final int TAG_DATE_TIME_ORIGINAL = 36867;
    private static final int TAG_DATE_TIME_DIGITIZED = 36868;
    private static final int TAG_SUB_SEC_TIME = 37520;
    private static final int TAG_SUB_SEC_TIME_ORIGINAL = 37521;
    private static final int TAG_SUB_SEC_TIME_DIGITIZED = 37522;
    private static final int TAG_EXPOSURE_TIME = 33434;
    private static final int TAG_F_NUMBER = 33437;
    private static final int TAG_EXPOSURE_PROGRAM = 34850;
    private static final int EXPOSURE_PROGRAM_NOT_DEFINED = 0;
    private static final int EXPOSURE_PROGRAM_MANUAL = 1;
    private static final int EXPOSURE_PROGRAM_NORMAL_PROGRAM = 2;
    private static final int EXPOSURE_PROGRAM_APERTURE_PRIORITY = 3;
    private static final int EXPOSURE_PROGRAM_SHUTTER_PRIORITY = 4;
    private static final int EXPOSURE_PROGRAM_CREATIVE_PROGRAM = 5;
    private static final int EXPOSURE_PROGRAM_ACTION_PROGRAM = 6;
    private static final int EXPOSURE_PROGRAM_PORTRAIT_MODE = 7;
    private static final int EXPOSURE_PROGRAM_LANDSCAPE_MODE = 8;
    private static final int TAG_SPECTRAL_SENSITIVITY = 34852;
    private static final int TAG_ISO_SPEED_RATINGS = 34855;
    private static final int TAG_OECF = 34856;
    private static final int TAG_SHUTTER_SPEED_VALUE = 37377;
    private static final int TAG_APERTURE_VALUE = 37378;
    private static final int TAG_BRIGHTNESS_VALUE = 37379;
    private static final int TAG_EXPOSURE_BIAS_VALUE = 37380;
    private static final int TAG_MAX_APERTURE_VALUE = 37381;
    private static final int TAG_SUBJECT_DISTANCE = 37382;
    private static final int TAG_METERING_MODE = 37383;
    private static final int METERING_MODE_UNKNOWN = 0;
    private static final int METERING_MODE_AVERAGE = 1;
    private static final int METERING_MODE_CENTER_WEIGHTED_AVERAGE = 2;
    private static final int METERING_MODE_SPOT = 3;
    private static final int METERING_MODE_MULTI_SPOT = 4;
    private static final int METERING_MODE_PATTERN = 5;
    private static final int METERING_MODE_PARTIAL = 6;
    private static final int METERING_MODE_OTHER = 255;
    private static final int TAG_LIGHT_SOURCE = 37384;
    private static final int LIGHT_SOURCE_UNKNOWN = 0;
    private static final int LIGHT_SOURCE_DAYLIGHT = 1;
    private static final int LIGHT_SOURCE_FLUORESCENT = 2;
    private static final int LIGHT_SOURCE_TUNGSTEN = 3;
    private static final int LIGHT_SOURCE_STANDARD_LIGHT_A = 17;
    private static final int LIGHT_SOURCE_STANDARD_LIGHT_B = 18;
    private static final int LIGHT_SOURCE_STANDARD_LIGHT_C = 19;
    private static final int LIGHT_SOURCE_D55 = 20;
    private static final int LIGHT_SOURCE_D65 = 21;
    private static final int LIGHT_SOURCE_D75 = 22;
    private static final int LIGHT_SOURCE_OTHER = 255;
    private static final int TAG_FLASH = 37385;
    private static final int FLASH_DID_NOT_FIRE = 0x0;
    private static final int FLASH_FIRED = 0x1;
    private static final int FLASH_STROBE_RETURN_LIGHT_NOT_DETECTED = 0x5;
    private static final int FLASH_STROBE_RETURN_LIGHT_DETECTED = 0x7;
    private static final int TAG_FOCAL_LENGTH = 37386;
    private static final int TAG_SUBJECT_AREA = 37396;
    private static final int TAG_FLASH_ENERGY = 41483;
    private static final int TAG_SPATIAL_FREQUENCY_RESPONSE = 41484;
    private static final int TAG_FOCAL_PLANE_X_RESOLUTION = 41486;
    private static final int TAG_FOCAL_PLANE_Y_RESOLUTION = 41487;
    private static final int TAG_FOCAL_PLANE_RESOLUTION_UNIT = 41488;
    private static final int FOCAL_PLANE_RESOLUTION_UNIT_NONE = 1;
    private static final int FOCAL_PLANE_RESOLUTION_UNIT_INCH = 2;
    private static final int FOCAL_PLANE_RESOLUTION_UNIT_CENTIMETER = 3;
    private static final int TAG_SUBJECT_LOCATION = 41492;
    private static final int TAG_EXPOSURE_INDEX = 41493;
    private static final int TAG_SENSING_METHOD = 41495;
    private static final int SENSING_METHOD_NOT_DEFINED = 1;
    private static final int SENSING_METHOD_ONE_CHIP_COLOR_AREA_SENSOR = 2;
    private static final int SENSING_METHOD_TWO_CHIP_COLOR_AREA_SENSOR = 3;
    private static final int SENSING_METHOD_THREE_CHIP_COLOR_AREA_SENSOR = 4;
    private static final int SENSING_METHOD_COLOR_SEQUENTIAL_AREA_SENSOR = 5;
    private static final int SENSING_METHOD_TRILINEAR_SENSOR = 7;
    private static final int SENSING_METHOD_COLOR_SEQUENTIAL_LINEAR_SENSOR = 8;
    private static final int TAG_FILE_SOURCE = 41728;
    private static final int FILE_SOURCE_DSC = 3;
    private static final int TAG_SCENE_TYPE = 41729;
    private static final int SCENE_TYPE_DSC = 1;
    private static final int TAG_CFA_PATTERN = 41730;
    private static final int TAG_CUSTOM_RENDERED = 41985;
    private static final int CUSTOM_RENDERED_NORMAL = 0;
    private static final int CUSTOM_RENDERED_CUSTOM = 1;
    private static final int TAG_EXPOSURE_MODE = 41986;
    private static final int EXPOSURE_MODE_AUTO_EXPOSURE = 0;
    private static final int EXPOSURE_MODE_MANUAL_EXPOSURE = 1;
    private static final int EXPOSURE_MODE_AUTO_BRACKET = 2;
    private static final int TAG_WHITE_BALANCE = 41987;
    private static final int WHITE_BALANCE_AUTO = 0;
    private static final int WHITE_BALANCE_MANUAL = 1;
    private static final int TAG_DIGITAL_ZOOM_RATIO = 41988;
    private static final int TAG_FOCAL_LENGTH_IN_35MM_FILM = 41989;
    private static final int TAG_SCENE_CAPTURE_TYPE = 41990;
    private static final int SCENE_CAPTURE_TYPE_STANDARD = 0;
    private static final int SCENE_CAPTURE_TYPE_LANDSCAPE = 1;
    private static final int SCENE_CAPTURE_TYPE_PORTRAIT = 2;
    private static final int SCENE_CAPTURE_TYPE_NIGHT_SCENE = 3;
    private static final int TAG_GAIN_CONTROL = 41991;
    private static final int GAIN_CONTROL_NONE = 0;
    private static final int GAIN_CONTROL_LOW_GAIN_UP = 1;
    private static final int GAIN_CONTROL_HIGH_GAIN_UP = 2;
    private static final int GAIN_CONTROL_LOW_GAIN_DOWN = 3;
    private static final int GAIN_CONTROL_HIGH_GAIN_DOWN = 4;
    private static final int TAG_CONTRAST = 41992;
    private static final int CONTRAST_NORMAL = 0;
    private static final int CONTRAST_SOFT = 1;
    private static final int CONTRAST_HARD = 2;
    private static final int TAG_SATURATION = 41993;
    private static final int SATURATION_NORMAL = 0;
    private static final int SATURATION_LOW = 1;
    private static final int SATURATION_HIGH = 2;
    private static final int TAG_SHARPNESS = 41994;
    private static final int SHARPNESS_NORMAL = 0;
    private static final int SHARPNESS_SOFT = 1;
    private static final int SHARPNESS_HARD = 2;
    private static final int TAG_DEVICE_SETTING_DESCRIPTION = 41995;
    private static final int TAG_SUBJECT_DISTANCE_RANGE = 41996;
    private static final int SUBJECT_DISTANCE_RANGE_UNKNOWN = 0;
    private static final int SUBJECT_DISTANCE_RANGE_MACRO = 1;
    private static final int SUBJECT_DISTANCE_RANGE_CLOSE_VIEW = 2;
    private static final int SUBJECT_DISTANCE_RANGE_DISTANT_VIEW = 3;
    private static final int TAG_IMAGE_UNIQUE_ID = 42016;
    /**
     * A tag pointing to an interoperability IFD (type LONG).
     */
    private static final int TAG_INTEROPERABILITY_IFD_POINTER = 40965;
    /**
     * A tag containing the EXIF version number (type UNDEFINED, count =
     * 4).  Conformance to the EXIF 2.1 standard is indicated using
     * the ASCII value "0210" (with no terminating NUL).
     */
    private static final int TAG_EXIF_VERSION = 36864;
    private static final int TAG_FLASHPIX_VERSION = 40960;
    private static final int TAG_COLOR_SPACE = 40961;
    private static final int COLOR_SPACE_SRGB = 1;
    private static final int COLOR_SPACE_UNCALIBRATED = 0xFFFF;
    private static final int TAG_COMPONENTS_CONFIGURATION = 37121;
    private static EXIFTIFFTagSet _theInstance;
    private static List<TIFFTag> _tags;

    private EXIFTIFFTagSet() {
        super(_tags);
    }

    private static void initTags() {
        _tags = new ArrayList<>(42);

        _tags.add(new EXIFTIFFTagSet.EXIFVersion());
        _tags.add(new EXIFTIFFTagSet.FlashPixVersion());
        _tags.add(new EXIFTIFFTagSet.ColorSpace());
        _tags.add(new EXIFTIFFTagSet.ComponentsConfiguration());
        _tags.add(new EXIFTIFFTagSet.CompressedBitsPerPixel());
        _tags.add(new EXIFTIFFTagSet.PixelXDimension());
        _tags.add(new EXIFTIFFTagSet.PixelYDimension());
        _tags.add(new EXIFTIFFTagSet.MakerNote());
        _tags.add(new EXIFTIFFTagSet.UserComment());
        _tags.add(new EXIFTIFFTagSet.RelatedSoundFile());
        _tags.add(new EXIFTIFFTagSet.DateTimeOriginal());
        _tags.add(new EXIFTIFFTagSet.DateTimeDigitized());
        _tags.add(new EXIFTIFFTagSet.SubSecTime());
        _tags.add(new EXIFTIFFTagSet.SubSecTimeOriginal());
        _tags.add(new EXIFTIFFTagSet.SubSecTimeDigitized());
        _tags.add(new EXIFTIFFTagSet.ExposureTime());
        _tags.add(new EXIFTIFFTagSet.FNumber());
        _tags.add(new EXIFTIFFTagSet.ExposureProgram());
        _tags.add(new EXIFTIFFTagSet.SpectralSensitivity());
        _tags.add(new EXIFTIFFTagSet.ISOSpeedRatings());
        _tags.add(new EXIFTIFFTagSet.OECF());
        _tags.add(new EXIFTIFFTagSet.ShutterSpeedValue());
        _tags.add(new EXIFTIFFTagSet.ApertureValue());
        _tags.add(new EXIFTIFFTagSet.BrightnessValue());
        _tags.add(new EXIFTIFFTagSet.ExposureBiasValue());
        _tags.add(new EXIFTIFFTagSet.MaxApertureValue());
        _tags.add(new EXIFTIFFTagSet.SubjectDistance());
        _tags.add(new EXIFTIFFTagSet.MeteringMode());
        _tags.add(new EXIFTIFFTagSet.LightSource());
        _tags.add(new EXIFTIFFTagSet.Flash());
        _tags.add(new EXIFTIFFTagSet.FocalLength());
        _tags.add(new EXIFTIFFTagSet.SubjectArea());
        _tags.add(new EXIFTIFFTagSet.FlashEnergy());
        _tags.add(new EXIFTIFFTagSet.SpatialFrequencyResponse());
        _tags.add(new EXIFTIFFTagSet.FocalPlaneXResolution());
        _tags.add(new EXIFTIFFTagSet.FocalPlaneYResolution());
        _tags.add(new EXIFTIFFTagSet.FocalPlaneResolutionUnit());
        _tags.add(new EXIFTIFFTagSet.SubjectLocation());
        _tags.add(new EXIFTIFFTagSet.ExposureIndex());
        _tags.add(new EXIFTIFFTagSet.SensingMethod());
        _tags.add(new EXIFTIFFTagSet.FileSource());
        _tags.add(new EXIFTIFFTagSet.SceneType());
        _tags.add(new EXIFTIFFTagSet.CFAPattern());
        _tags.add(new EXIFTIFFTagSet.CustomRendered());
        _tags.add(new EXIFTIFFTagSet.ExposureMode());
        _tags.add(new EXIFTIFFTagSet.WhiteBalance());
        _tags.add(new EXIFTIFFTagSet.DigitalZoomRatio());
        _tags.add(new EXIFTIFFTagSet.FocalLengthIn35mmFilm());
        _tags.add(new EXIFTIFFTagSet.SceneCaptureType());
        _tags.add(new EXIFTIFFTagSet.GainControl());
        _tags.add(new EXIFTIFFTagSet.Contrast());
        _tags.add(new EXIFTIFFTagSet.Saturation());
        _tags.add(new EXIFTIFFTagSet.Sharpness());
        _tags.add(new EXIFTIFFTagSet.DeviceSettingDescription());
        _tags.add(new EXIFTIFFTagSet.SubjectDistanceRange());
        _tags.add(new EXIFTIFFTagSet.ImageUniqueID());
        _tags.add(new EXIFTIFFTagSet.InteroperabilityIFD());
    }

    public static synchronized EXIFTIFFTagSet getInstance() {
        if (_theInstance == null) {
            initTags();
            _theInstance = new EXIFTIFFTagSet();
            _tags = null;
        }
        return _theInstance;
    }

    static class EXIFVersion extends TIFFTag {
        EXIFVersion() {
            super("EXIFversion",
                    TAG_EXIF_VERSION,
                    1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class FlashPixVersion extends TIFFTag {
        FlashPixVersion() {
            super("FlashPixVersion",
                    TAG_FLASHPIX_VERSION,
                    1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class ColorSpace extends TIFFTag {
        ColorSpace() {
            super("ColorSpace", TAG_COLOR_SPACE, 1 << TIFFTag.TIFF_SHORT);
            addValueName(COLOR_SPACE_SRGB, "sRGB");
            addValueName(COLOR_SPACE_UNCALIBRATED, "Uncalibrated");
        }
    }

    static class ComponentsConfiguration extends TIFFTag {
        ComponentsConfiguration() {
            super("ComponentsConfiguration", TAG_COMPONENTS_CONFIGURATION, 1 << TIFFTag.TIFF_UNDEFINED);
            addValueName(COMPONENTS_CONFIGURATION_DOES_NOT_EXIST, "DoesNotExist");
            addValueName(COMPONENTS_CONFIGURATION_Y, "Y");
            addValueName(COMPONENTS_CONFIGURATION_CB, "Cb");
            addValueName(COMPONENTS_CONFIGURATION_CR, "Cr");
            addValueName(COMPONENTS_CONFIGURATION_R, "R");
            addValueName(COMPONENTS_CONFIGURATION_G, "G");
            addValueName(COMPONENTS_CONFIGURATION_B, "B");
        }
    }

    static class CompressedBitsPerPixel extends TIFFTag {
        CompressedBitsPerPixel() {
            super("CompressedBitsPerPixel",
                    TAG_COMPRESSED_BITS_PER_PIXEL,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class PixelXDimension extends TIFFTag {
        PixelXDimension() {
            super("PixelXDimension",
                    TAG_PIXEL_X_DIMENSION,
                    (1 << TIFFTag.TIFF_SHORT)
                            | (1 << TIFFTag.TIFF_LONG));
        }
    }

    static class PixelYDimension extends TIFFTag {
        PixelYDimension() {
            super("PixelYDimension",
                    TAG_PIXEL_Y_DIMENSION,
                    (1 << TIFFTag.TIFF_SHORT)
                            | (1 << TIFFTag.TIFF_LONG));
        }
    }

    static class MakerNote extends TIFFTag {
        MakerNote() {
            super("MakerNote",
                    TAG_MAKER_NOTE,
                    1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class UserComment extends TIFFTag {
        UserComment() {
            super("UserComment",
                    TAG_USER_COMMENT,
                    1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class RelatedSoundFile extends TIFFTag {
        RelatedSoundFile() {
            super("RelatedSoundFile",
                    TAG_RELATED_SOUND_FILE,
                    1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class DateTimeOriginal extends TIFFTag {
        DateTimeOriginal() {
            super("DateTimeOriginal",
                    TAG_DATE_TIME_ORIGINAL,
                    1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class DateTimeDigitized extends TIFFTag {
        DateTimeDigitized() {
            super("DateTimeDigitized", TAG_DATE_TIME_DIGITIZED, 1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class SubSecTime extends TIFFTag {
        SubSecTime() {
            super("SubSecTime", TAG_SUB_SEC_TIME, 1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class SubSecTimeOriginal extends TIFFTag {
        SubSecTimeOriginal() {
            super("SubSecTimeOriginal", TAG_SUB_SEC_TIME_ORIGINAL, 1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class SubSecTimeDigitized extends TIFFTag {
        SubSecTimeDigitized() {
            super("SubSecTimeDigitized", TAG_SUB_SEC_TIME_DIGITIZED, 1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class ExposureTime extends TIFFTag {
        ExposureTime() {
            super("ExposureTime", TAG_EXPOSURE_TIME, 1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class FNumber extends TIFFTag {
        FNumber() {
            super("FNumber", TAG_F_NUMBER, 1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class ExposureProgram extends TIFFTag {
        ExposureProgram() {
            super("ExposureProgram", TAG_EXPOSURE_PROGRAM, 1 << TIFFTag.TIFF_SHORT);
            addValueName(EXPOSURE_PROGRAM_NOT_DEFINED, "Not Defined");
            addValueName(EXPOSURE_PROGRAM_MANUAL, "Manual");
            addValueName(EXPOSURE_PROGRAM_NORMAL_PROGRAM, "Normal Program");
            addValueName(EXPOSURE_PROGRAM_APERTURE_PRIORITY, "Aperture Priority");
            addValueName(EXPOSURE_PROGRAM_SHUTTER_PRIORITY, "Shutter Priority");
            addValueName(EXPOSURE_PROGRAM_CREATIVE_PROGRAM, "Creative Program");
            addValueName(EXPOSURE_PROGRAM_ACTION_PROGRAM, "Action Program");
            addValueName(EXPOSURE_PROGRAM_PORTRAIT_MODE, "Portrait Mode");
            addValueName(EXPOSURE_PROGRAM_LANDSCAPE_MODE, "Landscape Mode");
        }
    }

    static class SpectralSensitivity extends TIFFTag {
        SpectralSensitivity() {
            super("SpectralSensitivity", TAG_SPECTRAL_SENSITIVITY, 1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class ISOSpeedRatings extends TIFFTag {
        ISOSpeedRatings() {
            super("ISOSpeedRatings", TAG_ISO_SPEED_RATINGS, 1 << TIFFTag.TIFF_SHORT);
        }
    }

    static class OECF extends TIFFTag {
        OECF() {
            super("OECF", TAG_OECF, 1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class ShutterSpeedValue extends TIFFTag {

        ShutterSpeedValue() {
            super("ShutterSpeedValue", TAG_SHUTTER_SPEED_VALUE, 1 << TIFFTag.TIFF_SRATIONAL);
        }
    }

    static class ApertureValue extends TIFFTag {
        ApertureValue() {
            super("ApertureValue", TAG_APERTURE_VALUE, 1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class BrightnessValue extends TIFFTag {
        BrightnessValue() {
            super("BrightnessValue", TAG_BRIGHTNESS_VALUE, 1 << TIFFTag.TIFF_SRATIONAL);
        }
    }

    static class ExposureBiasValue extends TIFFTag {
        ExposureBiasValue() {
            super("ExposureBiasValue", TAG_EXPOSURE_BIAS_VALUE, 1 << TIFFTag.TIFF_SRATIONAL);
        }
    }

    static class MaxApertureValue extends TIFFTag {
        MaxApertureValue() {
            super("MaxApertureValue", TAG_MAX_APERTURE_VALUE, 1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class SubjectDistance extends TIFFTag {
        SubjectDistance() {
            super("SubjectDistance", TAG_SUBJECT_DISTANCE, 1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class MeteringMode extends TIFFTag {
        MeteringMode() {
            super("MeteringMode", TAG_METERING_MODE, 1 << TIFFTag.TIFF_SHORT);
            addValueName(METERING_MODE_UNKNOWN, "Unknown");
            addValueName(METERING_MODE_AVERAGE, "Average");
            addValueName(METERING_MODE_CENTER_WEIGHTED_AVERAGE, "CenterWeightedAverage");
            addValueName(METERING_MODE_SPOT, "Spot");
            addValueName(METERING_MODE_MULTI_SPOT, "MultiSpot");
            addValueName(METERING_MODE_PATTERN, "Pattern");
            addValueName(METERING_MODE_PARTIAL, "Partial");
            addValueName(METERING_MODE_OTHER, "Other");
        }
    }

    static class LightSource extends TIFFTag {
        LightSource() {
            super("LightSource", TAG_LIGHT_SOURCE, 1 << TIFFTag.TIFF_SHORT);
            addValueName(LIGHT_SOURCE_UNKNOWN, "Unknown");
            addValueName(LIGHT_SOURCE_DAYLIGHT, "Daylight");
            addValueName(LIGHT_SOURCE_FLUORESCENT, "Fluorescent");
            addValueName(LIGHT_SOURCE_TUNGSTEN, "Tungsten");
            addValueName(LIGHT_SOURCE_STANDARD_LIGHT_A, "Standard Light A");
            addValueName(LIGHT_SOURCE_STANDARD_LIGHT_B, "Standard Light B");
            addValueName(LIGHT_SOURCE_STANDARD_LIGHT_C, "Standard Light C");
            addValueName(LIGHT_SOURCE_D55, "D55");
            addValueName(LIGHT_SOURCE_D65, "D65");
            addValueName(LIGHT_SOURCE_D75, "D75");
            addValueName(LIGHT_SOURCE_OTHER, "Other");
        }
    }

    static class Flash extends TIFFTag {
        Flash() {
            super("Flash", TAG_FLASH, 1 << TIFFTag.TIFF_SHORT);
            addValueName(FLASH_DID_NOT_FIRE, "Flash Did Not Fire");
            addValueName(FLASH_FIRED, "Flash Fired");
            addValueName(FLASH_STROBE_RETURN_LIGHT_NOT_DETECTED,
                    "Strobe Return Light Not Detected");
            addValueName(FLASH_STROBE_RETURN_LIGHT_DETECTED,
                    "Strobe Return Light Detected");
        }
    }

    static class FocalLength extends TIFFTag {
        FocalLength() {
            super("FocalLength",
                    TAG_FOCAL_LENGTH,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class SubjectArea extends TIFFTag {
        SubjectArea() {
            super("SubjectArea",
                    TAG_SUBJECT_AREA,
                    1 << TIFFTag.TIFF_SHORT);
        }
    }

    static class FlashEnergy extends TIFFTag {
        FlashEnergy() {
            super("FlashEnergy",
                    TAG_FLASH_ENERGY,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class SpatialFrequencyResponse extends TIFFTag {
        SpatialFrequencyResponse() {
            super("SpatialFrequencyResponse",
                    TAG_SPATIAL_FREQUENCY_RESPONSE,
                    1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class FocalPlaneXResolution extends TIFFTag {
        FocalPlaneXResolution() {
            super("FocalPlaneXResolution",
                    TAG_FOCAL_PLANE_X_RESOLUTION,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class FocalPlaneYResolution extends TIFFTag {
        FocalPlaneYResolution() {
            super("FocalPlaneYResolution",
                    TAG_FOCAL_PLANE_Y_RESOLUTION,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class FocalPlaneResolutionUnit extends TIFFTag {
        FocalPlaneResolutionUnit() {
            super("FocalPlaneResolutionUnit",
                    TAG_FOCAL_PLANE_RESOLUTION_UNIT,
                    1 << TIFFTag.TIFF_SHORT);

            addValueName(FOCAL_PLANE_RESOLUTION_UNIT_NONE, "None");
            addValueName(FOCAL_PLANE_RESOLUTION_UNIT_INCH, "Inch");
            addValueName(FOCAL_PLANE_RESOLUTION_UNIT_CENTIMETER, "Centimeter");
        }
    }

    static class SubjectLocation extends TIFFTag {
        SubjectLocation() {
            super("SubjectLocation",
                    TAG_SUBJECT_LOCATION,
                    1 << TIFFTag.TIFF_SHORT);
        }
    }

    static class ExposureIndex extends TIFFTag {
        ExposureIndex() {
            super("ExposureIndex",
                    TAG_EXPOSURE_INDEX,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class SensingMethod extends TIFFTag {
        SensingMethod() {
            super("SensingMethod",
                    TAG_SENSING_METHOD,
                    1 << TIFFTag.TIFF_SHORT);

            addValueName(SENSING_METHOD_NOT_DEFINED, "Not Defined");
            addValueName(SENSING_METHOD_ONE_CHIP_COLOR_AREA_SENSOR,
                    "One-chip color area sensor");
            addValueName(SENSING_METHOD_TWO_CHIP_COLOR_AREA_SENSOR,
                    "Two-chip color area sensor");
            addValueName(SENSING_METHOD_THREE_CHIP_COLOR_AREA_SENSOR,
                    "Three-chip color area sensor");
            addValueName(SENSING_METHOD_COLOR_SEQUENTIAL_AREA_SENSOR,
                    "Color sequential area sensor");
            addValueName(SENSING_METHOD_TRILINEAR_SENSOR, "Trilinear sensor");
            addValueName(SENSING_METHOD_COLOR_SEQUENTIAL_LINEAR_SENSOR,
                    "Color sequential linear sensor");
        }
    }

    static class FileSource extends TIFFTag {
        FileSource() {
            super("FileSource",
                    TAG_FILE_SOURCE,
                    1 << TIFFTag.TIFF_UNDEFINED);

            addValueName(FILE_SOURCE_DSC, "DSC");
        }
    }

    static class SceneType extends TIFFTag {
        SceneType() {
            super("SceneType",
                    TAG_SCENE_TYPE,
                    1 << TIFFTag.TIFF_UNDEFINED);

            addValueName(SCENE_TYPE_DSC, "A directly photographed image");
        }
    }

    static class CFAPattern extends TIFFTag {
        CFAPattern() {
            super("CFAPattern",
                    TAG_CFA_PATTERN,
                    1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class CustomRendered extends TIFFTag {
        CustomRendered() {
            super("CustomRendered",
                    TAG_CUSTOM_RENDERED,
                    1 << TIFFTag.TIFF_SHORT);

            addValueName(CUSTOM_RENDERED_NORMAL, "Normal process");
            addValueName(CUSTOM_RENDERED_CUSTOM, "Custom process");
        }
    }

    static class ExposureMode extends TIFFTag {
        ExposureMode() {
            super("ExposureMode",
                    TAG_EXPOSURE_MODE,
                    1 << TIFFTag.TIFF_SHORT);

            addValueName(EXPOSURE_MODE_AUTO_EXPOSURE, "Auto exposure");
            addValueName(EXPOSURE_MODE_MANUAL_EXPOSURE, "Manual exposure");
            addValueName(EXPOSURE_MODE_AUTO_BRACKET, "Auto bracket");
        }
    }

    static class WhiteBalance extends TIFFTag {
        WhiteBalance() {
            super("WhiteBalance",
                    TAG_WHITE_BALANCE,
                    1 << TIFFTag.TIFF_SHORT);

            addValueName(WHITE_BALANCE_AUTO, "Auto white balance");
            addValueName(WHITE_BALANCE_MANUAL, "Manual white balance");
        }
    }

    static class DigitalZoomRatio extends TIFFTag {
        DigitalZoomRatio() {
            super("DigitalZoomRatio",
                    TAG_DIGITAL_ZOOM_RATIO,
                    1 << TIFFTag.TIFF_RATIONAL);
        }
    }

    static class FocalLengthIn35mmFilm extends TIFFTag {
        FocalLengthIn35mmFilm() {
            super("FocalLengthIn35mmFilm", TAG_FOCAL_LENGTH_IN_35MM_FILM, 1 << TIFFTag.TIFF_SHORT);
        }
    }

    static class SceneCaptureType extends TIFFTag {
        SceneCaptureType() {
            super("SceneCaptureType", TAG_SCENE_CAPTURE_TYPE, 1 << TIFFTag.TIFF_SHORT);
            addValueName(SCENE_CAPTURE_TYPE_STANDARD, "Standard");
            addValueName(SCENE_CAPTURE_TYPE_LANDSCAPE, "Landscape");
            addValueName(SCENE_CAPTURE_TYPE_PORTRAIT, "Portrait");
            addValueName(SCENE_CAPTURE_TYPE_NIGHT_SCENE, "Night scene");
        }
    }

    static class GainControl extends TIFFTag {
        GainControl() {
            super("GainControl", TAG_GAIN_CONTROL, 1 << TIFFTag.TIFF_SHORT);
            addValueName(GAIN_CONTROL_NONE, "None");
            addValueName(GAIN_CONTROL_LOW_GAIN_UP, "Low gain up");
            addValueName(GAIN_CONTROL_HIGH_GAIN_UP, "High gain up");
            addValueName(GAIN_CONTROL_LOW_GAIN_DOWN, "Low gain down");
            addValueName(GAIN_CONTROL_HIGH_GAIN_DOWN, "High gain down");
        }
    }

    static class Contrast extends TIFFTag {
        Contrast() {
            super("Contrast", TAG_CONTRAST, 1 << TIFFTag.TIFF_SHORT);
            addValueName(CONTRAST_NORMAL, "Normal");
            addValueName(CONTRAST_SOFT, "Soft");
            addValueName(CONTRAST_HARD, "Hard");
        }
    }

    static class Saturation extends TIFFTag {
        Saturation() {
            super("Saturation", TAG_SATURATION, 1 << TIFFTag.TIFF_SHORT);
            addValueName(SATURATION_NORMAL, "Normal");
            addValueName(SATURATION_LOW, "Low saturation");
            addValueName(SATURATION_HIGH, "High saturation");
        }
    }

    static class Sharpness extends TIFFTag {
        Sharpness() {
            super("Sharpness", TAG_SHARPNESS, 1 << TIFFTag.TIFF_SHORT);
            addValueName(SHARPNESS_NORMAL, "Normal");
            addValueName(SHARPNESS_SOFT, "Soft");
            addValueName(SHARPNESS_HARD, "Hard");
        }
    }

    static class DeviceSettingDescription extends TIFFTag {
        DeviceSettingDescription() {
            super("DeviceSettingDescription", TAG_DEVICE_SETTING_DESCRIPTION, 1 << TIFFTag.TIFF_UNDEFINED);
        }
    }

    static class SubjectDistanceRange extends TIFFTag {

        SubjectDistanceRange() {
            super("SubjectDistanceRange", TAG_SUBJECT_DISTANCE_RANGE, 1 << TIFFTag.TIFF_SHORT);
            addValueName(SUBJECT_DISTANCE_RANGE_UNKNOWN, "unknown");
            addValueName(SUBJECT_DISTANCE_RANGE_MACRO, "Macro");
            addValueName(SUBJECT_DISTANCE_RANGE_CLOSE_VIEW, "Close view");
            addValueName(SUBJECT_DISTANCE_RANGE_DISTANT_VIEW, "Distant view");
        }
    }

    static class ImageUniqueID extends TIFFTag {
        ImageUniqueID() {
            super("ImageUniqueID", TAG_IMAGE_UNIQUE_ID, 1 << TIFFTag.TIFF_ASCII);
        }
    }

    static class InteroperabilityIFD extends TIFFTag {
        InteroperabilityIFD() {
            super("InteroperabilityIFD", TAG_INTEROPERABILITY_IFD_POINTER, 1 << TIFFTag.TIFF_LONG,
                    EXIFInteroperabilityTagSet.getInstance());
        }
    }
}
