/*
 * $RCSfile: TIFFCIELabColorConverter.java,v $
 *
 * 
 * Copyright (c) 2005 Sun Microsystems, Inc. All  Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met: 
 * 
 * - Redistribution of source code must retain the above copyright 
 *   notice, this  list of conditions and the following disclaimer.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in 
 *   the documentation and/or other materials provided with the
 *   distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of 
 * contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any 
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND 
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR 
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
 * REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
 * INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES. 
 * 
 * You acknowledge that this software is not designed or intended for 
 * use in the design, construction, operation or maintenance of any 
 * nuclear facility. 
 *
 * $Revision: 1.1 $
 * $Date: 2005/02/11 05:01:44 $
 * $State: Exp $
 */
package com.sun.media.imageioimpl.plugins.tiff;

import com.sun.media.imageio.plugins.tiff.TIFFColorConverter;

public final class TIFFCIELabColorConverter extends TIFFColorConverter {

	// XYX coordinate or reference white (CIE D65)
	private static final float XN = 95.047f;
	private static final float YN = 100.0f;
	private static final float ZN = 108.883f;

	private static final float THRESHOLD = (float)Math.pow(0.008856, 1.0 / 3.0);

	private float clamp(float x) {
		if (x < 0.0f) {
			return 0.0f;
		} else if (x > 100.0f) {
			return 255.0f;
		} else {
			return x * (255.0f / 100.0f);
		}
	}

    @Override
	public void toRGB(float x0, float x1, float x2, float[] rgb) {
		float lStar = x0 * 100.0f / 255.0f;
		float aStar = x1 > 128.0f ? x1 - 256.0f : x1;
		float bStar = x2 > 128.0f ? x2 - 256.0f : x2;

		float yYn; // Y/Yn
		float fY; // 'F' value for Y

		if (lStar < 8.0f) {
			yYn = lStar / 903.3f;
			fY = 7.787f * yYn + 16.0f / 116.0f;
		} else {
			float yYnCubeRoot = (lStar + 16.0f) / 116.0f;
			yYn = yYnCubeRoot * yYnCubeRoot * yYnCubeRoot;
			fY = (float)Math.pow(yYn, 1.0 / 3.0);
		}
		float y = yYn * YN;

		float fX = fY + (aStar / 500.0f);
		float x;
		if (fX <= THRESHOLD) {
			x = XN * (fX - 16.0f / 116.0f) / 7.787f;
		} else {
			x = XN * fX * fX * fX;
		}

		float fZ = fY - bStar / 200.0f;
		float z;
		if (fZ <= THRESHOLD) {
			z = ZN * (fZ - 16.0f / 116.0f) / 7.787f;
		} else {
			z = ZN * fZ * fZ * fZ;
		}

		float red =  3.240479f * x - 1.537150f * y - 0.498535f * z;
		float green = -0.969256f * x + 1.875992f * y + 0.041556f * z;
		float blue =  0.055648f * x - 0.204043f * y + 1.057311f * z;

		rgb[0] = clamp(red);
		rgb[1] = clamp(green);
		rgb[2] = clamp(blue);
	}
}
