/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Parses the Ariane command lines.
 * Available options are predefined in the static function 'parse'.
 */
interface CommandLineParser {
    static List<String> parse( String[] args, Map<String, Object> options ) {
        final List<String> graphFilenames = new ArrayList<>();
        int argIndex = 0;
        options.put("nosplash", false);
        while (argIndex < args.length) {
            if (args[argIndex].charAt(0) == '-') {
                if ("-version".equals(args[argIndex]) || "-v".equals(args[argIndex])) {
                    System.err.println("Ariane \"" + ArianeSettings.VERSION_ID + "\"");
                    System.exit(0);
                } else if ("-nosplash".equals(args[argIndex])) {
                    options.put("nosplash", true);
                } else if ("-config".equals(args[argIndex])) {
                    argIndex++;
                    if (argIndex == args.length) {
                        System.err.println("No configuration directory specified ... option -config ignored.");
                    } else {
                        options.put("config", args[argIndex]);
                    }
                } else if ("-generate".equals(args[argIndex])) {
                    argIndex++;
                    if (argIndex == args.length) {
                        System.err.println("No scriptfile specified... option -generate ignored.");
                    } else {
                        options.put("generate", args[argIndex]);
                    }
                } else if ("-lib".equals(args[argIndex])) {
                    argIndex++;
                    if (argIndex == args.length) {
                        System.err.println("No operator library configuration file specified... option -lib ignored.");
                    } else {
                        options.put("lib", args[argIndex]);
                    }
                } else if ("-help".equalsIgnoreCase(args[argIndex]) || "-h".equalsIgnoreCase(args[argIndex])) {
                    System.err.println("Usage: ariane [options] [ariane-file ..]");
                    System.err.println("Options include: ");
                    System.err.println("  -help -h 		display this message");
                    System.err.println("  -version -v	print product version and exit");
                    System.err.println("  -nosplash		launch Ariane without the splash screen");
                    System.err.println("  -config <dir>	specify the configuration directory path isntead of $HOME/.greyc.");
                    System.err.println("  -generate <scriptfile>	generate the script of the specified ariane-file in the specified scriptfile regarding its suffix.");
                    System.err.println("  -lib <file>	use <file> instead of $PANDOREHOME/lib/pandore.cfg");
                    System.exit(0);
                } else {
                    System.err.println("Option '" + args[argIndex] + "' unknown option, ignored...");
                }
            } else {
                graphFilenames.add(args[argIndex]);
            }
            argIndex++;
        }
        if (options.get("generate") != null && graphFilenames.isEmpty()) {
            System.err.println("No ariane file specified... option -generate ignored.");
            options.remove("generate");
        } else if (options.get("generate") != null && graphFilenames.size() > 1) {
            System.err.println("Too much ariane files specified... option -generate ignored.");
            options.remove("generate");
        }
        return graphFilenames;
    }
}
