/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * Determines if a file is a text or anything else.
 */
public interface ImageFileUtils {
    static boolean isText( File file, boolean acceptUTF8encoding ) {
        if (!file.exists()) {
            return false;
        }
        int bufferSize = 1024;
        boolean isText = true;
        byte[] buffer = new byte[bufferSize];
        try {
            try (RandomAccessFile fis = new RandomAccessFile(file, "r")) {
                fis.seek(0);
                final int read = fis.read(buffer);
                int lastByteTranslated = 0;
                for (int i = 0; i < read; i++) {
                    final byte character = buffer[i];
                    int ub = character & 0xff;
                    int utf8value = lastByteTranslated + ub;
                    lastByteTranslated = ub << 8;

                    if (!(ub == 0x09    // tab
                            || ub == 0x0A   // line feed
                            || ub == 0x0C   // form feed
                            || ub == 0x0D   // carriage return
                            || ub >= 0x20 && ub <= 0x7E // Letters, Numbers and other "normal symbols"
                            || ub >= 0xA0 && ub <= 0xEE // Symbols of Latin-1
                            || acceptUTF8encoding && utf8value >= 0x2E2E && utf8value <= 0xC3BF)) {
                        isText = false;
                        break;
                    }
                }
            }
            return isText;
        } catch (IOException e) {
            return false;
        }
    }
}
