/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Locale;

/**
 * *
 * The class <code>PlatformSettings</code> is an utility class.
 * It holds the definition of platform specific values.
 */
public final class PlatformSettings {
    private static final String OPERATING_SYSTEM = System.getProperty("os.name").toLowerCase(Locale.ENGLISH);

    private PlatformSettings() {
    }

    public static boolean isWindows() {
        return OPERATING_SYSTEM.contains("win");
    }

    private static boolean isMac() {
        return OPERATING_SYSTEM.contains("mac");
    }

    public static String getLibraryPathPropertyName() {
        if (PlatformSettings.isMac()) {
            return "DYLD_LIBRARY_PATH";
        } else if (PlatformSettings.isWindows()) {
            return "PATH";
        } else {
            return "LD_LIBRARY_PATH";
        }
    }

    static String getHomePath() {
        return System.getProperty("user.home", ".");
    }

    static String getJavaLibraryPath() {
        return System.getProperty("java.library.path");
    }

    static String getCurrentJarPath() {
        // Tips: getting the absolute file name of the jar.
        Class<?> arianeClass = Ariane.class;
        URL tmp = arianeClass.getResource(arianeClass.getSimpleName() + ".class");
        String filename = tmp.getFile();
        int jarFilenameLength = filename.indexOf('!'); // Jar file uses the ! meta character
        if (jarFilenameLength > 0) {
            filename = filename.substring(0, jarFilenameLength);
        }

        try {
            URL url = new URL(filename);
            return new File(url.toURI()).getParent();
        } catch (URISyntaxException | MalformedURLException e) {
            return new File(filename).getParent();
        }
    }
}
