/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph;

import java.util.List;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;

/**
 * A <code>ThreadGraphExecution</code> controls the execution of a graph of items.
 */
public final class ThreadGraphExecution extends Thread {
    private GraphItemModel[] _selectedItems;

    /**
     * Creates a graph execution thread.
     *
     * @param selectedItems the list of item to be executed.
     */
    public ThreadGraphExecution( List<GraphItemModel> selectedItems ) {
        // We use a copy of the selectItems since the user can modifier the list
        // while the program is executing it.
        _selectedItems = selectedItems.toArray(new GraphItemModel[selectedItems.size()]);
    }

    @Override
    public void run() {
        Ariane.getPresenter().setExecution(true);
        // Force the re-execution
        for (GraphItemModel item : _selectedItems) {
            item.resetAll();
        }
        for (GraphItemModel item : _selectedItems) {
            item.execute();
        }
        _selectedItems = null;
        Ariane.getPresenter().setExecution(false);
    }
}
