/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import fr.greyc.pantheon.ariane.model.graph.slot.ParameterSlot;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.view.graph.element.ControlItemView;
import fr.greyc.pantheon.ariane.view.graph.element.GraphElementView;

/**
 * This is the base class of all control items.
 * It provides facilities to set the number of parameters,
 * inputs and outputs.
 */
abstract class ControlItem extends GraphItemModel {

    ControlItem( GraphElementView graphItemView, TreeItem treeItem, GraphPresenter graph, String name ) {
        super(graphItemView, treeItem, graph, name, null);
        createSlots(0, 0, 0, 0, 1);
    }

    ControlItem( GraphElementView graphItemView, GraphPresenter graph ) {
        super(graphItemView, graph);
        createSlots(0, 0, 0, 0, 1);
    }

    final void askAndSetPrototype( String name, int defaultInputImageNumber, int defaultParameterNumber, int defaultOutputImageNumber ) throws InterruptedException {
        ControlItemView.Prototype p = askForPrototype(name, defaultInputImageNumber, defaultParameterNumber, defaultOutputImageNumber);
        if (p == null) {
            throw new InterruptedException();
        }
        setPrototype(p.getName(), p.getInputs(), p.getParameters(), p.getOutputs());
    }

    final ControlItemView.Prototype askForPrototype( String name, int defaultInputImageNumber, int defaultParameterNumber, int defaultOutputImageNumber ) {
        return ((ControlItemView) getView()).askProperties(name, defaultInputImageNumber, defaultParameterNumber, defaultOutputImageNumber);
    }

    protected abstract void setPrototype( String name, int inputImageNumber, int inputParameterNumber, int outputImageNumber );

    @Override
    public final String toString() {
        StringBuilder commandLine = new StringBuilder("(" + getId() + "): " + getName());

        for (ParameterSlot isp : getInputParameterSlots()) {
            if (isp.getValue() != null) {
                commandLine.append(" ").append(isp.getValue());
            }
        }
        return commandLine.toString();
    }
}
