/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.exception.ArianeMissingInputException;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.graph.slot.InputDataSlot;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.view.graph.element.ControlStructureItemView;

import java.io.IOException;
import java.util.List;

/**
 * A <code>ControlStructureMergeItem</code> merges two inputs slots into
 * a unique output slot. The result is chosen as the first validated input.
 */
public final class MergeControlStructureItem extends ControlItem {
   public static final String TAGNAME = "Merge";

   public MergeControlStructureItem( TreeItem treeItem, GraphPresenter graph ) {
      super(new ControlStructureItemView(), treeItem, graph, treeItem.getName());
      createSlots(treeItem.getNumberOfInputDataSlots(), treeItem.getNumberOfParameterSlots(), 0, treeItem.getNumberOfOutputDataSlots(), 1);
   }

   public MergeControlStructureItem( GraphPresenter graph, List<XMLElement> xmlTree ) throws IOException {
      super(new ControlStructureItemView(), graph);
      createFromXML(xmlTree);
   }

   @Override
   protected void setPrototype( String name, int iIm, int iP, int oIm ) {
      /*
       * empty
       */
   }

   @Override
   public String getTagName() {
      return TAGNAME;
   }

   ///// 
   /////  E X E C U T I O N 
   /////
   @Override
   protected boolean primitiveExecute() throws ArianeRuntimeException {
      try {
         for (int i = 0; i < getInputDataSlots().size(); i++) {
            InputDataSlot isi = getInputDataSlots().get(i);
            if (isi.hasValue(false)) {
               getOutputDataSlots().get(0).setValue(isi.getValue());
               getOutputResultSlot().setValue(Float.toString(i));
               changeStatus(Status.EXECUTED);
               return true;
            }
         }
      } catch (ArianeMissingInputException e) {
         if (GraphPresenter.isExecutionRunning()) {
            Ariane.getPresenter().printErrorMessage(ArianeSettings.ErrorCode.INPUT_OUTPUT, getAbsoluteName(), getId(), e.getLocalizedMessage());
            getGraph().stopExecution();
            changeStatus(Status.ERROR);
         }
         throw new ArianeRuntimeException(e);
      }
      return false;
   }
}
