/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.view.graph.element.NoteElementView;

import java.io.IOException;
import java.util.List;

/**
 * A <code>NoteItem</code> is a comment item that can be added to a graph.
 * FIt contains only text and is visualized as a <code>JTextArea</code>.
 */
public final class NoteItem extends GraphItemModel {
    public static final String TAGNAME = "Note";

    public NoteItem( TreeItem treeItem, GraphPresenter graph ) {
        super(new NoteElementView(), treeItem, graph, treeItem.getName(), null); // TODO deplacer cette construction dans le presenter.
    }

    public NoteItem( GraphPresenter graph, List<XMLElement> xmlTree ) throws IOException {
        super(new NoteElementView(), graph);
        createFromXML(xmlTree);
        if (super.getBinary() != null) { // KEEP THIS FOR COMPATIBILITY REASONS
            super.setFormula(super.getBinary());
            super.setBinary(null);
        }
    }

    @Override
    public String toString() {
        return "(" + getId() + "):" + TAGNAME;
    }

    @Override
    public String getTagName() {
        return TAGNAME;
    }

    @Override
    protected boolean primitiveExecute() {
        return false;
    }
}
