/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.exception.ArianeMissingInputException;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.graph.slot.InputDataSlot;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.view.graph.element.ViewerElementView;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * A <code>ViewerItem</code> manages displaying of a data file in a graph.
 */
public final class ViewerItem extends DataFile {
    public static final String TAGNAME = "Viewer";

    public ViewerItem( TreeItem treeItem, GraphPresenter graph ) {
        super(new ViewerElementView(), treeItem, graph, treeItem.getName(), null);
        createSlots(1, 0, 0, 1, 0);
    }

    public ViewerItem( GraphPresenter graph, List<XMLElement> data ) throws IOException {
        super(new ViewerElementView(), graph);
        createFromXML(data);
    }

    @Override
    public String getTagName() {
        return TAGNAME;
    }

    /////
    ///// E X E C U T I O N
    /////
    private String getViewerCommand() {
        if (getBinary() == null) {
            return UserPreferences.getVisuCommandPath();
        } else {
            return getBinary();
        }
    }

    public String getViewerCommandName() {
        File file = new File(getViewerCommand());
        return file.getName();
    }

    @Override
    protected boolean primitiveExecute() throws ArianeRuntimeException {
        try {
            setCurrentFilename(null);
            InputDataSlot isi = getInputDataSlots().get(0);
            if (isi.hasValue(true)) {
                executeCoreAction(isi);
            } else {
                return false;
            }
            changeStatus(Status.EXECUTED);
            ((ViewerElementView) getView()).displayDataFileContents();
            return true;
        } catch (ArianeMissingInputException e) {
            if (GraphPresenter.isExecutionRunning()) {
                Ariane.getPresenter().printErrorMessage(e.getErrorCode(), getAbsoluteName(), getId(), e.getLocalizedMessage());
                getGraph().stopExecution();
                changeStatus(Status.ERROR);
            }
            throw new ArianeRuntimeException(e);
        }
    }

    void executeCoreAction( InputDataSlot isi ) {
        final String filename = isi.getValue();
        setCurrentFilename(filename);
        getOutputDataSlots().get(0).setValue(filename);
    }
}
