/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * This class parses the template string for the argument line of an operator.
 * Argument line uses : $m, $p, $i, $o $p0, $i1 ...[] wildcards.
 */
public final class ArgumentLineParser {
    private static boolean _isString;
    private static boolean _isSet;
    private static List<String> _commandLine;
    private static List<String> _optionalCommandLine;
    private static List<String> _temp;
    private static String _parsedText;
    private static String _format;
    private static int _index;
    private static char _character;

    private ArgumentLineParser() {
    }

    public static List<String> parse( String command, String customFormat, List<String> maskLine, List<String> parameterLine, List<String> inputLine, List<String> outputLine ) {
        _format = customFormat.replaceAll("\\[", "\\[ ").replaceAll("]", " \\]");
        if (_format.trim().isEmpty()) { // for the sake of compatibility: the Pandore default command line
            _format = " [$p] [-m $m] [$i] [$o]";
        }

        _commandLine = new ArrayList<>();
        _commandLine.add(command);

        _optionalCommandLine = null;
        _temp = _commandLine;
        _parsedText = "";
        _isSet = false;
        _isString = false;

        _index = 0;
        while (_index < _format.length()) {
            _character = _format.charAt(_index++);
            switch (_character) {
                case '"':
                    quoteQuote();
                    break;

                case ' ':
                    space();
                    break;

                case '[':
                    openBracket();
                    break;

                case ']':
                    closeBracket();
                    break;

                case '$':
                    metaCharacter(maskLine, parameterLine, inputLine, outputLine);
                    break;

                default:
                    regularCharacter();
                    break;
            }
        }
        if (_parsedText.length() > 0) {
            _commandLine.add(_parsedText);
        }

        return removeExtraSpaces(_commandLine);
    }

    private static void regularCharacter() {
        _parsedText += _character;
    }

    private static void space() {
        if (_isString) {
            _parsedText += " ";
        } else {
            while (_index < _format.length()) {
                _character = _format.charAt(_index++);
                if (_character != ' ') {
                    _index--;
                    break;
                }
            }
            _temp.add(_parsedText);
            _parsedText = "";
        }
    }

    private static void metaCharacter( List<String> maskLine, List<String> parameterLine, List<String> inputLine, List<String> outputLine ) {
        boolean end = false;
        while (!end && _index < _format.length()) {
            _character = _format.charAt(_index++);
            switch (_character) {
                case '$':
                    dollar();
                    end = true;
                    break;

                case '{':
                case '}':
                    break;

                case 'm':
                    masque(maskLine);
                    break;

                case 'p':
                    replaceMetaCharacter(parameterLine);
                    break;
                case 'i':
                    replaceMetaCharacter(inputLine);
                    break;
                case 'o':
                    replaceMetaCharacter(outputLine);
                    break;

                default:
                    _index--;
                    end = true;
                    break;
            }
        }
    }

    private static void replaceMetaCharacter( List<String> parameterLine ) {
        if (_index < _format.length()) {
            _character = _format.charAt(_index++);
            if (Character.isDigit(_character)) {
                int position = _character - '0';
                if (position < parameterLine.size()) {
                    _parsedText += parameterLine.get(position);
                    _isSet = true;
                }
            } else if (_character == '}') {
                masque(parameterLine);
            } else {
                masque(parameterLine);
                _index--;
            }
        } else {
            masque(parameterLine);
        }
    }

    private static void masque( List<String> maskLine ) {
        _isSet = !maskLine.isEmpty();
        _temp.add(_parsedText);
        _parsedText = "";
        _temp.addAll(maskLine);
    }

    private static void dollar() {
        _parsedText += "$";
    }

    private static void closeBracket() {
        if (_isSet) {
            _temp.add(_parsedText);
            _commandLine.addAll(_temp);
        }
        _parsedText = "";
        _temp = _commandLine;
    }

    private static void openBracket() {
        _optionalCommandLine = new LinkedList<>();
        _temp.add(_parsedText);
        _parsedText = "";
        _temp = _optionalCommandLine;
        _isSet = false;
    }

    private static void quoteQuote() {
        _isString = !_isString;
        _parsedText += "\"";
    }

    private static List<String> removeExtraSpaces( List<String> commandLine ) {
        List<String> newCommandLine = new LinkedList<>();
        if (commandLine != null) {
            for (String commandPart : commandLine) {
                if (!commandPart.trim().isEmpty()) {
                    newCommandLine.add(commandPart.trim());
                }
            }
        }
        return newCommandLine;
    }
}
