/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import fr.greyc.pantheon.ariane.Ariane;

import java.io.IOException;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.ImageFileUtils;
import fr.greyc.pantheon.ariane.UserPreferences;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Executes viewer command by the operating system.
 */
public final class ExternalViewerProcessBuilder extends OperatingSystemProcess {

    public ExternalViewerProcessBuilder( File dataFile ) {
        String viewerBinary;
        if (ImageFileUtils.isText(dataFile, false)) {
            String command = UserPreferences.getTextEditorPath();
            if (command == null || command.isEmpty()) {
                Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("FileNameItem.error2"));
                return;
            } else {
                viewerBinary = command;
            }
        } else {
            viewerBinary = UserPreferences.getVisuCommandPath();
            if (viewerBinary == null) {
                Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("FileNameItem.error3"));
                return;
            }
        }
        List<String> commandLine = new ArrayList<>();
        commandLine.add(viewerBinary);
        commandLine.add(dataFile.getAbsolutePath());
        super.createProcess(commandLine);
    }

    public void start() {
        if (getProcessBuilder() != null) {
            new Thread(this::launchViewer).start();
        }
    }

    private void launchViewer() {
        try {
            Process process = getProcessBuilder().start();
            try (BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()))) {
                String line;
                while ((line = reader.readLine()) != null) {
                    Ariane.getPresenter().printErrorMessage(line);
                }
            }
            process.waitFor();
        } catch (IOException | InterruptedException e) {
            Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("Slot.error5") + e.getMessage());
        }
    }
}
