/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import java.lang.reflect.MalformedParameterizedTypeException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class parses the template string for the list of default parameter values.
 */
public final class ItemSignatureParser {

    private ItemSignatureParser() {
        // Empty
    }

    public static String[][] parse( String formattedLine, int nbOfParameters ) throws MalformedParameterizedTypeException {
        if (formattedLine.trim().isEmpty()) {
            return null;
        }
        String[] parts = formattedLine.split(";");
        String[][] parameterValues = new String[nbOfParameters][2];
        Pattern pattern = Pattern.compile("\\s*\\$p?(\\d)\\s*:(.*)");
        for (String part : parts) {
            Matcher matcher = pattern.matcher(part);
            if (!matcher.matches()) {
                throw new MalformedParameterizedTypeException();
            }
            String parameter = matcher.group(1);
            int index = Integer.parseInt(parameter);
            if (index < 0 || index > nbOfParameters) {
                throw new MalformedParameterizedTypeException();
            }
            String namevalue = matcher.group(2);
            String[] pair = namevalue.split("=");
            if (pair[0].length() > 0) {
                parameterValues[index][0] = pair[0];
            } else {
                parameterValues[index][0] = null;
            }
            if (pair.length < 2) {
                parameterValues[index][1] = "";
            } else {
                parameterValues[index][1] = pair[1];
            }
        }
        return parameterValues;
    }
}
