/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.slot;

import fr.greyc.pantheon.ariane.view.graph.slot.LinkView;

import java.awt.Point;
import java.awt.geom.Line2D;
import java.util.ArrayList;

// TODO tests
/**
 * An <code>DataLink</code> is a connection between two slots that supports images.
 * It is represented as a polyline.
 */
public final class DataLinkModel extends LinkModel {

    DataLinkModel( InputSlot slot, LinkView linkView ) {
        super(slot, linkView);
    }

    @Override
    protected ArrayList<Point> createPolyline() {
        ArrayList<Point> path = new ArrayList<>();

        // Insert the first point.
        path.add(0, new Point(getSourceCoordinate()));

        final int[] xcoord = new int[6];
        final int[] ycoord = new int[6];
        xcoord[3] = (int) getTargetCoordinate().getX();
        ycoord[3] = (int) getTargetCoordinate().getY();
        xcoord[0] = (int) getSourceCoordinate().getX();
        ycoord[0] = (int) getSourceCoordinate().getY();

        if (xcoord[0] <= xcoord[3]) { // nominal case: item 1 in front of item 2
            xcoord[1] = xcoord[3] - (xcoord[3] - xcoord[0]) / 5; //*(1+_targetSlot.getRank()));
            ycoord[1] = ycoord[0];
            path.add(new Point(xcoord[1], ycoord[1]));
            xcoord[2] = xcoord[1];
            ycoord[2] = ycoord[3];
            path.add(new Point(xcoord[2], ycoord[2]));
        } else { // degenerate case... item 1 behind item 1
            xcoord[5] = xcoord[3];
            ycoord[5] = ycoord[3];
            xcoord[1] = xcoord[0] + 10;
            ycoord[1] = ycoord[0];
            if (ycoord[0] < ycoord[5] - 15) { // link above item 1.
                xcoord[2] = xcoord[1];
                ycoord[2] = (ycoord[5] - 10 - ycoord[0]) / 2 + ycoord[0];
                xcoord[3] = xcoord[5] - 10;
                ycoord[3] = ycoord[2];
            } else { // link below item 1.
                xcoord[2] = xcoord[0] + 10;
                ycoord[2] = ycoord[5] + 20;
                xcoord[3] = xcoord[5] - 10;
                ycoord[3] = ycoord[2];
            }
            xcoord[4] = xcoord[3];
            ycoord[4] = ycoord[5];

            // insert intermediate points
            path.add(new Point(xcoord[1], ycoord[1]));
            path.add(new Point(xcoord[2], ycoord[2]));
            path.add(new Point(xcoord[3], ycoord[3]));
            path.add(new Point(xcoord[4], ycoord[4]));
        }
        // insert the last point.
        path.add(new Point(getTargetCoordinate()));
        return path;
    }

    /**
     * Moves the current selected intermediate line
     * of the link according to the new coordinates.
     *
     * @param xycoord the new coordinates.
     */
    @Override
    public void move( Point xycoord ) {
        setUserDefined();
        int selectedLine = getSelectedLineId();
        if (selectedLine < 1) {
            return;
        }
        Line2D.Float[] polyline = getPolyline();
        if (polyline.length == 3) {
            float x;
            if (xycoord.x < polyline[0].x1 + GAP) {
                x = polyline[0].x1 + GAP;
            } else if (xycoord.x > polyline[2].x2 - GAP) {
                x = polyline[2].x2 - GAP;
            } else {
                x = xycoord.x;
            }
            polyline[0].x2 = x;
            polyline[1].x1 = x;
            polyline[1].x2 = x;
            polyline[2].x1 = x;
        } else {
            switch (selectedLine) {
                case 1: {
                    float x;
                    if (xycoord.x < polyline[0].x1 + GAP) {
                        x = polyline[0].x1 + GAP;
                    } else {
                        x = xycoord.x;
                    }
                    polyline[0].x2 = x;
                    polyline[1].x1 = x;
                    polyline[1].x2 = x;
                    polyline[2].x1 = x;
                }
                break;
                case 2: {
                    float y;
                    float low;
                    float high;
                    if (polyline[0].y1 > polyline[polyline.length - 1].y1) {
                        low = polyline[polyline.length - 1].y2 + GAP;
                        high = polyline[0].y1 - GAP;
                    } else {
                        low = polyline[0].y1 + GAP;
                        high = polyline[polyline.length - 1].y2 - GAP;
                    }

                    if (xycoord.y < low) {
                        y = low;
                    } else if (xycoord.y > high) {
                        y = high;
                    } else {
                        y = xycoord.y;
                    }
                    polyline[1].y2 = y;
                    polyline[2].y1 = y;
                    polyline[2].y2 = y;
                    polyline[3].y1 = y;
                }
                break;
                case 3: {
                    float x;
                    if (xycoord.x > polyline[polyline.length - 1].x2 - GAP) {
                        x = polyline[polyline.length - 1].x2 - GAP;
                    } else {
                        x = xycoord.x;
                    }
                    polyline[2].x2 = x;
                    polyline[3].x2 = x;
                    polyline[4].x1 = x;
                }
                break;
                default:
                    break;
            }
        }
    }

    /**
     * Modifies the link because one the source or target items has been moved.
     *
     * @return false if the link cannot be modified because it becomes too complex.
     */
    @Override
    protected boolean modifyPolyline() {
        double sourceX = getSourceCoordinate().getX();
        double targetX = getTargetCoordinate().getX();
        Line2D.Float[] polyline = getPolyline();
        if (polyline.length == 3 && sourceX >= targetX
                || polyline.length == 5 && sourceX < targetX) {
            return false;
        }

        double sourceY = getSourceCoordinate().getY();
        double targetY = getTargetCoordinate().getY();
        if ((int) sourceX != (int) polyline[0].x1 || (int) sourceY != (int) polyline[0].y1) {
            polyline[0].y1 = (float) sourceY;
            polyline[0].y2 = (float) sourceY;
            polyline[1].y1 = (float) sourceY;
            polyline[0].x1 = (float) sourceX;
            return !(sourceX > polyline[0].x2);
        } else if ((int) targetX != (int) polyline[polyline.length - 1].x2 || (int) targetY != (int) polyline[polyline.length - 1].y2) {
            polyline[polyline.length - 1].y2 = (float) targetY;
            polyline[polyline.length - 1].y1 = (float) targetY;
            polyline[polyline.length - 2].y2 = (float) targetY;

            polyline[polyline.length - 1].x2 = (float) targetX;
            return !(targetX < polyline[polyline.length - 1].x1);
        }
        return true;
    }
}
