/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.slot;

import fr.greyc.pantheon.ariane.view.graph.slot.LinkView;

import java.awt.Point;
import java.awt.geom.Line2D;
import java.util.ArrayList;

/**
 * An <code>ImageLink</code> is a connection between
 * two slots that supports mask images.
 * It is represented as a polyline.
 */
public final class MaskLinkModel extends LinkModel {
    MaskLinkModel( InputSlot slot, LinkView linkView ) {
        super(slot, linkView);
    }

    @Override
    protected ArrayList<Point> createPolyline() {
        ArrayList<Point> path = new ArrayList<>();

        // Insert the first point.
        path.add(0, new Point(getSourceCoordinate()));

        final int[] xcoord = new int[6];
        final int[] ycoord = new int[6];
        xcoord[4] = (int) getTargetCoordinate().getX();
        ycoord[4] = (int) getTargetCoordinate().getY();
        xcoord[0] = (int) getSourceCoordinate().getX();
        ycoord[0] = (int) getSourceCoordinate().getY();

        xcoord[1] = xcoord[0] + 15;
        ycoord[1] = ycoord[0];
        xcoord[2] = xcoord[1];
        ycoord[2] = ycoord[4] - 15;
        xcoord[3] = xcoord[4];
        ycoord[3] = ycoord[2];

        path.add(new Point(xcoord[1], ycoord[1]));
        path.add(new Point(xcoord[2], ycoord[2]));
        path.add(new Point(xcoord[3], ycoord[3]));

        // insert the last point.
        path.add(new Point(getTargetCoordinate()));

        return path;
    }

    /**
     * Moves the current selected intermediate line of the link according to the new coordinates.
     *
     * @param xycoord the new coordinates.
     */
    @Override
    public void move( Point xycoord ) {
        setUserDefined();
        int selectedLine = getSelectedLineId();
        if (selectedLine < 1) {
            return;
        }
        Line2D.Float[] polyline = getPolyline();
        if (selectedLine == 1) {
            float x;
            if (xycoord.x < polyline[0].x1 + GAP) {
                x = polyline[0].x1 + GAP;
            } else {
                x = xycoord.x;
            }
            polyline[0].x2 = x;
            polyline[1].x1 = x;
            polyline[1].x2 = x;
            polyline[2].x1 = x;
        } else if (selectedLine == 2) {
            float y;
            if (xycoord.y > polyline[polyline.length - 1].y2 - GAP) {
                y = polyline[polyline.length - 1].y2 - GAP;
            } else {
                y = xycoord.y;
            }
            polyline[1].y2 = y;
            polyline[2].y1 = y;
            polyline[2].y2 = y;
            polyline[3].y1 = y;
        }
    }

    /**
     * Modifies the link because one the source or target items has been moved.
     *
     * @return false if the link cannot be modified because it becomes too complex.
     */
    @Override
    protected boolean modifyPolyline() {
        double sourceX = getSourceCoordinate().getX();
        double sourceY = getSourceCoordinate().getY();
        double targetX = getTargetCoordinate().getX();
        double targetY = getTargetCoordinate().getY();
        Line2D.Float[] polyline = getPolyline();
        if ((int) sourceX != (int) polyline[0].x1 || (int) sourceY != (int) polyline[0].y1) {
            polyline[0].y1 = (float) sourceY;
            polyline[0].y2 = (float) sourceY;
            polyline[1].y1 = (float) sourceY;
            polyline[0].x1 = (float) sourceX;
            return !(sourceX > polyline[0].x2);
        } else if ((int) targetX != (int) polyline[polyline.length - 1].x2 || (int) targetY != (int) polyline[polyline.length - 1].y2) {
            polyline[polyline.length - 1].y2 = (float) targetY;
            polyline[polyline.length - 1].x2 = (float) targetX;
            polyline[polyline.length - 1].x1 = (float) targetX;
            polyline[polyline.length - 2].x2 = (float) targetX;

            return !(targetY < polyline[polyline.length - 1].y1);
        }
        return true;
    }
}
