/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.slot;

import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.element.DataFile;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.SlotPresenter;
import fr.greyc.pantheon.ariane.view.graph.slot.OutputDataSlotView;

/**
 * The class <code>OutputDataSlot</code> defines output image slot.
 * It can be connected to several <code>InputDataSlot</code>.
 * An <code>OutputDataSlot</code> can be set with a data file.
 */
public final class OutputDataSlot extends OutputSlot {
    private String _value;

    public OutputDataSlot( GraphItemModel item, int rank ) {
        super(new SlotPresenter(new OutputDataSlotView()), item, rank);
    }

    @Override
    public String toString() {
        if (getGraphItem() instanceof DataFile) {
            return "OutputDataSlot: " + getGraphItem().getBinary();
        }
        if (_value != null) {
            return "OutputDataSlot: " + _value;
        }
        return "OutputDataSlot: " + getTempName();
    }

    private String createTempFilename() {
        return UserPreferences.createTemporaryName(getGraph().getIdentifier(), getGraphItem().getId(), getRank());
    }

    /**
     * Creates the complete filename for a temporary file.
     * <br>Uses the value of temporary directory defined in the preferences.
     * <br><i>Does not use the java method <tt>createTempFile</tt>
     * because it creates the filename in the /temp directory</i>.
     *
     * @return the complete filename.
     */
    public String getTempName() {
        return UserPreferences.getTemporaryDirectory() + createTempFilename();
    }

    public String getValue() {
        return _value;
    }

    public void setValue() {
        _value = getTempName();
        setValidated();
    }

    public void setValue( String fileName ) {
        _value = fileName;
        setValidated();
    }

    @Override
    public boolean isAcceptableDropSite( SlotModel slot ) {
        return slot.getGraphItem() != this.getGraphItem()
                && (slot instanceof InputDataSlot || slot instanceof InputMaskSlot);
    }

    /////
    ///// I N P U T   /   O U T P U T
    /////
    @Override
    public void convertToXML( XMLElement element ) {
        element.addChildren(new XMLElement("StartItemId", Integer.toString(getGraphItem().getId())));
        element.addChildren(new XMLElement("StartSlotType", "OutSlotImage"));
        element.addChildren(new XMLElement("StartSlotIndex", Integer.toString(getRank())));
    }
}
