/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.slot;

import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.SlotPresenter;
import fr.greyc.pantheon.ariane.view.graph.slot.OutputResultSlotView;

/**
 * The class <code>OutputResultSlot</code> defines result slot.
 * <br>Result value can be :
 * <ul>
 * <li>SUCCESS
 * <li>FAILURE
 * <li>a float
 * <li>a string.
 * </ul>
 * <br>It can be connected to several <code>InputParameterSlot</code>.
 */
public final class OutputResultSlot extends OutputSlot {
    private String _result;

    public OutputResultSlot( GraphItemModel item, int rank ) {
        super(new SlotPresenter(new OutputResultSlotView()), item, rank);
    }

    @Override
    public String toString() {
        return "OutputResultSlot: " + getValue();
    }

    /////
    /////  V A L U E
    /////
    public void setValue( String result ) {
        _result = result;
        setValidated();
    }

    public String getValue() {
        return _result;
    }

    @Override
    public boolean isAcceptableDropSite( SlotModel slot ) {
        return slot.getGraphItem() != this.getGraphItem() && slot instanceof ParameterSlot;
    }

    @Override
    public void convertToXML( XMLElement element ) {
        element.addChildren(new XMLElement("StartItemId", Integer.toString(getGraphItem().getId())));
        element.addChildren(new XMLElement("StartSlotType", "OutSlotResult"));
        element.addChildren(new XMLElement("StartSlotIndex", Integer.toString(getRank())));
    }
}
