/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane.model.tree;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;

import java.util.List;

public final class ItemLibraryFactory {
    public static ItemLibrary createItemsFromXMLTree( List<XMLElement> xmlTree ) {
        ItemLibrary library = new ItemLibrary();
        if (xmlTree.isEmpty()) {
            return library;
        }
        // Discard tag <ARIANE>.
        List<XMLElement> xmlContents = xmlTree.get(0).getChildren();
        if (xmlContents == null) {
            return library;
        }
        for (XMLElement element : xmlContents) {
            if ("OperatorLibrary".equalsIgnoreCase(element.getName())
                    || "Library".equalsIgnoreCase(element.getName())
                    || "Bibliotheque".equalsIgnoreCase(element.getName())) {
                for (XMLElement item : element.getChildren()) {
                    String tag = item.getName();
                    if ("DocPath".equalsIgnoreCase(tag)) {
                        ArianeSettings.setLibraryDocumentationPath(item.getValue());
                    } else if ("CommandStatus".equalsIgnoreCase(tag)) {
                        UserPreferences.setCommandStatus(item.getValue());
                    } else if ("CommandVisu".equalsIgnoreCase(tag)) {
                        UserPreferences.setCommandVisu(item.getValue());
                    } else if ("Version".equalsIgnoreCase(tag)) {
                        library.version=item.getValue();
                        if (item.getValue().startsWith("Pandore")) {
                            UserPreferences.setCommandStatus("pstatus");
                            UserPreferences.setCommandVisu("pvisu");
                            UserPreferences.setPandore();
                        }
                    } else if ("Index".equalsIgnoreCase(tag)) {
                        library.pathToHTMLLibraryDocumentationFile = item.getValue();
                    } else if ("Operator".equalsIgnoreCase(tag) || "Operateur".equalsIgnoreCase(tag)) {
                        library.treeItems.add(new OperatorTreeItem(item.getChildren()));
                    }
                }
                return library;
            } else if ("SubroutineLibrary".equalsIgnoreCase(element.getName())) {
                for (XMLElement item : element.getChildren()) {
                    if ("Subroutine".equalsIgnoreCase(item.getName())) {
                        library.treeItems.add(new SubroutineTreeItem(item.getChildren()));
                    }
                }
                return library;
            }
        }
        return library;
    }
}
