/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 * 
 * https://clouard.users.greyc.fr
 * 
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.tree;

import fr.greyc.pantheon.ariane.model.graph.element.ForControlStructureItem;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;
import fr.greyc.pantheon.ariane.model.graph.element.MacroOperatorItem;
import fr.greyc.pantheon.ariane.model.graph.element.WhileControlStructureItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;

import java.awt.datatransfer.DataFlavor;
import java.io.IOException;
import java.util.List;

/**
 * A <code>SubroutineTreeItem</code> represents a subroutine
 * that can be inserted in an Ariane graph.
 */
public final class SubroutineTreeItem extends TreeItem {
    public static final DataFlavor[] MIME_TYPE = {new DataFlavor("application/x-java-serialized-subroutine-item-tree", "Subroutine Tree Item")};
    private List<XMLElement> _body;

    public SubroutineTreeItem( List<XMLElement> xmlTree ) {
        super(xmlTree);
    }

    @Override
    public DataFlavor[] getTransferDataFlavors() {
        return MIME_TYPE;
    }

    public List<XMLElement> getBody() {
        return _body;
    }

    @Override
    public GraphItemModel createGraphItem( GraphPresenter graph ) {
        // TODO deplacer dans une factory ?
        try {
            GraphItemModel item;
            switch (getType()) {
                case MACRO_OPERATOR:
                    item = new MacroOperatorItem(graph, _body, this);
                    break;
                case FOR_CONTROL:
                    item = new ForControlStructureItem(graph, _body, this);
                    break;
                case WHILE_CONTROL:
                    item = new WhileControlStructureItem(graph, _body, this);
                    break;
                default:
                    return null;
            }
            item.updateName();
            return item;
        } catch (IOException e) {
            return null;
        }
    }

    @Override
    public XMLElement convertToXMLTree() {
        XMLElement element = new XMLElement("Subroutine");
        element.addChildren(new XMLElement("Category", getCategory()));
        element.addChildren(new XMLElement("Name", getName()));
        element.addChildren(new XMLElement("Documentation", getDocumentationURL()));
        element.addChildren(new XMLElement("Signature", getSignature()));
        XMLElement item;
        switch (getType()) {
            case MACRO_OPERATOR:
                item = new XMLElement("MacroOperator");
                break;
            case FOR_CONTROL:
                item = new XMLElement("For");
                break;
            case WHILE_CONTROL:
                item = new XMLElement("While");
                break;
            default:
                return null;
        }
        item.addChildren(_body);
        element.addChildren(item);
        return element;
    }

    @Override
    protected void loadItemFromXMLTree( List<XMLElement> xmlelements ) {
        for (XMLElement item : xmlelements) {
            if ("Category".equalsIgnoreCase(item.getName())) {
                setCategory(item.getValue());
            } else if ("Name".equalsIgnoreCase(item.getName())) {
                setName(item.getValue());
            } else if ("Documentation".equalsIgnoreCase(item.getName())) {
                setDocumentationURL(item.getValue());
            } else if ("Signature".equalsIgnoreCase(item.getName())) {
                setSignature(item.getValue());
            } else if ("MacroOperator".equalsIgnoreCase(item.getName())) {
                setType(ItemType.MACRO_OPERATOR);
                _body = item.getChildren();
            } else if ("For".equalsIgnoreCase(item.getName())) {
                setType(ItemType.FOR_CONTROL);
                _body = item.getChildren();
            } else if ("While".equalsIgnoreCase(item.getName())) {
                setType(ItemType.WHILE_CONTROL);
                _body = item.getChildren();
            }
        }
    }
}
