/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane.model.tree;

import java.util.ArrayList;
import java.util.List;

/**
 * Creates tree items.
 */
public final class TreeItemFactory {
    private TreeItemFactory() {
    }

    public static List<TreeItem> createAllBuiltinItems() {
        List<TreeItem> items = new ArrayList<>();
        for (TreeItem.ItemType itemType : TreeItem.ItemType.values()) {
            if (itemType != TreeItem.ItemType.OPERATOR) {
                TreeItem treeItem = TreeItemFactory.createItem(itemType);
                items.add(treeItem);
            }
        }
        return items;
    }

    public static TreeItem createItem( TreeItem.ItemType itemType ) {
        switch (itemType) {
            case CONSTANT:
                return new OperatorTreeItem(TreeItem.ItemType.CONSTANT, "constant", "constant.html", 0, 0, 0, "");
            case EVAL:
                return new OperatorTreeItem(TreeItem.ItemType.EVAL, "eval", "eval.html", 0, 2, 0, "");
            case FILENAME:
                return new OperatorTreeItem(TreeItem.ItemType.FILENAME, "filename", "filename.html", 0, 0, 0, "");
            case FOR_CONTROL:
                return new OperatorTreeItem(TreeItem.ItemType.FOR_CONTROL, "for", "for.html", 3, 0, 0, "$0:Initial value;$1:Final value;$2:Increment");
            case FORMAT:
                return new OperatorTreeItem(TreeItem.ItemType.FORMAT, "format", "format.html", 0, 2, 0, "");
            case IF_CONTROL:
                return new OperatorTreeItem(TreeItem.ItemType.IF_CONTROL, "if", "if.html", 1, 2, 2, "");
            case LOADER:
                return new OperatorTreeItem(TreeItem.ItemType.LOADER, "loader", "loader.html", 0, 0, 1, "");
            case MACRO_OPERATOR:
                return new OperatorTreeItem(TreeItem.ItemType.MACRO_OPERATOR, "macro-operator", "macro.html", 0, 0, 0, "");
            case MERGE_CONTROL:
                return new OperatorTreeItem(TreeItem.ItemType.MERGE_CONTROL, "merge", "merge.html", 2, 0, 1, "");
            case OPERATOR:
                return new OperatorTreeItem(TreeItem.ItemType.OPERATOR, "operator", "", 0, 0, 0, "");
            case SAVER:
                return new OperatorTreeItem(TreeItem.ItemType.SAVER, "saver", "saver.html", 1, 0, 0, "");
            case SCRIPT:
                return new OperatorTreeItem(TreeItem.ItemType.SCRIPT, "script", "script.html", 0, 0, 0, "");
            case SELECT_CONTROL:
                return new OperatorTreeItem(TreeItem.ItemType.SELECT_CONTROL, "select", "select.html", 0, 0, 1, "$0:Index");
            case SYSTEM:
                return new OperatorTreeItem(TreeItem.ItemType.SYSTEM, "system", "system.html", 0, 0, 0, "");
            case VIEWER:
                return new OperatorTreeItem(TreeItem.ItemType.VIEWER, "viewer", "viewer.html", 1, 0, 0, "");
            case WHILE_CONTROL:
                return new OperatorTreeItem(TreeItem.ItemType.WHILE_CONTROL, "while", "while.html", 2, 0, 0, "");
            case NOTE:
            default:
                return new OperatorTreeItem(TreeItem.ItemType.NOTE, "note", "note.html", 0, 0, 0, "");
        }
    }
}
