/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.xml;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 * An <code>XMLElement</code> defines a pair tag-value
 * that can be read or write in an XML file.
 */
public final class XMLElement implements Transferable, Serializable {
    public static final DataFlavor FLAVOR = new DataFlavor(fr.greyc.pantheon.ariane.model.xml.XMLElement.class, "Ariane graph");
    private static final long serialVersionUID = 42L;
    private static final DataFlavor[] FLAVORS = {FLAVOR,};
    private final String _tag;
    private String _value = "";
    private List<XMLElement> _children;

    public XMLElement( String tag ) {
        _tag = tag;
    }

    public XMLElement( final String tag, final String value ) {
        _tag = tag;
        setValue(value);
    }

    public String getName() {
        return _tag;
    }

    public String getValue() {
        return _value;
    }

    public void setValue( String value ) {
        if (value != null) {
            _value = value;
        }
    }

    public List<XMLElement> getChildren() {
        return _children;
    }

    public void addChildren( XMLElement element ) {
        if (element != null) {
            if (_children == null) {
                _children = new LinkedList<>();
            }
            _children.add(element);
        }
    }

    public void addChildren( List<XMLElement> xmlTree ) {
        if (xmlTree == null) {
            return;
        }
        if (_children == null) {
            _children = xmlTree;
        } else { // Catenate 2 linked-lists.
            while (!xmlTree.isEmpty()) {
                _children.add(xmlTree.remove(0));
            }
        }
    }

    @Override
    public DataFlavor[] getTransferDataFlavors() {
        return FLAVORS.clone();
    }

    @Override
    public boolean isDataFlavorSupported( DataFlavor flavor ) {
        return flavor.equals(FLAVORS[0]);
    }

    @Override
    public Object getTransferData( DataFlavor flavor ) throws UnsupportedFlavorException, IOException {
        if (flavor.equals(FLAVORS[0])) {
            return this;
        }
        throw new UnsupportedFlavorException(flavor);
    }
}
