/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.xml;

import fr.greyc.pantheon.ariane.ArianeSettings;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

/**
 * <i>XMLFileInputStream</i> is a parser of XML tags.
 * It reads and build an XML tree of elements (tag, value).
 */
public final class XMLFileInputStream extends DefaultHandler {
    private FileInputStream _ifs;
    private final StringBuffer _readValue = new StringBuffer(255);
    private List<XMLElement> _finalElements;
    private Stack<XMLElement> _currentElements;

    public XMLFileInputStream( final String filename ) throws IOException {
        try {
            _ifs = new FileInputStream(filename);
        } catch (IOException e) {
            throw new IOException(ArianeSettings.getString("XMLFileInputStream.error.message1") + filename, e);
        }
    }

    public XMLFileInputStream( final File filename ) throws IOException {
        try {
            _ifs = new FileInputStream(filename);
        } catch (IOException e) {
            throw new IOException(ArianeSettings.getString("XMLFileInputStream.error.message1") + filename, e);
        }
    }

    public void parse() throws SAXException, ParserConfigurationException, IOException {
        InputSource inputStream = new InputSource(_ifs);

        SAXParserFactory spf = SAXParserFactory.newInstance();
        // Set namespaceAware to true to get a default parser (SAX2, JAXP 1.0=false!)
        spf.setNamespaceAware(true);
        SAXParser saxParser = spf.newSAXParser();
        XMLReader xmlReader = saxParser.getXMLReader();

        xmlReader.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
        xmlReader.setFeature("http://xml.org/sax/features/validation", false);

        xmlReader.setContentHandler(this);
        xmlReader.setErrorHandler(this);
        // Do parsing
        xmlReader.parse(inputStream);
    }

    @Override
    public void startDocument() throws SAXException {
        _currentElements = new Stack<>();
        _finalElements = new LinkedList<>();
    }

    @Override
    public void endDocument() throws SAXException {
        _currentElements = null;
    }

    @Override
    public void startElement( String uri, String tag, String qName, Attributes atts ) throws SAXException {
        XMLElement element = new XMLElement(tag);

        if (_currentElements.empty()) { // It is a sub-item.
            _finalElements.add(element);
        } else {
            XMLElement element1 = _currentElements.peek();
            element1.addChildren(element);
        }
        _currentElements.push(element);
        _readValue.setLength(0);
    }

    @Override
    public void endElement( String uri, String tag, String qName ) throws SAXException {
        XMLElement element = _currentElements.pop();
        element.setValue(_readValue.toString());
    }

    @Override
    public void characters( char[] characters, int start, int length ) throws SAXException {
        _readValue.append(characters, start, length);
    }

    public List<XMLElement> getElements() {
        return _finalElements;
    }

    public void close() throws IOException {
        _ifs.close();
    }
}
