/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane.presenter;

import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;
import fr.greyc.pantheon.ariane.view.command.ChangeTextEdit;
import fr.greyc.pantheon.ariane.view.graph.element.GraphElementView;

import javax.swing.JTextArea;

public final class GraphItemPresenter {
    private final GraphElementView _graphItemView;
    private final GraphItemModel _graphItem;

    public GraphItemPresenter( GraphItemModel graphItem, GraphElementView graphItemView ) {
        _graphItem = graphItem;
        _graphItemView = graphItemView;
        _graphItemView.setGraphElementPresenter(this);
    }

    public GraphElementView getView() {
        return _graphItemView;
    }

    public GraphItemModel getModel() {
        return _graphItem;
    }

    public void addInCurrentSelection( boolean shiftPressed ) {
        getModel().getGraph().addInSelection(getModel(), shiftPressed);
    }

    public void setNickName( String text ) {
        _graphItem.setNickName(text);
    }

    public boolean isSelected() {
        return getModel().isSelected();
    }

    public GraphItemModel.Status getExecutionStatus() {
        return getModel().getExecutionStatus();
    }

    public void updateFormula( JTextArea textArea, String newFormula ) {
        String oldFormula = getModel().getFormula();
        if (oldFormula == null) {
            oldFormula = "";
        }
        if (!newFormula.equals(oldFormula)) {
            getModel().getGraph().validateOneAction(new ChangeTextEdit(getModel(), textArea, newFormula, oldFormula));
            getModel().setFormula(newFormula);
        }
    }

    public boolean isExecuted() {
        return getModel().getExecutionStatus() == GraphItemModel.Status.EXECUTED;
    }

    public String getNewCommandVisu( String newCommand ) {
        if (newCommand != null && newCommand.length() > 0 && !newCommand.equals(UserPreferences.getCommandVisu())) {
            getModel().setBinary(newCommand);
        } else {
            getModel().setBinary(null);
        }
        getModel().getGraph().setModified(true);
        getModel().resetAll();


        if (getModel().getBinary() == null) {
            return UserPreferences.getCommandVisu();
        } else {
            return getModel().getBinary();
        }
    }
}
