/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane.presenter;

import fr.greyc.pantheon.ariane.model.graph.element.etc.ExternalViewerProcessBuilder;
import fr.greyc.pantheon.ariane.model.graph.slot.InputSlot;
import fr.greyc.pantheon.ariane.model.graph.slot.OutputDataSlot;
import fr.greyc.pantheon.ariane.model.graph.slot.SlotModel;
import fr.greyc.pantheon.ariane.view.graph.GraphView;
import fr.greyc.pantheon.ariane.view.graph.slot.LinkView;
import fr.greyc.pantheon.ariane.view.graph.slot.SlotView;

import java.io.File;

public final class SlotPresenter {
    private final SlotView _slotView;
    private SlotModel _slotModel;
    private boolean _selected;
    private static SlotModel _targetSlot;
    private static SlotModel _sourceSlot;

    public SlotPresenter( SlotView slotView ) {
        _slotView = slotView;
        _slotView.setPresenter(this);
    }

    public void setModel( SlotModel slotModel ) {
        _slotModel = slotModel;
    }

    public SlotView getView() {
        return _slotView;
    }

    public SlotModel getModel() {
        return _slotModel;
    }

    private void highlightSlot( boolean isSelected ) {
        _selected = isSelected;
        if (isSelected) {
            getView().setActiveBorder();
        } else {
            getView().setNormalBorder();
        }
    }

    public final void setSelected( boolean isSelected ) {
        if (getModel().isConnected()) {
            highlightSlot(isSelected);
            for (SlotModel slot : getModel().getConnectedSlots()) {
                slot.getPresenter().highlightSlot(isSelected);
            }
        }
    }

    public final void dropIntoGraph() {
        setSelected(false);
        if (_targetSlot == null) {
            getModel().getGraph().disconnectSlot(getModel());
        } else {
            getView().setNormalBorder();
            getModel().connectWith(_targetSlot);
        }
    }

    public final void startConnection() {
        synchronized (this) {
            _targetSlot = null;
            _sourceSlot = getModel();
        }
    }

    public final void mouseExited() {
        if (_sourceSlot != null) {
            if (!_selected) {
                getView().setNormalBorder();
            }
            _targetSlot = null;
        }
    }

    public final void mouseEntered() {
        if (_sourceSlot != null) {
            _targetSlot = getModel();
            // Accepts the drop action for a move action.
            if (_sourceSlot.isAcceptableDropSite(_targetSlot)) {
                getView().setActiveBorder();
            }
        }
    }

    public boolean isSelected() {
        return _selected;
    }

    public GraphView getGraphView() {
        return getModel().getGraph().getView();
    }

    public void displayDataFileContents() {
        OutputDataSlot slot = (OutputDataSlot) getModel();
        if (slot.isValidate()) {
            File file = new File(slot.getValue());
            if (file.exists()) {
                ExternalViewerProcessBuilder pb = new ExternalViewerProcessBuilder(file);
                pb.start();
            }
        }
    }

    public SlotPresenter getConnectedOutputPresenter() {
        return getModel().getConnectedSlots().get(0).getPresenter();
    }

    public LinkView drawLink() {
        if (getModel().isConnected()) {
            return ((InputSlot) getModel()).getLink().getView();
        } else {
            return null;
        }
    }
}
