/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.presenter;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.SubGraph;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;
import fr.greyc.pantheon.ariane.model.graph.element.VirtualItem;
import fr.greyc.pantheon.ariane.model.graph.slot.SlotModel;
import fr.greyc.pantheon.ariane.view.graph.XYConstraint;
import fr.greyc.pantheon.ariane.view.graph.XYLayout;

import java.awt.Point;

/**
 * A <code>SubGraph</code> holds configuration of a complex graph
 * item such as <code>MacroOperator</code> or <code>ControlStructureFor</code>.
 * Its inputs, parameters and outputs are held by a <code>VirtualOperator</code>.
 */
public final class SubGraphPresenter extends GraphPresenter {

    public SubGraphPresenter( GraphItemModel parent, String name, int inputDataSlotNumber, int inputParameterSlotNumber,
                              int outputDataSlotNumber, int intermediateDataSlotNumber, int intermediateParamaterSlotNumber ) {
        super(new SubGraph(parent));
        // TODO Releve du modele
        setName(name);
        VirtualItem _virtualOperator = new VirtualItem(this, name, inputDataSlotNumber, inputParameterSlotNumber, outputDataSlotNumber, intermediateDataSlotNumber, intermediateParamaterSlotNumber);
        ((SubGraph) getModel()).setVirtualItem(_virtualOperator);

        // TODO Releve de la vue
        addItemAtAbsolutePosition(_virtualOperator, new Point(0, 0), true);
        for (int i = 0; i < inputParameterSlotNumber; i++) {
            add(_virtualOperator.getVirtualInputParameterSlots().get(i), XYLayout.Orientation.NORTHWEST, i);
        }

        add(_virtualOperator.getVirtualInputMaskSlot(), XYLayout.Orientation.NORTHEAST, inputParameterSlotNumber);
        for (int i = 0; i < inputDataSlotNumber; i++) {
            add(_virtualOperator.getVirtualInputDataSlots().get(i), XYLayout.Orientation.WEST, i);
        }
        for (int i = 0; i < outputDataSlotNumber; i++) {
            add(_virtualOperator.getVirtualOutputDataSlots().get(i), XYLayout.Orientation.EAST, i);
        }
        add(_virtualOperator.getVirtualOutputResultSlot(), XYLayout.Orientation.SOUTH, 0);
        for (int i = 0; i < intermediateDataSlotNumber; i++) {
            add(_virtualOperator.getVirtualIntermediateDataSlots(i), XYLayout.Orientation.SOUTHEAST, i);
        }
        for (int i = 0; i < intermediateParamaterSlotNumber; i++) {
            add(_virtualOperator.getVirtualIntermediateResultSlot(i), XYLayout.Orientation.SOUTHWEST, i);
        }
    }

    @Override
    public GraphItemModel getSuperGraphItem() {
        return ((SubGraph) getModel()).getParent();
    }

    @Override
    public GraphPresenter getRoot() {
        return getSuperGraphItem().getGraph().getRoot();
    }

    @Override
    public boolean isRoot() {
        return false;
    }

    public VirtualItem getVirtualItem() {
        return ((SubGraph) getModel()).getVirtualItem();
    }

    @Override
    public boolean isModified() {
        return getRoot().isModified();
    }

    @Override
    public void setModified( boolean value ) {
        getRoot().setModified(value);
    }

    public void add( SlotModel slot, Object alignment, int index ) {
        XYConstraint constraint = new XYConstraint((XYLayout.Orientation) alignment, index);
        getView().add(slot.getView(), constraint);
    }

    public void remove( SlotModel slot ) {
        getView().remove(slot.getView());
    }

    public void showContent() {
        Ariane.getPresenter().addGraph(this);
    }

    @Override
    public boolean close() {
        closeFrame();
        return true;
    }

    @Override
    public void save() {
        getRoot().save();
    }

    @Override
    public void saveAs() {
        getRoot().saveAs();
    }
}
