/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane.presenter;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.view.explorer.ItemTreeModel;
import fr.greyc.pantheon.ariane.view.explorer.MutableTreeItem;

import java.io.File;
import java.util.Collections;
import java.util.List;

/**
 * Controls the tree editors (operator and subroutine).
 */
public final class TreeEditorPresenter {
    private final ItemTreeModel _itemTreeModel;

    TreeEditorPresenter( ItemTreeModel itemTree ) {
        _itemTreeModel = itemTree;
    }

    public static boolean saveSubroutineTree( List<TreeItem> subRoutineList ) {
        if (!subRoutineList.isEmpty()) {
            XMLElement element = new XMLElement("SubroutineLibrary");
            subRoutineList.forEach(item -> element.addChildren(item.convertToXMLTree()));
            List<XMLElement> preferences = Collections.singletonList(element);
            File file = new File(ArianeSettings.getSubroutineFilename());
            return UserPreferences.saveInSettingFile(file, preferences);
        }
        return false;
    }

    public void setAndSaveUserOperatorTree( MutableTreeItem node ) {
        setUserOperatorTree(node);
        saveUserOperatorTree();
    }

    public void setAndSaveSubroutineTree( MutableTreeItem node ) {
        setSubroutineTree(node);
        saveSubroutineTree(_itemTreeModel.getSubroutineTreeAsList());
    }

    private void setUserOperatorTree( MutableTreeItem node ) {
        _itemTreeModel.setUserOperatorTree(node);
    }

    private void saveUserOperatorTree() {
        List<TreeItem> operatorList = _itemTreeModel.getUserOperatorTreeAsList();
        if (!operatorList.isEmpty()) {
            XMLElement element = new XMLElement("OperatorLibrary");
            operatorList.forEach(item -> element.addChildren(item.convertToXMLTree()));
            List<XMLElement> preferences = Collections.singletonList(element);
            File file = new File(ArianeSettings.getPersonalOperatorFilename());
            UserPreferences.saveInSettingFile(file, preferences);
        }
    }

    private void setSubroutineTree( MutableTreeItem node ) {
        _itemTreeModel.setSubroutineTree(node);
    }
}
