/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.presenter;

import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;

/**
 * UndoRedoManager manages a list of UndoableEdits,
 * providing a way to undo or redo the appropriate actions.
 */
final class UndoRedoManager extends UndoManager {
    private CompoundEdit _edit;

    UndoRedoManager() {
        setLimit(UserPreferences.getHistorySize());
    }

    void validateAction( UndoableEdit action ) {
        if (_edit == null) {
            _edit = new CompoundEdit();
        }
        _edit.addEdit(action);
    }

    void startActionEdition() {
        _edit = new CompoundEdit();
    }

    void endActionEdition( GraphPresenter graph ) {
        if (_edit != null && _edit.isSignificant()) {
            _edit.end();
            addEdit(_edit);
        }
        graph.setModified(true);
        Ariane.getPresenter().updateMenu();
    }

    void clearCurrentListOfActions() {
        _edit = null;
    }

    @Override
    public void undo() {
        try {
            super.undo();
            Ariane.getPresenter().updateMenu();
        } catch (CannotUndoException e) {
            Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("Graph.history.error1"));
        }
    }

    @Override
    public void redo() {
        try {
            super.redo();
            Ariane.getPresenter().updateMenu();
        } catch (CannotRedoException e) {
            Ariane.getPresenter().printErrorMessage(ArianeSettings.getString("Graph.history.error1"));
        }
    }
}
