/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;
import javax.swing.border.EtchedBorder;

/**
 * A splash screen is a graphical control element consisting of a window containing an image,
 * a logo, and the current version of the software used during the initialization of the software.
 */
public final class SplashScreen extends JWindow {
    private final ProgressBar _progressBar = new ProgressBar();

    public SplashScreen( final ImageIcon splashImage ) {
        final JLabel splashButton = new JLabel("");
        if (splashImage != null) {
            final int imgWidth = splashImage.getIconWidth();
            final int imgHeight = splashImage.getIconHeight();
            final Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
            setLocation(scrSize.width / 2 - imgWidth / 2, scrSize.height / 2 - imgHeight / 2);
            splashButton.setIcon(addReleaseNumberOntoImage(splashImage));
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        getContentPane().setLayout(new BorderLayout(0, 0));
        getContentPane().add(splashButton, BorderLayout.CENTER);
        getContentPane().add(_progressBar, BorderLayout.SOUTH);
        setAlwaysOnTop(true);
        pack();
    }

    public void showProgress( final String message, final int percent ) {
        _progressBar.showStatus(message);
        _progressBar.showProgress(percent);
    }

    private ImageIcon addReleaseNumberOntoImage( final ImageIcon imageIcon ) {
        final Image image = imageIcon.getImage();
        final int width = image.getWidth(null);
        final int height = image.getHeight(null);
        final BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        bufferedImage.createGraphics().drawImage(image, 0, 0, this);
        bufferedImage.getGraphics().drawString("Version: " + ArianeSettings.VERSION_ID, 80, 220);

        return new ImageIcon(bufferedImage);
    }

    private static final class ProgressBar extends JPanel {
        private final JLabel _messageText = new JLabel();
        private final JProgressBar _progressIndicator = new JProgressBar();

        ProgressBar() {
            _progressIndicator.setMinimum(0);
            _progressIndicator.setMaximum(100);
            _progressIndicator.setMinimumSize(new Dimension(100, 20));
            _progressIndicator.setSize(new Dimension(100, 20));

            _messageText.setMinimumSize(new Dimension(300, 20));
            _messageText.setSize(new Dimension(300, 20));
            _messageText.setFont(_messageText.getFont().deriveFont(Font.BOLD, 9f));
            _messageText.setForeground(Color.black);

            setLayout(new BorderLayout());
            setBorder(new EtchedBorder(EtchedBorder.LOWERED));
            add(_messageText, BorderLayout.CENTER);
            add(_progressIndicator, BorderLayout.EAST);
        }

        void showStatus( final String message ) {
            _messageText.setText(message);
            paintImmediately(getBounds());
        }

        void showProgress( final int percent ) {
            _progressIndicator.setValue(percent);
        }
    }
}
