/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.awt.Color;
import java.awt.FlowLayout;
import java.util.Map;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;

final class Toolbar extends JPanel {
    private final JButton _newB;
    private final JButton _openB;
    private final JButton _saveB;
    private final JButton _saveAsB;
    private final JButton _printB;

    private final JButton _undoB;
    private final JButton _redoB;
    private final JButton _cutB;
    private final JButton _copyB;
    private final JButton _pasteB;
    private final JButton _eraseB;

    private final JButton _preferencesB;
    private final JButton _operatorTreeB;
    private final JButton _subroutineTreeB;

    private final JButton _executeB;
    private final JButton _generateB;
    private final JButton _stopB;

    private final JButton _helpB;

    Toolbar( Map<String, Action> actions ) {
        super(new FlowLayout(FlowLayout.LEFT));
        super.setBorder(new LineBorder(Color.BLACK));
        JToolBar toolbar = buildToolBar();
        _newB = (JButton) toolbar.add(createButton(actions.get("new")));
        _openB = (JButton) toolbar.add(createButton(actions.get("open")));
        _saveB = (JButton) toolbar.add(createButton(actions.get("save")));
        _saveAsB = (JButton) toolbar.add(createButton(actions.get("saveAs")));
        toolbar.addSeparator();
        _printB = (JButton) toolbar.add(createButton(actions.get("print")));
        toolbar.addSeparator();
        _preferencesB = (JButton) toolbar.add(createButton(actions.get("preferences")));
        _operatorTreeB = (JButton) toolbar.add(createButton(actions.get("operatorTree")));
        _subroutineTreeB = (JButton) toolbar.add(createButton(actions.get("subroutineTree")));
        toolbar.addSeparator();
        _undoB = (JButton) toolbar.add(createButton(actions.get("undo")));
        _redoB = (JButton) toolbar.add(createButton(actions.get("redo")));
        _copyB = (JButton) toolbar.add(createButton(actions.get("copy")));
        _cutB = (JButton) toolbar.add(createButton(actions.get("cut")));
        _pasteB = (JButton) toolbar.add(createButton(actions.get("paste")));
        _eraseB = (JButton) toolbar.add(createButton(actions.get("delete")));
        toolbar.addSeparator();
        _executeB = (JButton) toolbar.add(createButton(actions.get("exec")));
        _stopB = (JButton) toolbar.add(createButton(actions.get("stop")));
        toolbar.addSeparator();
        _generateB = (JButton) toolbar.add(createButton(actions.get("generate")));
        toolbar.addSeparator();
        _helpB = (JButton) toolbar.add(createButton(actions.get("help")));

        super.add(toolbar);
    }

    private JToolBar buildToolBar() {
        JToolBar toolbar = new JToolBar();
        toolbar.setFloatable(false);
        toolbar.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        toolbar.setBorderPainted(true);
        toolbar.setOpaque(true);
        toolbar.setBackground(toolbar.getBackground().darker());

        return toolbar;
    }

    private JButton createButton( Action action ) {
        JButton button = new JButton(action);
        button.setBorderPainted(false);
        return button;
    }

    void setInternationalText() {
        _newB.setToolTipText(ArianeSettings.getString("Menu.file.new"));
        _openB.setToolTipText(ArianeSettings.getString("Menu.file.open"));
        _saveB.setToolTipText(ArianeSettings.getString("Menu.file.save"));
        _saveAsB.setToolTipText(ArianeSettings.getString("Menu.file.saveas"));
        _printB.setToolTipText(ArianeSettings.getString("Menu.file.print"));
        _undoB.setToolTipText(ArianeSettings.getString("Menu.edit.undo"));
        _redoB.setToolTipText(ArianeSettings.getString("Menu.edit.redo"));
        _copyB.setToolTipText(ArianeSettings.getString("Menu.edit.copy"));
        _cutB.setToolTipText(ArianeSettings.getString("Menu.edit.cut"));
        _pasteB.setToolTipText(ArianeSettings.getString("Menu.edit.paste"));
        _eraseB.setToolTipText(ArianeSettings.getString("Menu.edit.delete"));
        _preferencesB.setToolTipText(ArianeSettings.getString("Menu.options.preferences"));
        _operatorTreeB.setToolTipText(ArianeSettings.getString("Menu.options.operators"));
        _subroutineTreeB.setToolTipText(ArianeSettings.getString("Menu.options.subroutines"));
        _executeB.setToolTipText(ArianeSettings.getString("Menu.graph.exec"));
        _generateB.setToolTipText(ArianeSettings.getString("Menu.graph.generate"));
        _helpB.setToolTipText(ArianeSettings.getString("Menu.help.help"));
        _stopB.setToolTipText(ArianeSettings.getString("Menu.graph.stop"));
    }
}
