/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.presenter.Presenter;
import fr.greyc.pantheon.swingext.FadingEffects;

import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;

/**
 * An Explorer is a tree of operator and subroutine items
 * the user can select to compose a graph.
 * It also provides a toolbar for custom facilities (search, documentation manual).
 */
public final class Explorer extends JPanel {
    private final Presenter _presenter;
    private final ItemTreeView _itemTreeView;
    private JMenu _explorerMenu;
    private Action _searchAction;
    private JMenuItem _searchMenuItem;
    private JButton _searchButton;
    private Action _documentationAction;
    private JMenuItem _documentationMenuItem;
    private JButton _documentationButton;

    public Explorer( Presenter presenter, ItemTreeView tree ) {
        super(new BorderLayout());
        _presenter = presenter;
        _itemTreeView = tree;
        initContents(tree);
    }

    public JMenu getMenu() {
        _explorerMenu = new JMenu();
        _explorerMenu.setMnemonic('R');

        _searchMenuItem = new JMenuItem(_searchAction);
        _searchMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        _explorerMenu.add(_searchMenuItem);

        _documentationMenuItem = new JMenuItem(_documentationAction);
        _documentationMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        _explorerMenu.add(_documentationMenuItem);

        return _explorerMenu;
    }

    public void changeLocale() {
        _searchMenuItem.setText(ArianeSettings.getString("Explorer.menuitem.search"));
        _searchButton.setToolTipText(ArianeSettings.getString("Explorer.menuitem.search"));
        _documentationMenuItem.setText(ArianeSettings.getString("Explorer.menuitem.index"));
        _documentationButton.setToolTipText(ArianeSettings.getString("Explorer.menuitem.index"));
        _explorerMenu.setText(ArianeSettings.getString("Explorer.menu.title"));
        _itemTreeView.changeLocale();
    }

    private void initContents( ItemTreeView tree ) {
        add(createToolbar(), BorderLayout.NORTH);
        JScrollPane scrollPane = new JScrollPane(tree);
        scrollPane.getVerticalScrollBar().setUnitIncrement(10);
        add(scrollPane, BorderLayout.CENTER);
    }

    private JToolBar createToolbar() {
        JToolBar toolbar = new JToolBar();
        toolbar.setFloatable(false);
        toolbar.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        toolbar.setBorderPainted(true);
        toolbar.setOpaque(false);

        _searchAction = new AbstractAction(null, ArianeSettings.getIcon("search.png")) {
            @Override
            public void actionPerformed( ActionEvent event ) {
                searchItemInTreeAction();
            }
        };
        _documentationAction = new AbstractAction(null, ArianeSettings.getIcon("index.png")) {
            @Override
            public void actionPerformed( ActionEvent event ) {
                // TODO mettre toutes les données de la library dans un objet Library ?
                _presenter.displayLibraryIndex();
            }
        };

        _searchButton = new JButton(_searchAction);
        _searchButton.setBorderPainted(false);
        toolbar.add(_searchButton);

        _documentationButton = new JButton(_documentationAction);
        _documentationButton.setBorderPainted(false);
        toolbar.add(_documentationButton);

        return toolbar;
    }

    private void searchItemInTreeAction() {
        FadingEffects.fadeIn(Ariane.getFrame());
        new SearchDialog(_itemTreeView).setVisible(true);
        FadingEffects.fadeOut(Ariane.getFrame());
    }
}
