/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;

import javax.swing.tree.DefaultMutableTreeNode;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A MutableTreeItem defines a node or a leaf in the tree.
 */
public final class MutableTreeItem extends DefaultMutableTreeNode {

    MutableTreeItem( Object itemContent, boolean allowsChildren ) {
        super(itemContent, allowsChildren);
    }

    MutableTreeItem() {
        super(null, true);
    }

    @Override
    public boolean isLeaf() {
        return !getAllowsChildren();
    }

    @Override
    public Object clone() {
        MutableTreeItem root = (MutableTreeItem) super.clone();
        Object x = getUserObject();
        if (x instanceof TreeItem) {
            try {
                x = ((TreeItem) x).clone();
            } catch (CloneNotSupportedException e) {
                Logger.getLogger(Ariane.class.getName()).log(Level.INFO, "Clone error.", e);
            }
        }
        root.setUserObject(x);
        for (int i = 0; i < getChildCount(); i++) {
            root.insert((MutableTreeItem) (((MutableTreeItem) getChildAt(i)).clone()), i);
        }
        return root;
    }

    public TreeItem getTreeItemObject() {
        return (TreeItem) getUserObject();
    }

    MutableTreeItem getNodeWithSpecifiedName( String name ) {
        for (int j = 0; j < getChildCount(); j++) {
            MutableTreeItem item = (MutableTreeItem) getChildAt(j);
            String itemName = item.toString();
            if (name.equalsIgnoreCase(itemName)) {
                return item;
            }
        }
        return null;
    }
}
