/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItemFactory;
import fr.greyc.pantheon.ariane.presenter.TreeEditorPresenter;
import fr.greyc.pantheon.swingext.GDialog;

import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
 * The class OperatorTreeEditor defines a dialog box
 * where users can modify the personal operators tree.
 */
public final class OperatorTreeEditor extends ItemTreeEditor {

    public OperatorTreeEditor( TreeEditorPresenter presenter, MutableTreeItem rootNode ) {
        super(ArianeSettings.getString("OperatorTreeEditor.window.title"), presenter, rootNode);
    }

    @Override
    protected JDialog editItem( GDialog dialog, MutableTreeItem currentNode, DefaultTreeModel treeModel ) {
        return new OperatorEditor(dialog, currentNode, treeModel);
    }

    @Override
    protected TreeItem createItem( String categoryName ) {
        // TODO : demander à la présentation ????
        TreeItem treeItem = TreeItemFactory.createItem(TreeItem.ItemType.OPERATOR);
        treeItem.setCategory(categoryName);
        return treeItem;
    }

    @Override
    protected boolean isNewActionEnable( TreePath path ) {
        return path != null && path.getPathCount() < 3;
    }

    @Override
    protected void addSpecificMenuItem( JPopupMenu popupMenu ) {
        JMenuItem item;
        item = new JMenuItem(ArianeSettings.getString("OperatorTreeEditor.addOperator"));
        item.addActionListener(event -> addNewItemBelowCurrentSelection());
        popupMenu.add(item);
    }

    @Override
    protected void storeInSettingFile( TreeEditorPresenter presenter, MutableTreeItem node ) {
        presenter.setAndSaveUserOperatorTree(node);
    }
}
