/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.swingext.GDialog;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * A SearchDialog is a dialog that allows
 * the user to select operators by name in the tree.
 */
final class SearchDialog extends GDialog {
    private final ItemTreeView _itemTreeView;

    public SearchDialog( ItemTreeView itemTreeView ) {
        super((JFrame) null, ArianeSettings.getString("SearchDialog.title"), true);
        _itemTreeView = itemTreeView;
        setResizable(false);
        displayContent();
        setLocationRelativeTo(Ariane.getFrame());
        setAlwaysOnTop(true);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing( WindowEvent event ) {
                cancelAction();
            }
        });
    }

    private void displayContent() {
        Box panel = Box.createVerticalBox();

        JPanel panel1 = new JPanel();
        panel.add(panel1);
        panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
        Box panel2 = Box.createVerticalBox();
        panel1.add(panel2);

        final JTextField textfieldWithTheSpecifiedName = new JTextField(30);
        JLabel label = new JLabel(ArianeSettings.getString("SearchDialog.text"));
        panel2.add(label);
        panel2.add(textfieldWithTheSpecifiedName);
        JPanel buttonPanel = new JPanel();

        JButton applyButton = new JButton(ArianeSettings.getString("SearchDialog.ok"), ArianeSettings.getIcon("gtk-apply.png"));
        applyButton.addActionListener(event -> applyAction(textfieldWithTheSpecifiedName));

        JButton cancelButton = new JButton();
        cancelButton.setAction(new AbstractAction(ArianeSettings.getString("SearchDialog.cancel"), ArianeSettings.getIcon("gtk-cancel.png")) {
            @Override
            public void actionPerformed( ActionEvent event ) {
                cancelAction();
            }
        });
        buttonPanel.add(applyButton);
        buttonPanel.add(cancelButton);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(panel1, BorderLayout.CENTER);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        setDefaultButton(applyButton);
        setCancelButton(cancelButton);
        pack();
        textfieldWithTheSpecifiedName.requestFocusInWindow();
    }

    private void cancelAction() {
        setVisible(false);
        dispose();
    }

    private void applyAction( JTextField textfieldWithTheSpecifiedName ) {
        String pattern = textfieldWithTheSpecifiedName.getText();
        if (!pattern.isEmpty()) {
            _itemTreeView.highlightItemsThatContainTheSpecifiedPattern(pattern);
        }
        setVisible(false);
        dispose();
    }
}
