/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph;

import java.awt.Point;

/**
 * A <code>XYConstraint</code> defines a constraint that can be specified
 * for locating a component in a XYLayout. A constraints can be either:
 * <ul>
 * <li>absolute location: x,y
 * <li>orientation: NORTHWEST, NORTHEAST, ..
 * </ul>
 */
public final class XYConstraint {
    private final int _rank;
    private XYLayout.Orientation _orientation;
    private Point _location;

    XYConstraint() {
        _rank = -1;
        _location = new Point(0, 0);
    }

    public XYConstraint( Point coordinates ) {
        _rank = -1;
        _location = new Point(coordinates.x, coordinates.y);
    }

    public XYConstraint( XYLayout.Orientation orientation, int rank ) {
        _orientation = orientation;
        _rank = rank;
    }

    boolean isAbsoluteCoordinates() {
        return _rank == -1;
    }

    public int getRank() {
        return _rank;
    }

    XYLayout.Orientation getOrientation() {
        return _orientation;
    }

    public Point getLocation() {
        return _location;
    }

    @Override
    public boolean equals( Object that ) {
        if (!(that instanceof XYConstraint)) {
            return false;
        }
        XYConstraint other = (XYConstraint) that;
        return other._location.equals(_location) && other._rank == _rank && other._orientation == _orientation;
    }

    @Override
    public int hashCode() {
        int code = 17;
        code = code * 37 + _orientation.ordinal();
        code = code * 37 + _rank;
        code = code * 37 + _location.x;
        code = code * 37 + _location.y;
        return code;
    }
}
