/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.element;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.slot.ParameterSlot;
import fr.greyc.pantheon.swingext.FadingEffects;
import fr.greyc.pantheon.swingext.GDialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

public abstract class ControlItemView extends GraphElementView {
    private boolean _validatedPrototype;

    /////
    ///// P R O P E R T I E S
    /////
    public final Prototype askProperties( String name, int defaultInputImageNumber, int defaultParameterNumber, int defaultOutputImageNumber ) {
        final GDialog dialog = new GDialog(null, getModel().getTagName(), true);
        dialog.setAlwaysOnTop(true);
        dialog.setResizable(false);
        _validatedPrototype = false;

        String nickName;
        if (name == null) {
            nickName = getName();
        } else {
            nickName = name;
        }
        final JTextField textFieldname = new JTextField(nickName, 10);
        JPanel jPanel = new JPanel(new GridBagLayout());
        jPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ArianeSettings.getString("MacroOperator.borderTitle")));

        GridBagConstraints constraint = new GridBagConstraints();
        constraint.fill = GridBagConstraints.NONE;
        constraint.anchor = GridBagConstraints.WEST;
        constraint.ipadx = 8;
        constraint.ipady = 8;
        constraint.gridy = 0;

        if (name != null) {
            constraint.gridy++;
            constraint.gridx = 0;
            jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.name")), constraint);
            constraint.gridx = 1;
            jPanel.add(textFieldname, constraint);
        }

        final JSpinner textFieldiP;
        if (defaultParameterNumber >= 0) {
            final SpinnerModel spModel = new SpinnerNumberModel(defaultParameterNumber, 0, 10, 1);
            textFieldiP = new JSpinner(spModel);
            constraint.gridy++;
            constraint.gridx = 0;
            jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.parameternumber")), constraint);
            constraint.gridx = 1;
            jPanel.add(textFieldiP, constraint);
        } else {
            textFieldiP = null;
        }
        final JSpinner textFieldiIm;
        if (defaultInputImageNumber >= 0) {
            final SpinnerModel spModel = new SpinnerNumberModel(defaultInputImageNumber, 1, 10, 1);
            textFieldiIm = new JSpinner(spModel);
            constraint.gridy++;
            constraint.gridx = 0;
            jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.inputnumber")), constraint);
            constraint.gridx = 1;
            jPanel.add(textFieldiIm, constraint);
        } else {
            textFieldiIm = null;
        }
        final JSpinner textFieldoIm;
        if (defaultOutputImageNumber >= 0) {
            final SpinnerModel spModel = new SpinnerNumberModel(defaultOutputImageNumber, 0, 10, 1);
            textFieldoIm = new JSpinner(spModel);
            constraint.gridy++;
            constraint.gridx = 0;
            jPanel.add(new JLabel(ArianeSettings.getString("MacroOperator.outputnumber")), constraint);
            constraint.gridx = 1;
            jPanel.add(textFieldoIm, constraint);
        } else {
            textFieldoIm = null;
        }

        JButton applyButton = new JButton(ArianeSettings.getString("PropertyViewer.ok"), ArianeSettings.getIcon("gtk-yes.png"));
        final Prototype prototype = new Prototype();
        applyButton.addActionListener(event -> {
            if (textFieldiIm != null) {
                prototype.setInputs((Integer) textFieldiIm.getValue());
            }
            if (textFieldiP != null) {
                prototype.setParameters((Integer) textFieldiP.getValue());
            }
            if (textFieldoIm != null) {
                prototype.setOutputs((Integer) textFieldoIm.getValue());
            }
            prototype.setName(textFieldname.getText());
            _validatedPrototype = true;
            dialog.setVisible(false);
        });

        JButton cancelButton = new JButton();
        cancelButton.setAction(new AbstractAction(ArianeSettings.getString("SubroutineEditor.cancel"),
                ArianeSettings.getIcon("gtk-cancel.png")) {
            @Override
            public final void actionPerformed( ActionEvent event ) {
                _validatedPrototype = false;
                dialog.setVisible(false);
            }
        });

        dialog.getContentPane().setLayout(new BorderLayout(5, 5));
        dialog.getContentPane().add(jPanel, BorderLayout.CENTER);
        JPanel panel = new JPanel();
        panel.add(applyButton);
        panel.add(cancelButton);
        dialog.setDefaultButton(applyButton);
        dialog.setCancelButton(cancelButton);
        dialog.getContentPane().add(panel, BorderLayout.SOUTH);
        dialog.pack();
        dialog.setLocationRelativeTo(Ariane.getFrame());
        FadingEffects.fadeIn(Ariane.getFrame());
        dialog.setVisible(true);
        FadingEffects.fadeOut(Ariane.getFrame());

        if (_validatedPrototype) {
            return prototype;
        } else {
            return null;
        }
    }

    /////
    ///// P R I N T I N G
    /////
    @Override
    public final void printComponent( Graphics gc ) {
        super.printComponent(gc);
        gc.setColor(Color.black);
        gc.setFont(UserPreferences.getDefaultFont());
        FontMetrics fm = gc.getFontMetrics();
        int row = fm.getHeight();
        // Name
        super.centerText(getModel().getNickName(), gc, row, false);

        // Print each parameters.
        // Print the value only of if the parameter is not connected.
        for (int i = 0; i < getModel().getInputParameterSlots().size(); i++) {
            ParameterSlot param = getModel().getInputParameterSlots().get(i);
            row += fm.getHeight() + 1;
            if (param.isConnected()) {
                super.centerText("$" + i + ":<linked>", gc, row, true);
            } else {
                super.centerText("$" + i + ":" + param.getValue(), gc, row, true);
            }
        }
        row += fm.getHeight() + 2;
        if (!getFormula().isEmpty()) {
            super.centerText(getFormula(), gc, row, true);
        }
    }

    public static class Prototype {

        private String _name;
        private int _inputs;
        private int _outputs;
        private int _parameters;

        private void setInputs( Integer number ) {
            _inputs = number;
        }

        private void setParameters( Integer number ) {
            _parameters = number;
        }

        private void setOutputs( Integer number ) {
            _outputs = number;
        }

        private void setName( String name ) {
            _name = name;
        }

        public String getName() {
            return _name;
        }

        public int getInputs() {
            return _inputs;
        }

        public int getParameters() {
            return _parameters;
        }

        public int getOutputs() {
            return _outputs;
        }
    }
}
