/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.element;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.element.DataFile;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel.Status;
import fr.greyc.pantheon.ariane.model.graph.element.etc.ExternalViewerProcessBuilder;
import fr.greyc.pantheon.ariane.view.graph.element.etc.DataFileColor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JTextField;

public abstract class DataFileView extends GraphElementView {
    private final String _windowTitle;

    DataFileView( String windowTitle ) {
        _windowTitle = windowTitle;
    }

    protected abstract void resetButtonText( JButton button );

    public abstract void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory );

    public final void displayDataFileContents() { // TODO deplacer dans Presenter ?
        String currentFilename = ((DataFile) getModel()).getCurrentFilename();
        if (currentFilename == null) {
            currentFilename = getModel().getBinary();
        }
        if (currentFilename == null) {
            return;
        }

        File file = new File(currentFilename);
        if (getModel().getExecutionStatus() == Status.EXECUTED && file.exists()) {
            ExternalViewerProcessBuilder pb = new ExternalViewerProcessBuilder(file);
            pb.start();
        }
    }

    @Override
    public final void printComponent( Graphics gc ) {
        super.printComponent(gc);

        gc.setColor(new Color(0, 0, 0));
        gc.setFont(UserPreferences.getDefaultFont());

        int row = gc.getFontMetrics().getHeight();

        super.centerText(ArianeSettings.getString(_windowTitle), gc, row, false);
    }

    @Override
    public final Color getBackgroundColor() {
        return DataFileColor.getColor();
    }

    static final class OpenFileChooser extends JFileChooser implements ComponentListener {

        private final Rectangle _bounds;

        OpenFileChooser( Rectangle bounds ) {
            super.setFileSelectionMode(JFileChooser.FILES_ONLY);
            _bounds = bounds;
        }

        @Override
        protected JDialog createDialog( Component parent ) {
            JDialog dialog = super.createDialog(parent);
            if (_bounds.getWidth() > 0) {
                dialog.setBounds(_bounds);
            }
            dialog.addComponentListener(this);
            return dialog;
        }

        @Override
        public void componentResized( ComponentEvent e ) {
            _bounds.setSize(e.getComponent().getSize());
        }

        @Override
        public void componentMoved( ComponentEvent e ) {
            _bounds.setLocation(e.getComponent().getLocation());
        }

        @Override
        public void componentShown( ComponentEvent e ) {
        }

        @Override
        public void componentHidden( ComponentEvent e ) {
        }
    }
}
