/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.element;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.element.DataFile;
import fr.greyc.pantheon.ariane.model.graph.element.LoaderItem;
import fr.greyc.pantheon.ariane.view.command.ChangeDatafileEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;
import fr.greyc.pantheon.swingext.FileTypeViewer;
import fr.greyc.pantheon.swingext.ImageFilePreviewer;

import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;

public final class LoaderItemView extends DataFileView {
    private static String _currentLoadingFolder;
    private static final Rectangle BOUNDS = new Rectangle();

    private final JButton _jbuttonSelectFile = new JButton();
    private final JTextField _binaryEditor = new JTextField("");

    public LoaderItemView() {
        super("loader");
    }

    private void selectDataFile() {
        JFileChooser fileChooser = new OpenFileChooser(BOUNDS);
        fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        fileChooser.setFileView(new FileTypeViewer());
        fileChooser.setAccessory(new ImageFilePreviewer(fileChooser));

        final String currentFile = getModel().getBinary();
        if (currentFile == null || currentFile.trim().isEmpty() || !(new File(currentFile).exists())) {
            if (_currentLoadingFolder == null) {
                _currentLoadingFolder = UserPreferences.getCurrentDirectory();
            }
            fileChooser.setCurrentDirectory(new File(_currentLoadingFolder));
            fileChooser.setSelectedFile(null);
        } else {
            File file = new File(currentFile);
            fileChooser.setSelectedFile(file);
        }
        if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            String path = fileChooser.getSelectedFile().getPath();
            ((LoaderItem) getModel()).setCurrentFilename(null);
            _currentLoadingFolder = fileChooser.getSelectedFile().getParent();
            changeBinaryName(getNameEditor(), path, true);
        }
    }

    private JMenuItem createSelectFileItem() {
        JMenuItem item;
        item = new JMenuItem(ArianeSettings.getString("PopupMenu.selectFile"));
        item.addActionListener(event -> selectDataFile());
        return item;
    }

    private JMenuItem createExecuteDirectoryItem() {
        JMenuItem item;
        item = new JMenuItem(ArianeSettings.getString("PopupMenu.executeDirectory"));
        item.addActionListener(event -> getGraph().executeFolder((LoaderItem) getModel()));
        return item;
    }

    @Override
    protected void onMouseDoubleClicked( MouseEvent event ) {
        selectDataFile();
    }

    @Override
    protected void resetButtonText( JButton button ) {
        button.setText(ArianeSettings.getString("PropertyViewer.load") + "...");
    }

    /////
    ///// M E N U
    /////
    @Override
    protected JPopupMenu initPopup() {
        JPopupMenu popupMenu = new JPopupMenu();
        popupMenu.add(createExecuteMenuItem());
        popupMenu.add(createExecuteDirectoryItem());
        popupMenu.add(createResetMenuItem());
        popupMenu.addSeparator();
        popupMenu.add(createCopyMenuItem());
        popupMenu.add(createCutMenuItem());
        popupMenu.add(createDeleteMenuItem());
        popupMenu.addSeparator();
        popupMenu.add(createSelectFileItem());
        return popupMenu;
    }

    @Override
    public JComponent getBinaryEditor() {
        return null;
    }

    @Override
    public void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory ) {
        JButton button = _jbuttonSelectFile;

        getModel().resetAll();
        final String oldBinary = getModel().getBinary();
        getModel().setBinary(binary);

        if (getName().equals(getModel().getNickName())
                || oldBinary != null && DataFile.getBaseFileName(oldBinary).equals(getModel().getNickName())) {
            if (binary == null) {
                getModel().setNickName(null);
            } else {
                getModel().setNickName(DataFile.getBaseFileName(binary));
            }
        }
        textfield1.setText(getModel().getNickName());
        _binaryEditor.setText(getModel().getBinary());
        if (button != null) {
            if (binary == null) {
                resetButtonText(button);
            } else {
                button.setText(DataFile.getBaseFileName(binary));
            }
        }

        getModel().updateName();
        getGraph().setModified(true);
        if (addInHistory) {
            getGraph().validateOneAction(new ChangeDatafileEdit((DataFile) this.getModel(), textfield1, binary, oldBinary));
        }
    }

    public void setButtonEnabled( boolean flag ) {
        _jbuttonSelectFile.setEnabled(flag);
    }

    @Override
    public List<PropertyPanel.PropertyCouple> getSpecificProperties() {
        String name;
        String binary = getModel().getBinary();
        if (binary == null) {
            name = "";
        } else {
            File file = new File(binary);
            name = file.getName();
        }

        if (name.isEmpty()) {
            resetButtonText(_jbuttonSelectFile);
        } else {
            _jbuttonSelectFile.setText(name);
        }
        _jbuttonSelectFile.addActionListener(event -> selectDataFile());
        ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<>();
        properties.add(new PropertyPanel.PropertyCouple(ArianeSettings.getString("PropertyViewer.fileName"), _jbuttonSelectFile));
        return properties;
    }
}
