/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.element;

import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.view.PropertyPanel;
import fr.greyc.pantheon.ariane.view.graph.element.etc.NoteColor;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.StringTokenizer;

import javax.swing.JPopupMenu;
import javax.swing.JTextArea;

public final class NoteElementView extends GraphElementView {
    private static final transient PropertyPanel BLANK_PROPERTY_PANEL = new PropertyPanel();
    private JTextArea _noteText;

    @Override
    public void draw() {
        removeAll();
        setBackground(getBackgroundColor());
        setToolTipText(toString());

        GridBagConstraints constraint = new GridBagConstraints();
        constraint.insets = new Insets(5, 2, 2, 2);
        constraint.gridheight = 1;
        constraint.gridwidth = 1;
        constraint.gridy++;
        constraint.gridx = 0;
        constraint.anchor = GridBagConstraints.CENTER;
        if (_noteText == null) {
            _noteText = new JTextArea(getFormula(), 1, 2);
            _noteText.setFont(UserPreferences.getDefaultFont());
            _noteText.setBackground(Color.WHITE);
            _noteText.setForeground(Color.BLACK);
            _noteText.setTabSize(2);
        }
        _noteText.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed( MouseEvent event ) {
                if (isSelected()) {
                    _noteText.getCaret().setVisible(true);
                } else {
                    onMouseReleased(event);
                }
            }
        });
        _noteText.addFocusListener(this);
        _noteText.setEditable(true);
        _noteText.getCaret().setVisible(false);
        constraint.gridwidth = GridBagConstraints.REMAINDER;
        constraint.gridy++;
        constraint.gridx = 0;
        add(_noteText, constraint);
    }

    @Override
    public void printComponent( Graphics graphicContext ) {
        super.printComponent(graphicContext);
        FontMetrics fontMetric = graphicContext.getFontMetrics();
        int row = fontMetric.getHeight();
        super.centerText(getName(), graphicContext, row, false);
        row += fontMetric.getHeight() + 4;
        graphicContext.setFont(UserPreferences.getDefaultFont());
        String formula = getFormula();
        if (!formula.isEmpty()) {
            StringTokenizer stringTokenizer = new StringTokenizer(formula, "\n");
            while (stringTokenizer.hasMoreTokens()) {
                graphicContext.drawString(stringTokenizer.nextToken(), 5, row);
                row += fontMetric.getHeight();
            }
        }
    }

    @Override
    protected void onMouseDoubleClicked( MouseEvent event ) {
        displayDocumentationManual();
    }

    @Override
    protected void focusGainedHook() {
        _noteText.moveCaretPosition(_noteText.getText().length());
    }

    @Override
    public void focusLost( FocusEvent event ) {
        if (event.getSource() == _noteText) {
            String newFormula = _noteText.getText();
            if (isShowing()) {
                getPresenter().updateFormula(_noteText, newFormula);
            }
        }
        _noteText.getCaret().setVisible(false);
        _noteText.setCaretPosition(0);
    }

    @Override
    protected PropertyPanel createPropertyPanel() {
        return BLANK_PROPERTY_PANEL;
    }

    @Override
    protected Color getBackgroundColor() {
        return NoteColor.getColor();
    }

    @Override
    protected JPopupMenu initPopup() {
        JPopupMenu popupMenu = new JPopupMenu();
        popupMenu.add(createDeleteMenuItem());
        return popupMenu;
    }
}
