/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.element;

import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.view.command.ChangeTextEdit;

import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

public final class ScriptItemView extends DefaultControlElementView {
    private JTextArea _scriptText;
    private boolean _visible;

    @Override
    protected JComponent createCommentEditor() {
        // Override the class in order to refresh
        // the graph when the text area is modified.
        final JTextArea scriptText = new JTextArea() {
            @Override
            public void repaint() {
                getGraph().getView().repaint();
            }
        };
        scriptText.setFont(UserPreferences.getDefaultFont());
        scriptText.setBackground(Color.WHITE);
        scriptText.setForeground(Color.BLACK);
        scriptText.setDocument(new ScriptDocument());
        scriptText.setTabSize(2);

        // Mouse events.
        scriptText.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased( MouseEvent event ) {
                if (getModel().isSelected()) {
                    scriptText.getCaret().setVisible(true);
                } else {
                    onMouseReleased(event);
                }
            }
        });

        scriptText.addFocusListener(this);
        scriptText.setEditable(true);
        scriptText.getCaret().setVisible(false);
        scriptText.setVisible(false);
        _visible = true;
        _scriptText = scriptText;
        return scriptText;
    }

    @Override
    protected void focusGainedHook() {
        _scriptText.moveCaretPosition(_scriptText.getText().length());
    }

    @Override
    public void focusLost( FocusEvent event ) {
        if (event.getSource() == _scriptText) {
            String newFormula = _scriptText.getText();
            if (!newFormula.equals(getFormula()) && isShowing()) {
                getGraph().validateOneAction(new ChangeTextEdit(getModel(), _scriptText, _scriptText.getText(), getFormula()));
                setFormula(_scriptText.getText());
                getGraph().setModified(true);
            }
        }
        _scriptText.getCaret().setVisible(false);
        _scriptText.setCaretPosition(0);
    }

    @Override
    protected void onMouseDoubleClicked( MouseEvent event ) {
        _visible = !_visible;
        _scriptText.setVisible(_visible);
    }

    static class ScriptDocument extends PlainDocument {

        @Override
        public final void insertString( int offs, String str, AttributeSet attributeSet ) throws BadLocationException {
            final StringBuilder read = new StringBuilder(str);
            final StringBuilder filtered = new StringBuilder(getText(0, offs));
            StringBuilder inserted = new StringBuilder();

            for (int i = 0; i < read.length(); i++) {
                if (read.charAt(i) == '\n' && read.length() == 1) {
                    String indentation = "";
                    for (int k = filtered.length() - 1; k >= 0; k--) {
                        if (filtered.charAt(k) == '\n') {
                            for (int l = k + 1; l < filtered.length(); l++) {
                                if (filtered.charAt(l) == ' ' || filtered.charAt(l) == '\t') {
                                    indentation += filtered.charAt(l);
                                } else {
                                    break;
                                }
                            }
                            break;
                        }
                    }
                    String toBeInserted = read.charAt(i) + indentation;
                    inserted.append(toBeInserted);
                    filtered.append(toBeInserted);
                } else {
                    inserted.append(read.charAt(i));
                    filtered.append(read.charAt(i));
                }
            }
            super.insertString(offs, inserted.toString(), attributeSet);
        }
    }
}
