/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.element;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.UserPreferences;
import fr.greyc.pantheon.ariane.view.command.ChangeViewerCommandEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;

public final class ViewerElementView extends DataFileView {
    private final JTextField _commandTextfield = new JTextField();

    public ViewerElementView() {
        super("viewer");
    }

    @Override
    protected void resetButtonText( JButton button ) {
        // Do nothing
    }

    @Override
    protected void onMouseDoubleClicked( MouseEvent event ) {
        if (getPresenter().isExecuted()) { // TODO Mettre totalement dans presenter : faire un presenter par type d'item.
            displayDataFileContents();
        }
    }

    @Override
    public void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory ) {
        // Empty.
    }

    @Override
    protected JPopupMenu initPopup() {
        JPopupMenu popupMenu = new JPopupMenu();
        popupMenu.add(createExecuteMenuItem());
        popupMenu.add(createResetMenuItem());
        popupMenu.addSeparator();
        popupMenu.add(createCopyMenuItem());
        popupMenu.add(createCutMenuItem());
        popupMenu.add(createDeleteMenuItem());
        return popupMenu;
    }

    @Override
    public List<PropertyPanel.PropertyCouple> getSpecificProperties() {
//        getPresenter().getViewerCommandName();
        if (getModel().getBinary() == null) {
            _commandTextfield.setText(UserPreferences.getCommandVisu());
        } else {
            _commandTextfield.setText(getModel().getBinary());
        }
        _commandTextfield.addFocusListener(new FocusListener() {
            @Override
            public void focusLost( FocusEvent event ) {
                // getPresenter().changeVisuCommandName(((JTextField) (event.getSource())).getText().trim(), true);
                changeVisuCommandName(((JTextField) (event.getSource())).getText().trim(), true);
            }

            @Override
            public void focusGained( FocusEvent event ) {
                // Unused
            }
        });
        ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<>();
        properties.add(new PropertyPanel.PropertyCouple(ArianeSettings.getString("PropertyViewer.commandName"), _commandTextfield));
        return properties;
    }

    public void changeVisuCommandName( String newCommand, boolean addInHistory ) {
        // TODO in presenter mais alors faire autant de presenter que de type d'element.
        final String oldCommand = getModel().getBinary();
        _commandTextfield.setText(getPresenter().getNewCommandVisu(newCommand));
        // getView().setText();
        updatePropertyViewer();
        if (addInHistory) {
            getGraph().validateOneAction(new ChangeViewerCommandEdit(this, newCommand, oldCommand));
        }
    }
}
