/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.slot;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.graph.slot.ParameterSlot;
import fr.greyc.pantheon.ariane.view.PropertyPanel;
import fr.greyc.pantheon.ariane.view.graph.GraphView;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.ImageIcon;
import javax.swing.JTextField;

public final class InputParameterSlotView extends SlotView {
    private static final transient ImageIcon CONNECTED_ICON = ArianeSettings.getIcon("ValueSlotOn.png");
    private static final transient ImageIcon DISCONNECTED_ICON = ArianeSettings.getIcon("ValueSlotOff.png");
    private final PropertyPanel.TextField _textfield;

    public InputParameterSlotView() {
        setIcon(DISCONNECTED_ICON);
        _textfield = buildValueEditor();
    }

    @Override
    public void setConnectedRendering() {
        setIcon(CONNECTED_ICON);
    }

    @Override
    public void setDisconnectedRendering() {
        setIcon(DISCONNECTED_ICON);
    }

    private PropertyPanel.TextField buildValueEditor() {
        final PropertyPanel.TextField textfield = new PropertyPanel.TextField();
        textfield.setEnabled(true);
        textfield.setEditable(true);
        textfield.addFocusListener(new FocusListener() {
            @Override
            public void focusLost( FocusEvent event ) {
                ((ParameterSlot) getPresenter().getModel()).changeValue(textfield.getText(), true);
            }

            @Override
            public void focusGained( FocusEvent event ) {
                // empty
            }
        });
        return textfield;
    }

    @Override
    protected Color getLinkColor() {
        return GraphView.getValueLinkColor();
    }

    @Override
    public void printChildren( Graphics gc ) {
        if (getPresenter().getModel().isConnected()) {
            super.printChildren(gc);
        } else {
            gc.setFont(gc.getFont().deriveFont(6.0f));
            Rectangle area = getBounds();
            gc.setColor(new Color(255, 255, 255));
            gc.fillRect((int) area.getX(), (int) area.getY(), (int) area.getWidth(), (int) area.getHeight());
            gc.setColor(new Color(0, 0, 0));
            gc.drawString(toString(), (int) area.getX(), (int) (area.getY() + 2 + area.getHeight() / 2.0));
        }
    }

    @Override
    public Point getAnchorPoint() {
        Point slotRelativeLocation = getLocation();
        Point itemAbsoluteLocation = getPresenter().getModel().getGraphItem().getLocation();
        return new Point(
                (int) (slotRelativeLocation.getX() + getWidth() / 2.0 + itemAbsoluteLocation.getX()),
                (int) (slotRelativeLocation.getY() + itemAbsoluteLocation.getY() - 3));
    }

    @Override
    public JTextField getValueEditor() {
        return _textfield;
    }

    public void setEnabledTextfield( boolean enabled ) {
        _textfield.setEditable(enabled);
    }

    public void updateTextfield( String value ) {
        _textfield.setText(value);
    }
}
