/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 *  https://clouard.users.greyc.fr
 *
 *  This software is governed by the CeCILL-B  license under French law and
 *  abiding by the rules of distribution of free software.  You can  use,
 *  modify and/ or redistribute the software under the terms of the CeCILL-B
 *  license as circulated by CEA, CNRS and INRIA at the following URL
 *  "http://www.cecill.info".
 *
 *  In this respect, the user's attention is drawn to the risks associated
 *  with loading,  using,  modifying and/or developing or reproducing the
 *  software by the user in light of its specific status of free software,
 *  that may mean  that it is complicated to manipulate,  and  that  also
 *  therefore means  that it is reserved for developers  and  experienced
 *  professionals having in-depth computer knowledge. Users are therefore
 *  encouraged to load and test the software's suitability as regards their
 *  requirements in conditions enabling the security of their systems and/or
 *  data to be ensured and,  more generally, to use and operate it in the
 *  same conditions as regards security.
 *
 *  The fact that you are presently reading this means that you have had
 *  knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.greyc.pantheon.ariane.view.graph.slot;

import fr.greyc.pantheon.ariane.model.graph.slot.LinkModel;

import java.awt.*;
import java.awt.geom.Line2D;

public final class LinkView {
    private static final BasicStroke REGULAR_LINE_STROKE = new BasicStroke(2.0F, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    private static final BasicStroke SELECTED_LINE_STROKE = new BasicStroke(4.0F, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    private LinkModel _link;

    public void setModel( LinkModel link ) {
        _link = link;
    }

    public final void draw( Graphics gc, Color color, boolean isSelected ) {
        Graphics2D gc2D = (Graphics2D) gc;
        boolean selected = _link.isSelected();
        // store old stroke.
        final Stroke oldStroke = gc2D.getStroke();
        gc2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        gc2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        gc2D.setStroke(isSelected || selected ? SELECTED_LINE_STROKE : REGULAR_LINE_STROKE);
        gc2D.setPaint(color);

        _link.buildPolyline();
        Shape shape = _link.buildLinkShape();
        gc2D.draw(shape);
        gc2D.setStroke(oldStroke);
    }

    public final Cursor getCursor() {
        Line2D.Float line = _link.getSelectedLinkPart();
        if (line != null) {
            if ((int) line.x1 == (int) line.x2) {
                return Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR);
            } else {
                return Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR);
            }
        }
        return null;
    }
}