/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.graph.slot;

import fr.greyc.pantheon.ariane.presenter.SlotPresenter;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.border.LineBorder;

public abstract class SlotView extends JLabel implements MouseListener, MouseMotionListener {
    private static final int LINE_WIDTH = 1;
    private static final transient LineBorder ACTIVE_BORDER = (LineBorder) BorderFactory.createLineBorder(Color.RED, LINE_WIDTH);
    private static final transient LineBorder NORMAL_BORDER = (LineBorder) BorderFactory.createLineBorder(new Color(0, 0, 0, 0), LINE_WIDTH);
    private SlotPresenter _slotPresenter;

    SlotView() {
        setNormalBorder();
        setFocusable(false);
        addMouseListener(this);
        addMouseMotionListener(this);
    }

    final SlotPresenter getPresenter() {
        return _slotPresenter;
    }

    public final void setPresenter( SlotPresenter slotPresenter ) {
        _slotPresenter = slotPresenter;
    }

    public void setActiveBorder() {
        setBorder(ACTIVE_BORDER);
    }

    public void setNormalBorder() {
        setBorder(NORMAL_BORDER);
    }

    public abstract void setConnectedRendering();

    public abstract void setDisconnectedRendering();

    public abstract Point getAnchorPoint();

    @Override
    public final void print( Graphics gc ) {
        gc.fillRect(0, 0, getWidth(), getHeight());
    }

    public final void drawLink( Graphics gc ) {
        LinkView l = _slotPresenter.drawLink();
        if (l != null) {
            l.draw(gc, getLinkColor(), _slotPresenter.isSelected());
        }
    }

    protected abstract Color getLinkColor();

    public abstract JComponent getValueEditor();

    @Override
    public final void mouseEntered( MouseEvent event ) {
        _slotPresenter.mouseEntered();
    }

    @Override
    public final void mouseExited( MouseEvent event ) {
        _slotPresenter.mouseExited();
    }

    @Override
    public final void mousePressed( MouseEvent event ) {
        setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        // TODO dans presenter.
        _slotPresenter.startConnection();
        _slotPresenter.getGraphView().startConnectionElastic(getLocationOnScreen().x + 4, getLocationOnScreen().y + 4);
    }

    @Override
    public final void mouseDragged( MouseEvent event ) {
        // TODO dans presenter ?
        _slotPresenter.getGraphView().drawConnectionElastic(event.getLocationOnScreen());
    }

    @Override
    public final void mouseReleased( MouseEvent event ) {
        setCursor(null);
        // TODO dans presenter.
        _slotPresenter.dropIntoGraph();
        _slotPresenter.getGraphView().stopConnectionElastic();
        _slotPresenter.getGraphView().requestFocusInWindow();
    }

    @Override
    public final void mouseClicked( MouseEvent event ) {
      /*empty */
    }

    @Override
    public final void mouseMoved( MouseEvent event ) {
      /*empty */
    }
}
