/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.io.File;

import javax.swing.Icon;
import javax.swing.filechooser.FileView;

/**
 * A convenience implementation of the FileView interface
 * that manages name, icon, traversable, and file type information.
 * <p>
 * This implementation will work well with file systems that
 * use "dot" extensions to indicate file type.
 * For example: "picture.pan" as for Pandore image.
 */
public final class FileTypeViewer extends FileView {

    @Override
    public Icon getIcon( File file ) {
        String extension = getExtension(file);
        if (("png".equalsIgnoreCase(extension)
                || "jpg".equalsIgnoreCase(extension)
                || "jpeg".equalsIgnoreCase(extension)
                || "gif".equalsIgnoreCase(extension)
                || "bmp".equalsIgnoreCase(extension)
                || "tif".equalsIgnoreCase(extension)
                || "tiff".equalsIgnoreCase(extension)
                || "ppm".equalsIgnoreCase(extension)
                || "pbm".equalsIgnoreCase(extension)
                || "pgm".equalsIgnoreCase(extension)
                || "pan".equalsIgnoreCase(extension))) {
            return ArianeSettings.getIcon("imageicon.png");
        }
        return null;
    }

    private String getExtension( File file ) {
        String name = file.getName();
        if (name != null) {
            int extensionIndex = name.lastIndexOf('.');
            if (extensionIndex < 0) {
                return null;
            }
            return name.substring(extensionIndex + 1).toLowerCase();
        }
        return null;
    }
}
