/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLEditorKit.InsertHTMLTextAction;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

/**
 * The class <code>HTMLDocumentEditor</code> defines
 * a dialog window to edit HTML documents.
 */
public final class HTMLDocumentEditor extends JDialog {
    public static final int OK_OPTION = JOptionPane.OK_OPTION;
    private static final Dimension DEFAULT_TAB_SIZE = new Dimension(500, 300);

    private HTMLDocument _document;
    private JTextPane _htmlEditorPane;
    private JTextArea _textEditorPane;
    private JTabbedPane _tabbedPane;
    private File _currentFile;
    private int _answer;

    private transient final UndoableEditListener _undoHandler = new UndoHandler();
    private final UndoManager _undoManager = new UndoManager();
    private UndoAction _undoAction;
    private RedoAction _redoAction;

    public HTMLDocumentEditor( JDialog parent ) {
        super(parent, null, true);
        HTMLEditorKit editorKit = new HTMLEditorKit();
        _document = (HTMLDocument) editorKit.createDefaultDocument();
        initDialog();
    }

    public int edit() {
        startExistingDocument();
        _htmlEditorPane.requestFocusInWindow();
        setVisible(true);
        return _answer;
    }

    public File getSelectedFile() {
        return _currentFile;
    }

    public void setSelectedFile( File file ) {
        _currentFile = file;
    }

    private void initDialog() {
        JToolBar toolbar1 = createMenuToolbar();
        JToolBar toolbar2 = createEditToolbar();

        _document.addUndoableEditListener(_undoHandler);
        resetUndoManager();

        _tabbedPane = new JTabbedPane();

        JScrollPane htmlEditorScrollPane = createHTMLTab();
        _tabbedPane.addTab("HTML", null, htmlEditorScrollPane, "HTML View");

        JScrollPane textEditorScrollPane = createTEXTTab();
        _tabbedPane.addTab("TEXT", null, textEditorScrollPane, "Text View");

        _tabbedPane.addChangeListener(e -> updateEditView());

        JPanel toolbarPanel = new JPanel();
        toolbarPanel.setLayout(new BorderLayout());
        toolbarPanel.add(toolbar1, BorderLayout.NORTH);
        toolbarPanel.add(toolbar2, BorderLayout.CENTER);

        getContentPane().add(toolbarPanel, BorderLayout.NORTH);
        getContentPane().add(_tabbedPane, BorderLayout.CENTER);

        pack();
        setLocationRelativeTo(getParent());
        addWindowListener(new FrameListener());
    }

    private JScrollPane createTEXTTab() {
        _textEditorPane = new JTextArea();
        JScrollPane textEditorScrollPane = new JScrollPane(_textEditorPane);
        textEditorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        textEditorScrollPane.setPreferredSize(DEFAULT_TAB_SIZE);
        return textEditorScrollPane;
    }

    private JScrollPane createHTMLTab() {
        _htmlEditorPane = new JTextPane(_document);
        _htmlEditorPane.setContentType("text/html; charset=ISO-8859-1");
        JScrollPane htmlEditorScrollPane = new JScrollPane(_htmlEditorPane);
        htmlEditorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        htmlEditorScrollPane.setPreferredSize(DEFAULT_TAB_SIZE);
        return htmlEditorScrollPane;
    }

    private JToolBar createMenuToolbar() {
        JToolBar toolbar1 = new JToolBar();
        toolbar1.setFloatable(false);
        toolbar1.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        toolbar1.setBorderPainted(true);
        toolbar1.setOpaque(false);

        EditorAction quitA = new EditorAction("Quit", ArianeSettings.getIcon("logout.png")) {
            @Override
            public void actionPerformed( ActionEvent event ) {
                quitConfirmed();
            }
        };
        toolbar1.add(new JToolbarButton(quitA));

        toolbar1.addSeparator();

        EditorAction cutAction = new EditorAction("Cut", ArianeSettings.getIcon("html/cut.png"), new DefaultEditorKit.CutAction());
        EditorAction copyAction = new EditorAction("Copy", ArianeSettings.getIcon("html/copy.png"), new DefaultEditorKit.CopyAction());
        EditorAction pasteAction = new EditorAction("Paste", ArianeSettings.getIcon("html/paste.png"), new DefaultEditorKit.PasteAction());
        toolbar1.add(new JToolbarButton(cutAction));
        toolbar1.add(new JToolbarButton(copyAction));
        toolbar1.add(new JToolbarButton(pasteAction));

        toolbar1.addSeparator();

        _undoAction = new UndoAction(ArianeSettings.getIcon("html/undo.png"));
        _redoAction = new RedoAction(ArianeSettings.getIcon("html/redo.png"));
        toolbar1.add(new JToolbarButton(_undoAction));
        toolbar1.add(new JToolbarButton(_redoAction));
        return toolbar1;
    }

    private JToolBar createEditToolbar() {
        JToolBar toolbar2 = new JToolBar();
        toolbar2.setFloatable(false);
        toolbar2.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        toolbar2.setBorderPainted(true);
        toolbar2.setOpaque(false);

        ListElement[] blockElements = ListElement.createBlockElementList();
        JComboBox<ListElement> blockElementMenu = new JComboBox<>(blockElements);
        blockElementMenu.addActionListener(event -> {
            @SuppressWarnings("unchecked")
            JComboBox<ListElement> comboBox = (JComboBox<ListElement>) event.getSource();
            ListElement element = (ListElement) comboBox.getSelectedItem();
            element.actionPerformed(event);
        });
        toolbar2.add(blockElementMenu);

        toolbar2.addSeparator();

        EditorAction boldAction = new EditorAction("Bold", ArianeSettings.getIcon("html/bold.png"), new StyledEditorKit.BoldAction());
        EditorAction italicAction = new EditorAction("Italic", ArianeSettings.getIcon("html/italic.png"), new StyledEditorKit.ItalicAction());
        EditorAction underlineAction = new EditorAction("Underline", ArianeSettings.getIcon("html/underline.png"), new StyledEditorKit.UnderlineAction());
        EditorAction strikeThroughAction = new EditorAction("StrikeThrough", ArianeSettings.getIcon("html/strikethrough.png"), new StrikeThroughAction());
        EditorAction subscriptAction = new EditorAction("Subscript", ArianeSettings.getIcon("html/subscript.png"), new SubscriptAction());
        EditorAction superscriptAction = new EditorAction("Superscript", ArianeSettings.getIcon("html/superscript.png"), new SuperscriptAction());
        toolbar2.add(new JToolbarToggleButton(boldAction));
        toolbar2.add(new JToolbarToggleButton(italicAction));
        toolbar2.add(new JToolbarToggleButton(underlineAction));
        toolbar2.add(new JToolbarToggleButton(strikeThroughAction));
        toolbar2.add(new JToolbarToggleButton(subscriptAction));
        toolbar2.add(new JToolbarToggleButton(superscriptAction));

        toolbar2.addSeparator();

        EditorAction alignLeft = new EditorAction("Alignleft", ArianeSettings.getIcon("html/align_left.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_LEFT));
        EditorAction alignCenter = new EditorAction("AlignCenter", ArianeSettings.getIcon("html/align_center.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_CENTER));
        EditorAction alignRight = new EditorAction("AlignRight", ArianeSettings.getIcon("html/align_right.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_RIGHT));
        EditorAction alignJustified = new EditorAction("AlignJustifed", ArianeSettings.getIcon("html/align_justified.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_JUSTIFIED));
        toolbar2.add(new JToolbarToggleButton(alignLeft));
        toolbar2.add(new JToolbarToggleButton(alignCenter));
        toolbar2.add(new JToolbarToggleButton(alignRight));
        toolbar2.add(new JToolbarToggleButton(alignJustified));

        String[] fontTypes = {"Default", "SansSerif", "Serif", "Monospaced", "Dialog", "DialogInput"};
        JComboBox<String> fontTypeMenu = new JComboBox<>(fontTypes);
        fontTypeMenu.addActionListener(event -> {
            @SuppressWarnings("unchecked")
            JComboBox<String> cb = (JComboBox<String>) event.getSource();
            String font = (String) cb.getSelectedItem();
            Action action = new StyledEditorKit.FontFamilyAction(font, font);
            action.actionPerformed(event);
        });

        toolbar2.add(fontTypeMenu);
        Integer[] fontSizes = {6, 8, 10, 12, 14, 16, 20, 24, 32, 36, 48, 72};
        JComboBox<Integer> fontSizeMenu = new JComboBox<>(fontSizes);
        fontSizeMenu.addActionListener(event -> {
            @SuppressWarnings("unchecked")
            JComboBox<Integer> cb = (JComboBox<Integer>) event.getSource();
            Integer size = (Integer) cb.getSelectedItem();
            Action action = new StyledEditorKit.FontSizeAction(String.valueOf(size), size);
            action.actionPerformed(event);
        });
        toolbar2.add(fontSizeMenu);
        fontSizeMenu.setSelectedIndex(4);
        Pair[] colors = {new Pair("Black", Color.BLACK),
                new Pair("Red", Color.RED),
                new Pair("Orange", Color.ORANGE),
                new Pair("Yellow", Color.YELLOW),
                new Pair("Green", Color.GREEN),
                new Pair("Blue", Color.BLUE),
                new Pair("Cyan", Color.CYAN),
                new Pair("Magenta", Color.MAGENTA)};
        JComboBox<Pair> colorBox = new JComboBox<>(colors);
        colorBox.addActionListener(event -> {
            @SuppressWarnings("unchecked")
            JComboBox<Pair> cb = (JComboBox<Pair>) event.getSource();
            Pair pair = (Pair) cb.getSelectedItem();
            Action action = new StyledEditorKit.ForegroundAction(pair._name, pair._color);
            action.actionPerformed(event);
        });
        toolbar2.add(colorBox);

        EditorAction breakRule = new EditorAction("InsertBreak", ArianeSettings.getIcon("html/br.png"), new HTMLEditorKit.InsertHTMLTextAction("InsertBreak", "<br>", HTML.Tag.BODY, HTML.Tag.BR));
        toolbar2.add(new JToolbarButton(breakRule));
        EditorAction hRule = new EditorAction("InsertHR", ArianeSettings.getIcon("html/hrule.png"), new HTMLEditorKit.InsertHTMLTextAction("InsertHR", "<hr>", HTML.Tag.BODY, HTML.Tag.HR));
        toolbar2.add(new JToolbarButton(hRule));
        EditorAction unorderedList = new EditorAction("UnorderedList", ArianeSettings.getIcon("html/listunordered.png"), new InsertHTMLTextAction("Itemize", "<ul><li></li></ul>", HTML.Tag.UL, HTML.Tag.LI, HTML.Tag.BODY, HTML.Tag.UL));
        toolbar2.add(new JToolbarButton(unorderedList));
        EditorAction orderedList = new EditorAction("OrderedList", ArianeSettings.getIcon("html/listordered.png"), new InsertHTMLTextAction("Enumerate", "<ol><li></li></ol>", HTML.Tag.UL, HTML.Tag.LI, HTML.Tag.BODY, HTML.Tag.OL));
        toolbar2.add(new JToolbarButton(orderedList));
        toolbar2.add(new JToolbarButton(new LinkAction(ArianeSettings.getIcon("html/link.png"))));
        toolbar2.add(new JToolbarButton(new ImageAction(ArianeSettings.getIcon("html/image.png"))));
        toolbar2.add(new JToolbarButton(new TableAction(ArianeSettings.getIcon("html/table.png"))));

        toolbar2.addSeparator();
        return toolbar2;
    }

    private void updateEditView() {
        if (_tabbedPane.getSelectedIndex() == 0) {
            _htmlEditorPane.setText("");
            insertHTML(_htmlEditorPane, _textEditorPane.getText(), 0);
        } else {
            _textEditorPane.setText(_htmlEditorPane.getText());
        }
    }

    private void insertHTML( JEditorPane editor, String html, int location ) {
        try {
            HTMLEditorKit kit = (HTMLEditorKit) editor.getEditorKit();
            Document doc = editor.getDocument();
            StringReader reader = new StringReader(html);
            kit.read(reader, doc, location);
        } catch (IOException | BadLocationException ex) {
            // Nothing to do
        }
    }

    private void resetUndoManager() {
        _undoManager.discardAllEdits();
        _undoAction.update();
        _redoAction.update();
    }

    void startExistingDocument() {
        HTMLEditorKit editorKit = new HTMLEditorKit();
        _document = (HTMLDocument) editorKit.createDefaultDocument();
        _document.putProperty("IgnoreCharsetDirective", Boolean.TRUE);

        try {
            if (_currentFile == null) {
                throw new IOException();
            }
            FileReader fileReader = new FileReader(_currentFile);
            editorKit.read(fileReader, _document, 0);
            fileReader.close();
            setTitle(_currentFile.getName());
        } catch (IOException | BadLocationException e) {
            _currentFile = null;
            setTitle(ArianeSettings.getString("HTMLViewer.title"));
        }
        Document oldDoc = _htmlEditorPane.getDocument();
        oldDoc.removeUndoableEditListener(_undoHandler);
        _document.addUndoableEditListener(_undoHandler);
        _htmlEditorPane.setDocument(_document);
        resetUndoManager();
    }

    private boolean quitConfirmed() {
        String[] options = {ArianeSettings.getString("button.save.text"),
                ArianeSettings.getString("button.nosave.text"),
                ArianeSettings.getString("button.cancel.text")};
        _answer = MessageBox.showConfirmDialog(this,
                ArianeSettings.getString("HTMLDocumentEditor.quit.message"),
                ArianeSettings.getString("HTMLDocumentEditor.quit.title"),
                options, options[0]);
        if (_answer == 2) {
            return false;
        } else if (_answer == 0) {
            try {
                if (!saveDocument()) {
                    return false;
                }
            } catch (IOException e) {
                JOptionPane.showMessageDialog(this, ArianeSettings.getString("HTMLViewer.saveErrorMessage"), ArianeSettings.getString("HTMLViewer.saveErrorTitle"), JOptionPane.ERROR_MESSAGE);
                return false;
            }
        }
        setVisible(false);
        dispose();
        return true;
    }

    private boolean saveDocument() throws IOException {
        if (_currentFile == null) {
            return saveAs();
        } else {
            save();
            return true;
        }
    }

    private void save() throws IOException {
        try (FileWriter fw = new FileWriter(_currentFile)) {
            fw.write(_htmlEditorPane.getText());
        }
    }

    private boolean saveAs() throws IOException {
        File current = new File(".");
        JFileChooser chooser = new JFileChooser(current);
        chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        chooser.setFileFilter(new HTMLFileFilter());
        int approval = chooser.showSaveDialog(this);
        if (approval == JFileChooser.APPROVE_OPTION) {
            File newFile = chooser.getSelectedFile();
            if (newFile.exists()) {
                String[] options = {ArianeSettings.getString("button.save.text"),
                        ArianeSettings.getString("button.nosave.text"),
                        ArianeSettings.getString("button.cancel.text")};
                String message = newFile.getAbsolutePath() + ArianeSettings.getString("HTMLDocumentEditor.overrideMessage");
                int result = MessageBox.showConfirmDialog(this, message,
                        ArianeSettings.getString("HTMLDocumentEditor.quit.title"),
                        options, options[0]);
                if (result == 2) {
                    return false;
                } else if (result == 0) {
                    _currentFile = newFile;
                    setTitle(_currentFile.getName());
                    save();
                }
            } else {
                _currentFile = new File(newFile.getAbsolutePath());
                setTitle(_currentFile.getName());
                save();
            }
        }
        return true;
    }

    private final class FrameListener extends WindowAdapter {

        @Override
        public void windowClosing( WindowEvent event ) {
            if (quitConfirmed()) {
                ((JDialog) (event.getWindow())).setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
            } else {
                ((JDialog) (event.getWindow())).setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            }
        }
    }

    private static final class SubscriptAction extends StyledEditorKit.StyledTextAction {

        SubscriptAction() {
            super(StyleConstants.Subscript.toString());
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            JEditorPane editor = getEditor(event);
            if (editor != null) {
                StyledEditorKit kit = getStyledEditorKit(editor);
                MutableAttributeSet attr = kit.getInputAttributes();
                boolean subscript = !StyleConstants.isSubscript(attr);
                SimpleAttributeSet sas = new SimpleAttributeSet();
                StyleConstants.setSubscript(sas, subscript);
                setCharacterAttributes(editor, sas, false);
            }
        }
    }

    private static final class SuperscriptAction extends StyledEditorKit.StyledTextAction {

        SuperscriptAction() {
            super(StyleConstants.Superscript.toString());
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            JEditorPane editor = getEditor(event);
            if (editor != null) {
                StyledEditorKit kit = getStyledEditorKit(editor);
                MutableAttributeSet attr = kit.getInputAttributes();
                boolean superscript = !StyleConstants.isSuperscript(attr);
                SimpleAttributeSet sas = new SimpleAttributeSet();
                StyleConstants.setSuperscript(sas, superscript);
                setCharacterAttributes(editor, sas, false);
            }
        }
    }

    private static final class StrikeThroughAction extends StyledEditorKit.StyledTextAction {

        StrikeThroughAction() {
            super(StyleConstants.StrikeThrough.toString());
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            JEditorPane editor = getEditor(event);
            if (editor != null) {
                StyledEditorKit kit = getStyledEditorKit(editor);
                MutableAttributeSet attr = kit.getInputAttributes();
                boolean strikeThrough = !StyleConstants.isStrikeThrough(attr);
                SimpleAttributeSet sas = new SimpleAttributeSet();
                StyleConstants.setStrikeThrough(sas, strikeThrough);
                setCharacterAttributes(editor, sas, false);
            }
        }
    }

    private static final class HTMLFileFilter extends javax.swing.filechooser.FileFilter {

        @Override
        public boolean accept( File file ) {
            return file.isDirectory() || file.getName().toLowerCase().indexOf(".htm") > 0;
        }

        @Override
        public String getDescription() {
            return "html";
        }
    }

    private final class UndoHandler implements UndoableEditListener {

        @Override
        public void undoableEditHappened( UndoableEditEvent event ) {
            _undoManager.addEdit(event.getEdit());
            _undoAction.update();
            _redoAction.update();
        }
    }

    private final class UndoAction extends AbstractAction {

        UndoAction( Icon icon ) {
            super("Undo", icon);
            super.setEnabled(false);
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            try {
                _undoManager.undo();
            } catch (CannotUndoException ex) {
                // Nothing to do
            }
            update();
            _redoAction.update();
        }

        void update() {
            if (_undoManager.canUndo()) {
                setEnabled(true);
                putValue(Action.NAME, _undoManager.getUndoPresentationName());
            } else {
                setEnabled(false);
                putValue(Action.NAME, "Undo");
            }
        }
    }

    private final class RedoAction extends AbstractAction {

        RedoAction( Icon icon ) {
            super("Redo", icon);
            super.setEnabled(false);
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            try {
                _undoManager.redo();
            } catch (CannotRedoException e) {
                //				System.err.println("Unable to redo: " + ex);
                //				ex.printStackTrace();
            }
            update();
            _undoAction.update();
        }

        void update() {
            if (_undoManager.canRedo()) {
                setEnabled(true);
                putValue(Action.NAME, _undoManager.getRedoPresentationName());
            } else {
                setEnabled(false);
                putValue(Action.NAME, "Redo");
            }
        }
    }

    private static final class ListElement extends HTMLEditorKit.InsertHTMLTextAction {

        private ListElement( String name, String html, HTML.Tag parentTag, HTML.Tag addTag ) {
            super(name, html, parentTag, addTag, null, null);
        }

        private static ListElement[] createBlockElementList() {
            return new ListElement[]{
                    new ListElement("Body Text", "<div> </div>", HTML.Tag.BODY, HTML.Tag.DIV),
                    new ListElement("Paragraph", "<p> </p>", HTML.Tag.BODY, HTML.Tag.P),
                    new ListElement("Preformatted", "<pre> </pre>", HTML.Tag.BODY, HTML.Tag.PRE),
                    new ListElement("Heading 1", "<h1> </h1>", HTML.Tag.BODY, HTML.Tag.H1),
                    new ListElement("Heading 2", "<h2> </h2>", HTML.Tag.BODY, HTML.Tag.H2),
                    new ListElement("Heading 3", "<h3> </h3>", HTML.Tag.BODY, HTML.Tag.H3),
                    new ListElement("Heading 4", "<h4> </h4>", HTML.Tag.BODY, HTML.Tag.H4),
                    new ListElement("Heading 5", "<h5> </h5>", HTML.Tag.BODY, HTML.Tag.H5),
                    new ListElement("Heading 6", "<h6> </h6>", HTML.Tag.BODY, HTML.Tag.H6)};
        }

        @Override
        public String toString() {
            return (String) getValue(Action.NAME);
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            super.actionPerformed(event);
            JEditorPane pane = getEditor(event);
            pane.requestFocusInWindow();
        }
    }

    private class EditorAction extends AbstractAction {

        private final Action _action;

        EditorAction( String name, Icon icon ) {
            super(name, icon);
            _action = null;
        }

        EditorAction( String name, Icon icon, Action action ) {
            super(name, icon);
            _action = action;
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            _action.actionPerformed(event);
            _htmlEditorPane.requestFocusInWindow();
        }
    }

    private final class TableAction extends EditorAction {

        TableAction( Icon icon ) {
            super(ArianeSettings.getString("HTMLViewer.table"), icon);
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            JPanel panel = new JPanel(new GridLayout(2, 2));
            JSpinner rowEditor = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
            JSpinner colEditor = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
            panel.add(new JLabel(ArianeSettings.getString("HTMLViewer.column")));
            panel.add(colEditor);
            panel.add(new JLabel(ArianeSettings.getString("HTMLViewer.row")));
            panel.add(rowEditor);
            if (JOptionPane.showConfirmDialog(HTMLDocumentEditor.this,
                    panel, "Table",
                    JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION) {
                HTMLEditorKit kit = (HTMLEditorKit) _htmlEditorPane.getEditorKit();
                HTMLDocument doc = (HTMLDocument) _htmlEditorPane.getDocument();
                int numRows = (Integer) rowEditor.getValue();
                int numCols = (Integer) colEditor.getValue();

                StringBuilder html = new StringBuilder("<table border=1>");
                for (int row = 0; row < numRows; row++) {
                    html.append("<tr>\n");
                    for (int col = 0; col < numCols; col++) {
                        html.append("\t<td>\n</td>\n");
                    }
                    html.append("</tr>\n");
                }
                html.append("</table>");
                try {
                    kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), html.toString(), 0, 0, HTML.Tag.TABLE);
                } catch (IOException | BadLocationException e) {
                    // Nothing to do
                }
            }
        }
    }

    private final class LinkAction extends EditorAction {

        LinkAction( Icon icon ) {
            super(ArianeSettings.getString("HTMLViewer.link"), icon);
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            HTMLEditorKit kit = (HTMLEditorKit) _htmlEditorPane.getEditorKit();
            HTMLDocument doc = (HTMLDocument) _htmlEditorPane.getDocument();
            String value = JOptionPane.showInputDialog(HTMLDocumentEditor.this, "URL:");
            try {
                kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), "<a href=\"" + value + "\">", 0, 0, HTML.Tag.A);
            } catch (IOException | BadLocationException e) {
                // Nothing to do
            }
            _htmlEditorPane.requestFocusInWindow();
        }
    }

    private final class ImageAction extends EditorAction {

        ImageAction( Icon icon ) {
            super(ArianeSettings.getString("HTMLViewer.image"), icon);
        }

        @Override
        public void actionPerformed( ActionEvent event ) {
            HTMLEditorKit kit = (HTMLEditorKit) _htmlEditorPane.getEditorKit();
            HTMLDocument doc = (HTMLDocument) _htmlEditorPane.getDocument();
            String value = JOptionPane.showInputDialog(HTMLDocumentEditor.this, "Image file:");
            try {
                kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), "<img src=\"" + value + "\">", 0, 0, HTML.Tag.IMG);
            } catch (IOException | BadLocationException e) {
                // Nothing to do
            }
        }
    }

    private static final class JToolbarToggleButton extends JToggleButton {

        JToolbarToggleButton( Action action ) {
            super(action);
            super.setText(null);
            super.setToolTipText((String) action.getValue(Action.NAME));
        }
    }

    private static final class JToolbarButton extends JButton {

        JToolbarButton( Action action ) {
            super(action);
            super.setText(null);
            super.setToolTipText((String) action.getValue(Action.NAME));
        }
    }

    private static final class Pair {

        private final String _name;
        private final Color _color;

        Pair( String name, Color color ) {
            _name = name;
            _color = color;
        }

        @Override
        public String toString() {
            return _name;
        }
    }
}
