/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import fr.greyc.pantheon.ariane.ArianeSettings;

import java.awt.Component;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

/**
 * Interface that overrides JOptionPane.showConfirmDialog
 * with custom yes / no icons.
 */
public final class MessageBox {

    private MessageBox() {
    }

    public static int showConfirmDialog( Component parentComponent, Object message, String title, String[] options, Object initialValue ) {
        final JOptionPane pane = new JOptionPane(message, JOptionPane.QUESTION_MESSAGE);
        final JDialog dialog = pane.createDialog(parentComponent, title);

        final JButton button1 = new JButton(options[0], ArianeSettings.getIcon("gtk-yes.png"));
        button1.addActionListener(e -> pane.setValue(button1));
        Object[] buttons = new Object[options.length];
        buttons[0] = button1;

        final JButton button2 = new JButton(options[1], ArianeSettings.getIcon("gtk-no.png"));
        button2.addActionListener(e -> pane.setValue(button2));
        buttons[1] = button2;

        if (options.length > 2) {
            final JButton button3 = new JButton(options[2], ArianeSettings.getIcon("gtk-cancel.png"));
            button3.addActionListener(e -> pane.setValue(button3));
            buttons[2] = button3;
        }
        pane.setOptions(buttons);

        pane.setInitialValue(initialValue);
        pane.selectInitialValue();
        dialog.pack();
        dialog.setVisible(true);
        dialog.dispose();

        Object selectedValue = pane.getValue();
        if (selectedValue == null) {
            return 2;
        }

        for (int counter = 0; counter < buttons.length; counter++) {
            if (buttons[counter].equals(selectedValue)) {
                return counter;
            }
        }
        return 2;
    }
}
