/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;

import javax.swing.border.AbstractBorder;

/**
 * Rounded borders are used with RoundedPanels.
 */
public final class RoundedBorder extends AbstractBorder {
    private static final int LOWERED = 1;
    private static final Color TRANSPARENT = new Color(0, 0, 0, 0);
    private static final int SHADOW_GAP = 1;
    private final int _etchType;
    private Color _highlight;

    public RoundedBorder( int etchType ) {
        _etchType = etchType;
    }

    public RoundedBorder( int etchType, final Color highlight ) {
        _etchType = etchType;
        _highlight = highlight;
    }

    @Override
    public void paintBorder( final Component component, final Graphics graphics, int x, int y, int width, int height ) {
        Graphics2D graphics2d = (Graphics2D) graphics;
        graphics.translate(x, y);
        graphics.setColor(_etchType == LOWERED ? getHighlightColor(component) : TRANSPARENT);
        graphics2d.setStroke(new BasicStroke(2));
        graphics2d.drawRoundRect(0, 0, width - SHADOW_GAP, height - SHADOW_GAP, RoundedPanel.CORNER_ARC.width, RoundedPanel.CORNER_ARC.height);
    }

    @Override
    public Insets getBorderInsets( final Component component ) {
        return new Insets(2, 2, 3, 3);
    }

    @Override
    public Insets getBorderInsets( final Component component, final Insets insets ) {
        insets.top = 2;
        insets.left = 2;
        insets.bottom = 3;
        insets.right = 3;
        return insets;
    }

    @Override
    public boolean isBorderOpaque() {
        return true;
    }

    private Color getHighlightColor( final Component component ) {
        if (_highlight == null) {
            return component.getBackground().brighter();
        } else {
            return _highlight;
        }
    }
}
