/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.RenderingHints;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.RoundRectangle2D;

import javax.swing.JPanel;

/**
 * <code>RoundedPanel</code> is a JPanel with rounder corners.
 */
public class RoundedPanel extends JPanel {
    public static final Dimension CORNER_ARC = new Dimension(12, 12);

    protected RoundedPanel() {
        super.setOpaque(false);
    }

    @Override
    protected final void paintComponent( final Graphics graphics ) {
        final int width = getWidth();
        final int height = getHeight();
        final Graphics2D graphics2D = (Graphics2D) graphics;

        graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        final Area shape = new Area(new RoundRectangle2D.Float(0, 0, width, height, CORNER_ARC.width, CORNER_ARC.height));
        final Point2D startTop = new Point2D.Float(0, 0);
        final Point2D stopBottom = new Point2D.Float(0, height);
        float offset = 12.0f;
        final float[] fractionsTopToBottom = {0.0f, offset / height, 1.0f};
        final Color[] colorsTopToBottom = {
                Color.WHITE,
                getBackground(),
                getBackground().darker()
        };

        final LinearGradientPaint leftToRightGradient = new LinearGradientPaint(startTop, stopBottom, fractionsTopToBottom, colorsTopToBottom);
        graphics2D.setPaint(leftToRightGradient);
        graphics2D.fill(shape);
    }
}
