/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane;

import static org.junit.jupiter.api.Assertions.*;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

public final class CommandLineParserTest {
    private static final PrintStream ORIGINAL = System.err;

    @BeforeAll
    public static void setUpBeforeClass() {
        System.setErr(new PrintStream(new OutputStream() {
            @Override
            public void write( int data ) throws IOException {
            }
        }));
    }

    @AfterAll
    public static void tearDownAfterClass() {
        System.setErr(ORIGINAL);
    }

    @Test
    void testParsingWithNoArgumentArgument() throws Exception {
        String[] args = {};

        Map<String, Object> options = new HashMap<>();
        options.put("nosplash", false);
        CommandLineParser.parse(args, options);
        boolean nosplash = (Boolean) options.get("nosplash");
        assertFalse(nosplash);
    }

    @Test
    void testParsingSplashArgument() throws Exception {
        String[] args = {"-nosplash"};

        Map<String, Object> options = new HashMap<>();
        options.put("nosplash", false);
        CommandLineParser.parse(args, options);
        boolean noSplash = (Boolean) options.get("nosplash");
        assertTrue(noSplash);
    }

    @Test
    void testParsingConfigDirnameArgument() throws Exception {
        String[] args = {"-config", "/home/tmp/.greyc"};

        Map<String, Object> options = new HashMap<>();
        CommandLineParser.parse(args, options);
        String path = (String) options.get("config");
        assertEquals("/home/tmp/.greyc", path);
    }

    @Test
    void testParsingLibArgument() throws Exception {
        String[] args = {"-lib", "/home/tmp/pandore"};

        Map<String, Object> options = new HashMap<>();
        CommandLineParser.parse(args, options);
        String path = (String) options.get("lib");
        assertEquals("/home/tmp/pandore", path);
    }

    @Test
    void testParsingWrongArgument() throws Exception {
        String[] args = {"-updateFormula", "abs"};

        Map<String, Object> options = new HashMap<>();
        options.put("nosplash", false);
        CommandLineParser.parse(args, options);
        boolean doSplash = (Boolean) options.get("nosplash");
        assertFalse(doSplash);
    }

    @Test
    void testParsingWorkspaceFiles() throws Exception {
        String[] args = {"./updateFormula.updateFormula.aws", "/tmp/titi.aws"};

        Map<String, Object> options = new HashMap<>();
        options.put("nosplash", false);
        List<String> workspaceFiles = CommandLineParser.parse(args, options);
        assertArrayEquals(args, workspaceFiles.toArray());
    }

    @Test
    void testParsingGenerateOptionWithRightArguments() throws Exception {
        String[] args = {"-generate", "/home/tmp/script.py", "graph1test.aws"};

        Map<String, Object> options = new HashMap<>();
        CommandLineParser.parse(args, options);
        String path = (String) options.get("generate");
        assertEquals("/home/tmp/script.py", path);
    }

    @Test
    void testParsingGenerateOptionWithoutGraphFiles() throws Exception {
        String[] args = {"-generate", "/home/tmp/script.py"};

        Map<String, Object> options = new HashMap<>();
        CommandLineParser.parse(args, options);
        String path = (String) options.get("generate");
        assertNull(path);
    }

    @Test
    void testParsingGenerateWithSeveralGraphfiles() throws Exception {
        String[] args = {"-generate", "/home/tmp/script.py", "graph1test.aws", "graph2test.aws"};

        Map<String, Object> options = new HashMap<>();
        CommandLineParser.parse(args, options);
        String path = (String) options.get("generate");
        assertNull(path);
    }

    @Test
    void testParsingGenerateWithoutScriptfile() throws Exception {
        String[] args = {"-generate"};

        Map<String, Object> options = new HashMap<>();
        CommandLineParser.parse(args, options);
        String path = (String) options.get("generate");
        assertNull(path);
    }
}
