/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model;

import fr.greyc.pantheon.ariane.UserPreferences;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.io.IOException;

import static org.junit.jupiter.api.Assertions.*;

public final class UserPreferencesTest {

    @BeforeAll
    public static void oneTimeSetUp() {
        UserPreferences.initializePreferencesWithDefaultValues();
    }

    @Test
    void changeTooltipPreferencesTest() {
        UserPreferences.setToolTipState(true);
        boolean value = UserPreferences.isToolTipEnabled();

        assertTrue(value);
        UserPreferences.setToolTipState(false);
        value = UserPreferences.isToolTipEnabled();
        assertFalse(value);
    }

    @Test
    void changeHistorySizePreferencesTest() {
        int defaultHistorySize = UserPreferences.getHistorySize();
        defaultHistorySize += 1;
        UserPreferences.setHistorySize(defaultHistorySize);
        int historySize = UserPreferences.getHistorySize();
        assertEquals(defaultHistorySize, historySize);
    }

    @Test
    void changeWebBrowserPreferencesTestWhenIsNotNull() {
        String path = "/home/thor/bin/webbrowser";
        UserPreferences.setWebBrowserPath(path);
        String actualPath = UserPreferences.getWebBrowserPath();
        assertEquals(path, actualPath);
    }

    @Test
    void changeTextEditorPreferencesTest() {
        String path = "/home/thor/bin/texteditor";
        UserPreferences.setTextEditorPath(path);
        String actualPath = UserPreferences.getTextEditorPath();
        assertEquals(path, actualPath);
    }

    @Test
    void changeCommandVisuPreferencesTest() {
        String path = "/home/thor/bin/visu.exe";
        UserPreferences.setCommandVisu(path);
        String result = UserPreferences.getCommandVisu();
        assertEquals(result, path);
    }

    @Test
    void changeCommandVisuPreferencesWithSpaceThenReturnEmpty() {
        String path = "    ";
        UserPreferences.setCommandVisu(path);
        String result = UserPreferences.getCommandVisu();
        assertTrue(result.isEmpty());
    }

    @Test
    void changeCommandStatusPreferencesTest() {
        String path = "/home/thor/bin/status.exe";
        UserPreferences.setCommandStatus(path);
        String result = UserPreferences.getCommandStatus();
        assertEquals(result, path);
    }

    @Test
    void changeFontSizePreferencesTest() {
        final int expected = 12;
        UserPreferences.setDefaultFontSize(expected);
        int result = UserPreferences.getDefaultFontSize();
        assertEquals(expected, result);
    }

    @Test
    void changeFileDirectoryPreferencesTest() {
        final String expected = "/home/thor/work";
        UserPreferences.setPreferredFileDirectory(expected);
        String result = UserPreferences.getPreferredFileDirectory();
        assertEquals(expected, result);
    }

    @Test
    void changeTemporaryFileExtensionPreferencesTest() {
        final String expected = ".tmp";
        UserPreferences.setTemporaryFileExtension(expected);
        String result = UserPreferences.getTemporaryFileExtension();
        assertEquals(expected, result);
    }

    @Test
    void changeBinaryPreferencesTest() {
        final String expected = "/home/thor/bin/";
        UserPreferences.setBinaryDirectory(expected);
        String result = UserPreferences.getPreferredBinaryDirectory();
        assertEquals(expected, result);
    }

    @Test
    public void whenChangeTemporaryDirectoryPreferencesWithNotExistingPathThenThrowException() throws IOException {
        final String expected = "/dummy/thor/tmp/";
        assertThrows(IOException.class, () -> UserPreferences.setTemporaryDirectoryBase(expected));
    }

    @Test
    public void whenChangeTemporaryDirectoryPreferencesWithExistingNotWritablePathThenThrowException() throws IOException {
        final String expected = "/usr/bin/";
        assertThrows(IOException.class, () -> UserPreferences.setTemporaryDirectoryBase(expected));
    }

    @Test
    void whenChangeTemporaryDirectoryPreferencesWithExistingWritablePathThenModifyPath() throws IOException {
        final String expected = "/tmp/";
        UserPreferences.setTemporaryDirectoryBase(expected);
        String result = UserPreferences.getTemporaryDirectoryBase();
        assertEquals(expected, result);
    }
}
