/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.element.GraphItemModel;
import fr.greyc.pantheon.ariane.model.graph.element.NoteItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItemFactory;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.model.xml.XMLFileInputStream;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.presenter.RootGraphPresenter;
import org.junit.jupiter.api.Test;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

final class GraphModelTest {
    private static final String TEST_GRAPHTEST_AWS = "src/test/resources/graph1test.aws";

    @Test
    void testFileExist() throws IOException, SAXException, ParserConfigurationException {
        Ariane.initForTest();
        File testFile = new File(TEST_GRAPHTEST_AWS);
        assertTrue(testFile.exists());
    }

    @Test
    void testGivenLoadingTreefromXMLWhenTestingFirstItemThenReturnFirstItem() throws IOException, SAXException, ParserConfigurationException {
        Ariane.initForTest();
        File testFile = new File(TEST_GRAPHTEST_AWS);
        XMLFileInputStream convert = new XMLFileInputStream(testFile);
        convert.parse();
        convert.close();

        List<XMLElement> tree = convert.getElements().get(0).getChildren();
        assertEquals("Graph", tree.get(0).getName());
    }

    @Test
    void testGivenLoadingTreefromXMLThenTestFirstItem() throws IOException, SAXException, ParserConfigurationException {
        File testFile = new File(TEST_GRAPHTEST_AWS);
        XMLFileInputStream convert = new XMLFileInputStream(testFile);
        convert.parse();
        convert.close();
        GraphModel graphModel = new RootGraph();
        GraphPresenter graph = new RootGraphPresenter("Test graph", graphModel);
        graphModel.createFromXMLTree(convert.getElements(), graph);
        assertEquals(1, graphModel.getListOfGraphItems().get(0).getId());
    }

    @Test
    void testWhenAddingItemThenItIsInFirstPosition() throws IOException, SAXException, ParserConfigurationException {
        GraphModel graphModel = new RootGraph();
        GraphPresenter graph = new RootGraphPresenter("Test graph", graphModel);

        NoteItem item = new NoteItem(TreeItemFactory.createItem(TreeItem.ItemType.NOTE), graph);
        graphModel.add(item);

        List<GraphItemModel> items = graphModel.getListOfGraphItems();
        assertEquals(item, items.get(0));
    }

    @Test
    void testWhenAddingItemAndSearchItThenReturnTrue() throws IOException, SAXException, ParserConfigurationException {
        GraphModel graphModel = new RootGraph();
        GraphPresenter graph = new RootGraphPresenter("Test graph", graphModel);
        NoteItem item = new NoteItem(TreeItemFactory.createItem(TreeItem.ItemType.NOTE), graph);
        graphModel.add(item);
        assertEquals(item, graphModel.getGraphItem(item.getInternalId()));
    }

    @Test
    void testWhenAddingAndRemovingItemThenReturnEmpty() throws IOException, SAXException, ParserConfigurationException {
        GraphModel graphModel = new RootGraph();
        GraphPresenter graph = new RootGraphPresenter("Test graph", graphModel);
        NoteItem item = new NoteItem(TreeItemFactory.createItem(TreeItem.ItemType.NOTE), graph);
        graphModel.add(item);
        graphModel.remove(item);

        assertTrue(graphModel.getListOfGraphItems().isEmpty());
    }

    @Test
    void testWhenAddingAndDisposeThenReturnEmpty() throws IOException, SAXException, ParserConfigurationException {
        GraphModel graphModel = new RootGraph();
        GraphPresenter graph = new RootGraphPresenter("Test graph", graphModel);
        NoteItem item = new NoteItem(TreeItemFactory.createItem(TreeItem.ItemType.NOTE), graph);
        graphModel.add(item);
        graphModel.dispose();
        assertTrue(graphModel.getListOfGraphItems().isEmpty());
    }

    @Test
    void testGraphNameWhenLoadingFileThenReturnFileWitoutExtension() throws IOException, SAXException, ParserConfigurationException {
        File file = new File("updateFormula.aws");
        GraphPresenter graph = new RootGraphPresenter(file);
        assertEquals("updateFormula", graph.getName());
    }
}
