/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

public final class DataFileTest {

    @Test
    public void getRelativePathTestWhenFilenameIsNullOrEmptyThenReturnFilename() {
        String filename = null;
        String directory = null;
        String fileDirectory = null;
        assertNull(DataFile.getRelativePath(filename, directory, fileDirectory));

        filename = "";
        directory = null;
        fileDirectory = null;
        assertEquals("", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenDirectoryIsNullOrEmptyThenReturnFilename() {
        String filename = "updateFormula.gif";
        String directory = null;
        String fileDirectory = null;
        assertEquals("updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));

        filename = "updateFormula.gif";
        directory = null;
        fileDirectory = null;
        assertEquals("updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenFilenameIsRelativeThenReturnFilename() {
        String filename = "updateFormula.gif";
        String directory = "/disc/thor/pantheon";
        String fileDirectory = null;
        assertEquals("updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenFilenameIsAbsoluteAndOutOfPreferenceAndOfDirectoryThenReturnAbsolutePath() {
        String filename = "/data/images/updateFormula.gif";
        String directory = "/tmp";
        String fileDirectory = "/disc/thor";
        assertEquals("/data/images/updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenFilenameIsAbsoluteAndInDirectoryThenReturnRelativePath() {
        String filename = "/disc/thor/pantheon/updateFormula.gif";
        String directory = "/disc/thor/pantheon";
        String fileDirectory = null;
        assertEquals("./updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenFilenameIsAbsoluteAndInPreferenceButOutOfDirectoryThenReturnAbsolutePath() {
        String filename = "/disc/thor/autre/updateFormula.gif";
        String directory = "/tmp";
        String fileDirectory = "/disc/thor/";
        assertEquals("$FILE_DIRECTORY/autre/updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenFilenameIsAbsoluteAndInPreferenceAndInDirectoryThenReturnAbsolutePath() {
        String filename = "/disc/thor/autre/updateFormula.gif";
        String directory = "/disc/thor/";
        String fileDirectory = "/disc/thor/";
        assertEquals("$FILE_DIRECTORY/autre/updateFormula.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }

    @Test
    public void getRelativePathTestWhenFilenameIsAbsoluteAndInDirectoryThenReturnRelativePathX() {
        String filename = "/disc/thor/titi.gif";
        String directory = "/disc/thor/pantheon";
        String fileDirectory = "/disc/thor";
        assertEquals("$FILE_DIRECTORY/titi.gif", DataFile.getRelativePath(filename, directory, fileDirectory));
    }
}
