/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.assertEquals;

import java.awt.Point;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import fr.greyc.pantheon.ariane.model.tree.TreeItemFactory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.presenter.RootGraphPresenter;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

public final class MacroOperatorItemTest {
    private GraphPresenter _graph;

    @BeforeAll
    public static void setUpClass() {
        Ariane.initForTest();
    }

    @BeforeEach
    public void setUp() {
        _graph = new RootGraphPresenter("graph");
    }

    @Test
    public void testEvalItemCreation() {
        MacroOperatorItem item = new MacroOperatorItem(TreeItemFactory.createItem(TreeItem.ItemType.MACRO_OPERATOR), _graph, 1, 1, 1);
        assertTrue(1 == item.getInputDataSlots().size()
                && 1 == item.getInputParameterSlots().size()
                && 1 == item.getOutputDataSlots().size()
                && null != item.getInputMaskSlot()
                && 1 != item.getOutputResultSlots().size());
    }

    @Test
    public void testItemCreationFromXML() throws IOException {
        List<XMLElement> subgraphXml = new ArrayList<>();
        subgraphXml.add(new XMLElement("ItemList"));

        List<XMLElement> xml = new ArrayList<>();
        xml.add(new XMLElement("NickName", "essai"));
        xml.add(new XMLElement("Name", "macro-operator"));
        xml.add(new XMLElement("Identifier", "30"));
        xml.add(new XMLElement("X-coordinate", "36"));
        xml.add(new XMLElement("Y-coordinate", "15"));
        xml.add(new XMLElement("Parameters", "1"));
        xml.add(new XMLElement("InputFiles", "1"));
        xml.add(new XMLElement("OutputFiles", "1"));
        XMLElement subgraph = new XMLElement("SubGraph");
        subgraph.addChildren(subgraphXml);
        xml.add(subgraph);

        MacroOperatorItem item = new MacroOperatorItem(_graph, xml);
        assertTrue("essai".equals(item.getNickName())
                && "macro-operator".equals(item.getName())
                && 1 == item.getInputDataSlots().size()
                && 1 == item.getInputParameterSlots().size()
                && 1 == item.getOutputDataSlots().size()
                && null != item.getInputMaskSlot()
                && null != item.getOutputResultSlot()
                && 30 == item.getInternalId()
                && item.getLocation().equals(new Point(36, 15)));
    }

    @Test
    public void whenChangeNickNameThenGraphNameIschanged() {
        final String oldName = "before";
        MacroOperatorItem item = new MacroOperatorItem(TreeItemFactory.createItem(TreeItem.ItemType.MACRO_OPERATOR), new RootGraphPresenter(oldName), 1, 1, 1);
        item.createSubGraph(oldName, 0, 0, 0, 0, 0);
        final String newName = "after";
        item.setNickName(newName, false);
        assertEquals(newName, item.getSubGraph().getName());
    }

//   @Test
//   public void whenIncreaseNbOfParametersThenPrototypeIschanged() {
//      fail("toto");
//   }

//   @Test
//   public void whenIncreaseNbOfInputsThenPrototypeIschanged() {
//      fail("toto");
//   }
//   @Test
//   public void whenIncreaseNbOfOutputsThenPrototypeIschanged() {
//      fail("toto");
//   }
//
//   @Test
//   public void whenDecreaseNbOfParametersAndParametersAreNotSetsThenPrototypeIsChanged() {
//      fail("toto");
//   }
//   @Test
//
//   public void whenDecreaseNbOfParametersAndParametersAreSetsThenThrowException() {
//      fail("toto");
//   }
//
//   @Test
//   public void whenDecreaseNbOfInputsAndInputsAreNotSetsThenPrototypeIsChanged() {
//      fail("toto");
//   }
//   @Test
//
//   public void whenDecreaseNbOfInputsAndInputsAreSetsThenThrowException() {
//      fail("toto");
//   }
//
//   @Test
//   public void whenDecreaseNbOfOutputsAndOutputsAreNotSetsThenPrototypeIsChanged() {
//      fail("toto");
//   }
//   @Test
//
//   public void whenDecreaseNbOfOutputsAndOutputsAreSetsThenThrowException() {
//      fail("toto");
//   }
}
