/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.File;

import fr.greyc.pantheon.ariane.model.tree.TreeItemFactory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.presenter.RootGraphPresenter;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

public final class MergeControlStructureItemTest {
    private GraphPresenter _graph;

    @BeforeAll
    public static void setUpClass() {
        Ariane.initForTest();
    }

    @BeforeEach
    public void setUp() {
        _graph = new RootGraphPresenter("graph");
    }

    @Test
    public void testEvalItemCreation() {
        MergeControlStructureItem item = new MergeControlStructureItem(TreeItemFactory.createItem(TreeItem.ItemType.MERGE_CONTROL), _graph);
        assertTrue(2 == item.getInputDataSlots().size()
                && 0 == item.getInputParameterSlots().size()
                && 1 == item.getOutputDataSlots().size()
                && null == item.getInputMaskSlot()
                && null != item.getOutputResultSlot());
    }

    @Test
    public void testExecutionWith2ndInputValidated() throws ArianeRuntimeException {
        LoaderItem item1 = new LoaderItem(TreeItemFactory.createItem(TreeItem.ItemType.LOADER), _graph);
        String directory = new File(".").getAbsoluteFile().getParent();
        File file1 = new File(directory + "/src/test/resources/graph1test.aws");
        String filename2 = file1.getAbsolutePath();
        item1.setBinary(filename2);

        MergeControlStructureItem item = new MergeControlStructureItem(TreeItemFactory.createItem(TreeItem.ItemType.MERGE_CONTROL), _graph);
        item.getInputDataSlots().get(1).setConnectedSlot(item1.getOutputDataSlots().get(0));
        _graph.startTestExecution();
        item.primitiveExecute();
        assertEquals(filename2, item.getOutputDataSlots().get(0).getValue());
    }
}
