/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.awt.Point;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import fr.greyc.pantheon.ariane.model.tree.TreeItemFactory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.exception.ArianeMissingInputException;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.presenter.RootGraphPresenter;

public final class ViewerItemTest {
    private static GraphPresenter _graph;

    @BeforeAll
    public static void setUpClass() {
        Ariane.initForTest();
    }

    @BeforeEach
    public void setUp() {
        _graph = new RootGraphPresenter("graph");
    }

    @Test
    public void testItemCreation() {
        ViewerItem item = new ViewerItem(TreeItemFactory.createItem(TreeItem.ItemType.VIEWER), _graph);
        assertTrue(1 == item.getInputDataSlots().size()
                && 0 == item.getInputParameterSlots().size()
                && 1 == item.getOutputDataSlots().size()
                && null == item.getInputMaskSlot()
                && null == item.getOutputResultSlot());
    }

    @Test
    public void testItemCreationFromXML() throws IOException {
        List<XMLElement> xml = new ArrayList<>();

        xml.add(new XMLElement("NickName", "updateFormula"));
        xml.add(new XMLElement("Name", "viewer"));
        xml.add(new XMLElement("Identifier", "30"));
        xml.add(new XMLElement("X-coordinate", "36"));
        xml.add(new XMLElement("Y-coordinate", "15"));
        xml.add(new XMLElement("InputFiles", "1"));
        xml.add(new XMLElement("OutputFiles", "1"));
        String expected = "gedit";
        xml.add(new XMLElement("Binary", expected));

        ViewerItem item = new ViewerItem(_graph, xml);

        assertTrue("updateFormula".equals(item.getNickName())
                && "viewer".equals(item.getName())
                && 30 == item.getInternalId()
                && item.getLocation().equals(new Point(36, 15))
                && item.getInputDataSlots().size() == 1
                && item.getInputParameterSlots().isEmpty()
                && item.getOutputDataSlots().size() == 1
                && expected.equals(item.getBinary()));
    }

    @Test
    public void testItemExecution() throws ArianeRuntimeException, ArianeMissingInputException {
        LoaderItem item1 = new LoaderItem(TreeItemFactory.createItem(TreeItem.ItemType.LOADER), _graph);

        String directory = new File(".").getAbsoluteFile().getParent();
        File file = new File(directory + "/src/test/resources/graph1test.aws");
        String filename = file.getAbsolutePath();
        item1.setBinary(filename);
        item1.primitiveExecute();

        ViewerItem item2 = new ViewerItem(TreeItemFactory.createItem(TreeItem.ItemType.VIEWER), _graph);
        item2.getInputDataSlots().get(0).setConnectedSlot(item1.getOutputDataSlots().get(0));

        item2.executeCoreAction(item2.getInputDataSlots().get(0));

        assertEquals(filename, item2.getOutputDataSlots().get(0).getValue());
    }
}
