/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItemFactory;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.GraphPresenter;
import fr.greyc.pantheon.ariane.presenter.RootGraphPresenter;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.awt.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertTrue;

public final class WhileControlStructureItemTest {
    private GraphPresenter _graph;

    @BeforeAll
    public static void setUpClass() {
        Ariane.initForTest();
    }

    @BeforeEach
    public void setUp() {
        _graph = new RootGraphPresenter("graph");
    }

    @Test
    void testEvalItemCreation() {
        TreeItem treeItem = TreeItemFactory.createItem(TreeItem.ItemType.WHILE_CONTROL);
        CompositeItem item = new WhileControlStructureItem(treeItem, _graph, 1, 1, 1);
        assertTrue(1 == item.getInputDataSlots().size()
                && 1 + 3 == item.getInputParameterSlots().size()
                && 1 == item.getOutputDataSlots().size()
                && null != item.getInputMaskSlot()
                && 1 != item.getOutputResultSlots().size());
    }

    @Test
    void testItemCreationFromXML() throws IOException {
        List<XMLElement> subgraphXml = new ArrayList<>();
        subgraphXml.add(new XMLElement("ItemList"));

        List<XMLElement> xml = new ArrayList<>();
        xml.add(new XMLElement("NickName", "essai"));
        xml.add(new XMLElement("Name", "while"));
        xml.add(new XMLElement("Identifier", "30"));
        xml.add(new XMLElement("X-coordinate", "36"));
        xml.add(new XMLElement("Y-coordinate", "15"));
        xml.add(new XMLElement("Parameters", "2"));
        xml.add(new XMLElement("InputFiles", "1"));
        xml.add(new XMLElement("OutputFiles", "1"));
        String formula = "$0==$1";
        xml.add(new XMLElement("Formula", formula));

        XMLElement subgraph = new XMLElement("SubGraph");
        subgraph.addChildren(subgraphXml);
        xml.add(subgraph);

        CompositeItem item = new WhileControlStructureItem(_graph, xml);
        assertTrue("essai".equals(item.getNickName())
                && "while".equals(item.getName())
                && 1 == item.getInputDataSlots().size()
                && 2 == item.getInputParameterSlots().size()
                && 1 == item.getOutputDataSlots().size()
                && null != item.getInputMaskSlot()
                && null != item.getOutputResultSlot()
                && formula.equals(item.getFormula())
                && 30 == item.getInternalId()
                && item.getLocation().equals(new Point(36, 15)));
    }
}
