/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import org.junit.jupiter.api.Test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

final class ArgumentLineParserTest {

    @Test
    void testRegularArgumentLineParser() {
        String command = "pbinarization";
        String customFormat = "[$p] [-m $m] [$i] [$o]";
        List<String> maskLine = new ArrayList<>();

        List<String> parameterLine = Arrays.asList("19.49363435889024", "1e30");
        List<String> inputLine = Arrays.asList("./images/tangram.pan");
        List<String> outputLine = Arrays.asList("/tmp/ariane211/data_2_4_0.tmp");
        List<String> commandLine = ArgumentLineParser.parse(command, customFormat, maskLine, parameterLine, inputLine, outputLine);
        List<String> oracle = new ArrayList<>();
        oracle.add(command);
        oracle.addAll(parameterLine);
        oracle.addAll(inputLine);
        oracle.addAll(outputLine);
        assertEquals(oracle, commandLine);
    }

    @Test
    void testIrregularArgumentLineParser() {
        String command = "pbinarization";
        String customFormat = "[-m $m] -o [$o] -i [$i] -p [$p] ";
        List<String> maskLine = new ArrayList<>();

        List<String> parameterLine = Arrays.asList("19.49363435889024", "1e30");
        List<String> inputLine = Arrays.asList("./images/tangram.pan");
        List<String> outputLine = Arrays.asList("/tmp/ariane211/data_2_4_0.tmp");
        List<String> commandLine = ArgumentLineParser.parse(command, customFormat, maskLine, parameterLine, inputLine, outputLine);
        List<String> oracle = new ArrayList<>();
        oracle.add(command);
        oracle.add("-o");
        oracle.addAll(outputLine);
        oracle.add("-i");
        oracle.addAll(inputLine);
        oracle.add("-p");
        oracle.addAll(parameterLine);
        assertEquals(oracle, commandLine);
    }
}
