/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import java.lang.reflect.MalformedParameterizedTypeException;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

final class ItemSignatureParserTest {

    @Test
    void testParse() throws Exception {
        String[][] values = ItemSignatureParser.parse("$0:initial value;$1:final value;$2:increment", 3);
        assertTrue("initial value".equals(values[0][0])
                && "final value".equals(values[1][0])
                && "increment".equals(values[2][0]));
    }

    @Test
    void testParseValidExpression1() throws Exception {
        String[][] values = ItemSignatureParser.parse("$0:initial value;$1:;", 2);
        assertEquals("initial value", values[0][0]);
    }

    @Test
    void testParseValidExpression2() throws Exception {
        String[][] values = ItemSignatureParser.parse("$0:initial value;$1:", 2);
        assertNull(values[1][0]);
    }

    @Test
    void testEmpty() throws Exception {
        String[][] values = ItemSignatureParser.parse("", 0);
        assertNull(values);
    }

    @Test
    void testMalformedExpression1() throws Exception {
        assertThrows(MalformedParameterizedTypeException.class, () -> ItemSignatureParser.parse("#0", 1));
    }

    @Test
    void testMalformedExpression2() throws Exception {
        assertThrows(MalformedParameterizedTypeException.class, () -> ItemSignatureParser.parse("$0;", 1));
    }

    @Test
    void testMalformedExpression3() throws Exception {
        assertThrows(MalformedParameterizedTypeException.class, () -> ItemSignatureParser.parse("$0:initial value;$1", 1));
    }
}
