/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.exception.ArianeArithmeticException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.util.Locale;

import static org.junit.jupiter.api.Assertions.*;

public final class LogicalExpressionTest {

    @BeforeAll
    public static void setUpBeforeClass() {
        ArianeSettings.setCurrentLocale(new Locale("fr", "FR"));
    }

    @Test
    void checkValidTrueExpression() throws ArianeArithmeticException {
        final String formula1 = "$0 >= 0 & $1 >= 0";
        LogicalExpression expression1 = new LogicalExpression().compile(formula1);
        boolean result1 = expression1.evaluate(0, 1);
        assertTrue(result1);
    }

    @Test
    void checkEvaluationSequence() {
        LogicalExpression expression1 = new LogicalExpression();
        assertThrows(NullPointerException.class, () -> expression1.evaluate(0, 1));
    }

    @Test
    void checkUnvalidExpression() throws ArianeArithmeticException {
        final String formula1 = "$0 >=";
        assertThrows(ArianeArithmeticException.class, () -> new LogicalExpression().compile(formula1));
    }

    @Test
    void checkValidFalseExpression() throws ArianeArithmeticException {
        final String formula1 = "$0 >= 10 & $1 >= 0";
        LogicalExpression expression1 = new LogicalExpression().compile(formula1);
        boolean result1 = expression1.evaluate(0, 1);
        assertFalse(result1);
    }

    @Test
    void checkComplexeExpression() throws ArianeArithmeticException {
        final String formula1 = "$0 >= 10 & $1 >= 0 | $0 < $1 & $0 != $1 | $0 == $1";
        LogicalExpression expression1 = new LogicalExpression().compile(formula1);
        boolean result1 = expression1.evaluate(0, 1);
        assertTrue(result1);
    }

    @Test
    void checkOperandOrder() throws ArianeArithmeticException {
        final String formula1 = "5 > 4";
        LogicalExpression expression1 = new LogicalExpression().compile(formula1);
        boolean result1 = expression1.evaluate(0, 1);
        assertTrue(result1);
    }

    @Test
    void testFloatValues() throws ArianeArithmeticException {
        final String formula1 = "5.5e1>-1.e30";
        LogicalExpression expression1 = new LogicalExpression().compile(formula1);
        boolean result1 = expression1.evaluate(0, 1);
        assertTrue(result1);
    }
}
