/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.model.graph.element.etc;

import fr.greyc.pantheon.ariane.ArianeSettings;
import fr.greyc.pantheon.ariane.model.exception.ArianeArithmeticException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.util.Locale;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

public final class StringExpressionTest {

    @BeforeAll
    public static void setUpBeforeClass() {
        ArianeSettings.setCurrentLocale(new Locale("fr", "FR"));
    }

    @Test
    void checkValidExpression1() throws ArianeArithmeticException {
        final String formula1 = "\"toto\"";
        StringExpression expression1 = new StringExpression(formula1, 0);
        String result1 = expression1.evaluate(null);
        assertEquals("toto", result1);
    }

    void checkValidExpression2() throws ArianeArithmeticException {
        final String formula2 = "quote($0)";
        StringExpression expression2 = new StringExpression(formula2, 1);
        final String[] arg2 = {"a"};
        String result2 = expression2.evaluate(arg2);
        assertEquals("\'a\'", result2);
    }

    @Test
    void checkValidExpression3() throws ArianeArithmeticException {
        final String formula3 = "$0 + \"toto\" + $1";
        StringExpression expression3 = new StringExpression(formula3, 2);
        final String[] arg3 = {"a", "b"};
        String result3 = expression3.evaluate(arg3);
        assertEquals("atotob", result3);
    }

    @Test
    void checkInValidExpressions1() {
        final String formula1 = "toto";
        assertThrows(ArianeArithmeticException.class, () -> new StringExpression(formula1, 0));
    }

    @Test
    void checkInValidExpressions2() throws ArianeArithmeticException {
        final String formula1 = "quite($0)";
        assertThrows(ArianeArithmeticException.class, () -> new StringExpression(formula1, 0));
    }
}
