/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.presenter;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.view.explorer.ItemTreeModel;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class PresenterTest {
    private static Presenter _presenter;

    @BeforeAll
    public static void setUpBeforeClass() {
        Ariane.initForTest();
        ItemTreeModel itemTree = new ItemTreeModel();
        _presenter = new Presenter(itemTree);
    }

    @Test
    final void testGeneratesScriptCodeFromGraph() {
        String scriptFilename = "/tmp/updateFormula.py";
        List<String> graphFilenames = Arrays.asList("src/test/resources/graph1test.aws");
        File file = new File(scriptFilename);
        assertTrue(_presenter.generatesScriptCodeFromCurrentGraph(scriptFilename, graphFilenames));
        assertTrue(file.exists());
        file.delete();
    }

    @Test
    final void testGeneratesScriptCodeFromGraphWithBadSuffix() {
        String scriptFilename = "/tmp/updateFormula.txt";
        List<String> graphFilenames = Arrays.asList("src/test/resources/graph1test.aws");
        File file = new File(scriptFilename);
        assertFalse(_presenter.generatesScriptCodeFromCurrentGraph(scriptFilename, graphFilenames));
        file.delete();
    }

    @Test
    final void testGeneratesScriptCodeFromGraphWithBadScriptFile() {
        String scriptFilename = "/bin/updateFormula.py";
        List<String> graphFilenames = Arrays.asList("src/test/resources/graph1test.aws");
        File file = new File(scriptFilename);
        assertTrue(_presenter.generatesScriptCodeFromCurrentGraph(scriptFilename, graphFilenames));
        assertFalse(file.exists());
        file.delete();
    }

    @Test
    final void testGeneratesScriptCodeFromGraphWithBadGraph() {
        String scriptFilename = "/bin/updateFormula.py";
        List<String> graphFilenames = Arrays.asList("bidon/graph1test.aws");
        File file = new File(scriptFilename);
        assertFalse(_presenter.generatesScriptCodeFromCurrentGraph(scriptFilename, graphFilenames));
        file.delete();
    }
}
