/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import java.io.PrintStream;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.verify;

final class ConsoleTest {
    private Console _console;

    @Mock
    private PrintStream _printOutputStream;
    @Mock
    private Console.ReadWriteThread _outputReadWriteThread;

    @BeforeEach
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        _console = new Console(_printOutputStream, _outputReadWriteThread);
    }

    @Test
    void whenSearchWithEmptyConsoleThenNegativeOffset() {
        Console.PopupMenuListener popupMenu = _console.new PopupMenuListener();
        int offset = popupMenu.searchAndHighlight("essai");
        assertTrue(offset < 0);
    }

    @Test
    void whenSearchExistingWordThenOffsetNonZero() throws BadLocationException {
        Console.PopupMenuListener popupMenu = _console.new PopupMenuListener();
        String initString = "Ceci est un essai de texte avec le nom essai, et le premier commence à 12";
        StyledDocument doc = _console.getStyledDocument();
        doc.insertString(0, initString, null);
        int offset = popupMenu.searchAndHighlight("essai");
        assertEquals(12, offset);
    }

    @Test
    void whenClearThenDocumentEmpty() throws BadLocationException {
        String initString = "Ceci est un essai de texte avec le nom essai et le premier commence à 12";
        StyledDocument doc = _console.getStyledDocument();
        doc.insertString(0, initString, null);
        _console.clear();
        assertEquals(0, _console.getDocument().getLength());
    }

    @Test
    void should_return_success_when_comparing_two_identical_annotations() {
        String initString = "Ceci est un essai de texte avec le nom essai et le premier commence à 12";
        _console.displayMessage(initString, "blue");
        verify(_printOutputStream).print("<font color='blue'>Ceci est un essai de texte avec");
    }
}
