/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.ariane.view.utils;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import javax.swing.*;
import java.io.IOException;

import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

class ImageButtonTest {
    private static final String BACKGROUND_TEXT = "Background text";
    private ImageButton _button;

    @BeforeEach
    public void setUp() {
        _button = new ImageButton(null, BACKGROUND_TEXT);
    }

    @Test
    void whenLoadRegularImageExpectNonEmptyIcon() throws IOException {
        ImageIcon a = _button.getThumbnail("src/test/resources/image.png");
        assertTrue(a.getIconWidth() > 0);
    }

    @Test
    void whenLoadInvalidFilePathExpectNull() throws IOException {
        ImageIcon a = _button.getThumbnail("test/xxxx.png");
        assertNull(a);
    }

    @Test
    void whenLoadRegularTextFileExpectNonEmptyIcon() {
        ImageIcon a = _button.getThumbnail("src/test/resources/graph1test.aws");
        assertTrue(a.getIconWidth() > 0);
    }

    @Test
    void whenLoadInvalidExistingFileExpectNull() {
        ImageIcon a = _button.getThumbnail("/dev/null");
        assertNull(a);
    }
}
