/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext;

import fr.greyc.pantheon.ariane.ArianeSettings;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.util.Locale;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

public final class HTMLDocumentEditorTest {
    private HTMLDocumentEditor _editor;

    @BeforeAll
    public static void setUpBeforeClass() {
        ArianeSettings.setCurrentLocale(new Locale("fr", "FR"));
    }

    @BeforeEach
    public void setUp() {
        _editor = new HTMLDocumentEditor(null);
    }

    @AfterEach
    public void tearDown() {
        _editor.dispose();
    }

    @Test
    void testEmptyFile() {
        _editor.startExistingDocument();
        assertNull(_editor.getSelectedFile());
    }

    @Test
    void testRegularFile() {
        File file = new File("src/test/resources/documentation.html");
        _editor.setSelectedFile(file);
        _editor.startExistingDocument();
        assertEquals(file, _editor.getSelectedFile());
    }

    @Test
    void testUnvalidFile() {
        _editor = new HTMLDocumentEditor(null);
        File file = new File("/bin/documentation.html");
        _editor.setSelectedFile(file);
        _editor.startExistingDocument();
        assertNull(_editor.getSelectedFile());
    }
}
