/*
 * Copyright or (C) or Copr. Normandie Univ, ENSICAEN, GREYC, Caen, France
 *
 * https://clouard.users.greyc.fr
 *
 * This software is governed by the CeCILL-B  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-B
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.greyc.pantheon.swingext.multivieweditor;

import org.junit.jupiter.api.Test;

import java.awt.Component;

import javax.swing.JPanel;

import static org.junit.jupiter.api.Assertions.*;

public final class GMultiviewEditorTest {

    @Test
    public void whenCreateComponentExpectDivisionSizeZero() {
        GMultiviewEditor editor = new MultiviewEditor();
        assertEquals(0, editor.getDividerSize());
    }

    @Test
    public void whenAddingNewComponentExpectComponentInserted() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component = new JPanel();
        editor.addComponent(component, "updateFormula");
        assertTrue(editor.getAllComponents().contains(component));
    }

    @Test
    public void whenAddingTwiceSameComponentExpectOneTab() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        Component component2 = new JPanel();
        editor.addComponent(component1, "c1");
        editor.addComponent(component2, "c2");
        editor.addComponent(component1, "c1");
        assertEquals(2, editor.getAllComponents().size());
    }

    @Test
    public void whenAddingAndRemovingComponentExpectZeroTab() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        editor.addComponent(component1, "c1");
        editor.removeComponent(component1);
        assertEquals(0, editor.getAllComponents().size());
    }

    @Test
    public void whenRemovingComponentExpectPreviousComponentSelected() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        editor.addComponent(component1, "c1");
        Component component2 = new JPanel();
        editor.addComponent(component2, "c2");
        Component component = editor.removeComponent(component2);
        assertEquals(component, component1);
    }

    @Test
    public void whenSplitRemoveTabExpectDivisionLocationZero() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        editor.addComponent(component1, "c1");
        editor.splitVertically();
        editor.remove(component1);
        assertTrue(0 >= editor.getDividerLocation());
    }

    @Test
    public void whenSplitExpectNewDividerLocation() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        editor.addComponent(component1, "c1");
        editor.splitVertically();
        assertTrue(0 > editor.getDividerLocation());
    }

    @Test
    public void whenUnsplitExpectDivisionLocationZero() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        editor.addComponent(component1, "c1");
        editor.splitVertically();
        editor.unsplit();
        assertEquals(0, editor.getDividerLocation());
    }

    @Test
    public void whenSplitMoveTabInLeftExpectTabInLeftComponent() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        editor.addComponent(component1, "c1");
        DnDTabbedPane leftTabbedPane = (DnDTabbedPane) editor.getLeftComponent();
        DnDTabbedPane rightTabbedPane = (DnDTabbedPane) editor.getRightComponent();
        editor.splitVertically();
        rightTabbedPane.exportTab(0, leftTabbedPane, 0);
        assertTrue(1 == leftTabbedPane.getTabCount()
                && 0 == rightTabbedPane.getTabCount());
    }

    @Test
    public void whenSplitMoveTabInLeftUnsplitExpectTabInRightComponent() {
        GMultiviewEditor editor = new MultiviewEditor();
        Component component1 = new JPanel();
        Component component2 = new JPanel();
        editor.addComponent(component1, "c1");
        editor.addComponent(component2, "c2");
        editor.splitVertically();
        DnDTabbedPane leftTabbedPane = (DnDTabbedPane) editor.getLeftComponent();
        DnDTabbedPane rightTabbedPane = (DnDTabbedPane) editor.getRightComponent();
        rightTabbedPane.exportTab(0, leftTabbedPane, 0);
        editor.unsplit();
        assertTrue(0 == leftTabbedPane.getTabCount()
                && 2 == rightTabbedPane.getTabCount());
    }

    public final class MultiviewEditor extends GMultiviewEditor {

        @Override
        public void close( Component component ) {
            // Empty
        }

        @Override
        public Component getSelectedComponent() {
            return null;
        }

        @Override
        public void setSelectedComponent( Component component ) {
            // Empty
        }
    }
}
